#{{{ imports
import os
import shutil
import glob
import math
import re
import itertools
from collections import OrderedDict
from os.path import join as pjoin 
# own modules
from initialize_classes import out, fold
from matched_module import *
#}}}
#{{{ def: atoi(text)
def atoi(text):
    return int(text) if text.isdigit() else text
#}}}
#{{{ def: natural_keys(text)
def natural_keys(text):
    '''
    alist.sort(key=natural_keys) sorts in human order
    http://nedbatchelder.com/blog/200712/human_sorting.html
    (See Toothy's implementation in the comments)
    '''
    return [ atoi(c) for c in re.split('(\d+)', text) ]
#}}}
#{{{ def: isfloat(value)
def isfloat(value):
    try:
        float(value)
        return True
    except ValueError:
        return False
#}}}
#{{{ def: accumulate(list_in)
def accumulate(list_in):
    list_out = []
    total = 0
    for value in list_in:
        total += value
        list_out.append(total)
    return list_out
#}}}
#{{{ class: result()
class result(): # class to deal with the results, adding files, copying them to the results folder etc.
#{{{ def: save_input_with_results(self)
    def save_input_with_result(self):
        # this routine saves the input (at beginning of run) in the result folder
        dest_folder = pjoin(fold.result_folder_path,"input_of_run")
        try:
            os.makedirs(dest_folder)
        except:
            pass
        file_to_copy = pjoin(fold.input_folder_path,"parameter.dat")
        shutil.copy(file_to_copy,dest_folder)
        file_to_copy = pjoin(fold.input_folder_path,"distribution.dat")
        shutil.copy(file_to_copy,dest_folder)
        file_to_copy = pjoin(fold.input_folder_path,"model.dat")
        shutil.copy(file_to_copy,dest_folder)
#}}}    
#{{{ def: save_previous(self)
    def save_previous(self):
        # this routine checks wether there are results in the result folder of this run and moves them to saved_result_$i
        result_sub_folder = glob.glob(pjoin(fold.result_folder_path,"*-run"))
        if os.path.isdir(pjoin(fold.result_folder_path,"summary")):
            result_sub_folder.append(pjoin(fold.result_folder_path,"summary"))
#        if os.path.isdir(pjoin(fold.result_folder_path,"input_of_run")):
#            result_sub_folder.append(pjoin(fold.result_folder_path,"input_of_run"))
        if os.path.isdir(pjoin(fold.result_folder_path,"gnuplot")):
            result_sub_folder.append(pjoin(fold.result_folder_path,"gnuplot"))
        save_folder = "saved_result_1" # start with result 1
        save_folders = glob.glob(pjoin(fold.result_folder_path,"saved_result_*"))
        max_number = 0
        for folder in save_folders:
            try:
                if "_" in folder.rsplit('_', 1)[1]:
                    next
                current_number = int(folder.rsplit('_', 1)[1])
            except:
                current_number = 0
                pass
            if current_number > max_number:
                max_number = current_number
        out_folder = pjoin(fold.result_folder_path,"saved_result_%s" % (max_number+1))
        if result_sub_folder:
            out.print_info("Saving previous result...")
            if os.path.exists(out_folder):
                out.print_error("Folder %s that should be used for saving the previous result already exists." % out_folder)
            else:
                os.makedirs(out_folder)
            for folder in result_sub_folder:
                shutil.move(folder,out_folder)
            try:
                shutil.copytree(pjoin(fold.result_folder_path,"input_of_run"),pjoin(out_folder,"input_of_run"))
            except:
                pass
        try:
            shutil.move(pjoin(fold.result_folder_path,"CITATIONS.bib"),pjoin(out_folder,"CITATIONS.bib"))
        except:
            pass
#}}}    
#{{{ def: copy_previous(self)
    def copy_previous(self):
        # this routine checks wether there are results in the result folder of this run and copies them to saved_result_$i
        result_sub_folder = glob.glob(pjoin(fold.result_folder_path,"*-run"))
        if os.path.isdir(pjoin(fold.result_folder_path,"summary")):
            result_sub_folder.append(pjoin(fold.result_folder_path,"summary"))
#        if os.path.isdir(pjoin(fold.result_folder_path,"input_of_run")):
#            result_sub_folder.append(pjoin(fold.result_folder_path,"input_of_run"))
        if os.path.isdir(pjoin(fold.result_folder_path,"gnuplot")):
            result_sub_folder.append(pjoin(fold.result_folder_path,"gnuplot"))
        save_folder = "saved_result_1" # start with result 1
        save_folders = glob.glob(pjoin(fold.result_folder_path,"saved_result_*"))
        max_number = 0
        for folder in save_folders:
            try:
                if "_" in folder.rsplit('_', 1)[1]:
                    next
                current_number = int(folder.rsplit('_', 1)[1])
            except:
                current_number = 0
                pass
            if current_number > max_number:
                max_number = current_number
        out_folder = pjoin(fold.result_folder_path,"saved_result_%s" % (max_number+1))
        if result_sub_folder:
            out.print_info("Copying previous result...")
            if os.path.exists(out_folder):
                out.print_error("Folder %s that should be used for saving the previous result already exists." % out_folder)
            else:
                os.makedirs(out_folder)
            for folder in result_sub_folder:
                shutil.copytree(folder,pjoin(out_folder,os.path.basename(folder)))
            try:
                shutil.copytree(pjoin(fold.result_folder_path,"input_of_run"),pjoin(out_folder,"input_of_run"))
            except:
                pass
        try:
            shutil.copytree(pjoin(fold.result_folder_path,"CITATIONS.bib"),pjoin(out_folder,"CITATIONS.bib"))
        except:
            pass
#}}}    
#{{{ def: add_result_files(self,file_list,out_file)
    def add_result_files(self,file_list,out_file):
        # this routine adds a number of files (that MUST have the same structure), keeping the first column fixed,
        # adding the second column linear, and adding the third in quadature, and writes it to out_file
        firsttime = True
        for file_path in file_list:
            new_data = self.readin_file(file_path) # reads space-separated matrix from file
            if firsttime: # initialize data array
                data = [[0 for col in range(len(new_data[0]))] for row in range(len(new_data))]
                firsttime = False
            for row in range(len(new_data)):
                for column in range(len(new_data[row])):
                    if column == 0:
                        data[row][column] = float(new_data[row][column])
                    elif column == 1:
                        data[row][column] += float(new_data[row][column])
                    elif column == 2:
                        data[row][column] += float(new_data[row][column])**2
                    else:
                        out.print_error("The rate file added in add_rate_files contains more than 3 columns")
        with open(out_file,'w') as f:
            for row in range(len(data)):
                line = ""
                for column in range(len(data[row])):
                    length = 15
                    if column == 2:
                        line = line+" "*(length-len(str("%.8g" % math.sqrt(data[row][column]))))+"%.8g" % math.sqrt(data[row][column])
                    else:
                        line = line+" "*(length-len("%.8g" % data[row][column]))+"%.8g" % data[row][column]
                f.write(line+"\n")
            f.close()
#}}}    
#{{{ def: get_central_min_max_from_distribution_files(self,file_list,out_file)
    def get_central_min_max_from_distribution_files(self,file_list,out_file):
        # this routine combines a number of files (that MUST have the same structure), keeping the first column fixed,
        # having the central (first element of file list) in the second and third column, and computing the minumum 
        # and maximum for the columns 4-7
        firsttime = True
        for file_path in file_list:
            new_data = self.readin_file(file_path) # reads space-separated matrix from file
            if firsttime: # initialize data array
                data = [[0 for col in range(7)] for row in range(len(new_data))] # 7, because 7 columns are needed
            for row in range(len(new_data)):
                if firsttime:
                    try:
                        data[row][0] = float(new_data[row][0]) # first column stays always the same (take it from first file)
                        data[row][1] = float(new_data[row][1]) # second column is the central (first file)
                        data[row][2] = float(new_data[row][2]) # third column is the cerror of the entral (first file)
                        data[row][3] = float(new_data[row][1]) # this is the starting point to get the minimu
                        data[row][4] = float(new_data[row][2]) # the error of the minimum
                        data[row][5] = float(new_data[row][1]) # this is the starting point to get the maximum
                        data[row][6] = float(new_data[row][2]) # the error of the maximimum
                    except:
                        data[row][0] = float('NaN')
                        data[row][1] = float('NaN')
                        data[row][2] = float('NaN')
                        data[row][3] = float('NaN')
                        data[row][4] = float('NaN')
                        data[row][5] = float('NaN')
                        data[row][6] = float('NaN')
                else:
                    try:
                        if(float(new_data[row][1]) < data[row][3]): # if the cross section in the new file is smaller
                            data[row][3] = float(new_data[row][1]) # set the minimum to this value
                            data[row][4] = float(new_data[row][2]) # the error of the minimum
                    except:
                        data[row][3] = float('NaN') # set the minimum to this value
                        data[row][4] = float('NaN') # the error of the minimum                        
                    try:
                        if(float(new_data[row][1]) > data[row][5]): # if the cross section in the new file is bigger
                            data[row][5] = float(new_data[row][1]) # set the maximum to this value
                            data[row][6] = float(new_data[row][2]) # the error of the minimum
                    except:
                        data[row][5] = float('NaN') # set the minimum to this value
                        data[row][6] = float('NaN') # the error of the minimum                        
            firsttime = False
        with open(out_file,'w') as f:
            for row in range(len(data)):
                line = ""
                for column in range(7): # 7, because 7 columns were created
                    length = 15
                    line = line+" "*(length-len("%.8g" % data[row][column]))+"%.8g" % data[row][column]
                f.write(line+"\n")
            f.close()
#}}}    
#{{{ def: readin_file(self,file_path)
    def readin_file(self,file_path):
        # this reads a file and writes it into a table
        with open(file_path,'r') as f:
            table = [row.strip().split() for row in f if not row.split()[0].startswith("#") and not row.split()[0].startswith("%")]
        return table
#}}}    
#{{{ def: readin_file_float(self,file_path)
    def readin_file_float(self,file_path):
        # this reads a file and writes it converted to float into a table
        with open(file_path,'r') as f:
            table = [[float(i) for i in row.strip().split() if isfloat(i)] for row in f if not row.split()[0].startswith("#") and not row.split()[0].startswith("%")]
        return table
#}}}    
#{{{ def: get_column_from_table(self,table,column
    def get_column_from_table(self,table,column_number):
        # this returns column "column_number"
        column_array_out = []
        for row in range(len(table)):
            for column in range(len(table[row])):
                if column == column_number:
                    column_array_out.append(table[row][column])
        return column_array_out
#}}}    
#{{{ def: seven_point_variation(self,path)
    def seven_point_variation(self,path):
        # this check wether the current folder contains a 7-point variation
        n_scales = 0
        try:
            file_path = glob.glob(pjoin(path,"*","*CV*.dat"))[0]
        except:
            out.print_error("There is no *.dat file in path %s that could tell you what kind of variation was used in routine seven_point_variation." % path)
        with open(file_path) as in_file:
            for line in in_file:
                if line.strip():
                    n_scales += 1
        if n_scales == 7:
            return True
        else:
            return False
#}}}    
#{{{ def: nine_point_variation(self,path)
    def nine_point_variation(self,path):
        # this check wether the current folder contains a 9-point variation
        n_scales = 0
        try:
            file_path = glob.glob(pjoin(path,"*","*CV*.dat"))[0]
        except:
            out.print_error("There is no *.dat file in path %s that could tell you what kind of variation was used in routine nine_point_variation.")
        with open(file_path) as in_file:
            for line in in_file:
                if line.strip():
                    n_scales += 1
        if n_scales == 9:
            return True
        else:
            return False
#}}}    
#{{{ def: seven_point_variation_distribution(self,path)
    def seven_point_variation_distribution(self,path):
        # this check wether the current folder contains a 7-point variation for distributions
        scale_files = glob.glob(pjoin(path,"scale.*"))
        if len(scale_files) == 0:
            out.print_error("There are no scale.* files in path %s that could tell you what kind of variation was used in routine seven_point_variation_distribution" % path)
        n_scales = len(scale_files)
        if n_scales == 7:
            return True
        else:
            return False
#}}}    
#{{{ def: nine_point_variation_distribution(self,path)
    def nine_point_variation_distribution(self,path):
        # this check wether the current folder contains a 9-point variation for distributions
        scale_files = glob.glob(pjoin(path,"scale.*"))
        if len(scale_files) == 0:
            out.print_error("There are no scale.* files in path %s that could tell you what kind of variation was used in routine nine_point_variation_distribution" % path)
        n_scales = len(scale_files)
        if n_scales == 9:
            return True
        else:
            return False
#}}}    
#{{{ def: seven_point_mapping(self,factor)
    def seven_point_mapping(self,factor):
        # this returns an array that contains up and down variation for muR and muF with a factor
        mapping = []
        mapping.append([1./factor,1./factor]) # first row
        mapping.append([1./factor,1.])        # second row
        mapping.append([factor,1.])           # ...
        mapping.append([1.,1.])               # ..
        mapping.append([1.,factor])           # .
        mapping.append([1.,1/factor])       
        mapping.append([factor,factor])       
        return mapping
#}}}    
#{{{ def: nine_point_mapping(self,factor)
    def nine_point_mapping(self,factor):
        # this returns an array that contains up and down variation for muR and muF with a factor
        mapping = []
        mapping.append([1./factor,factor])    # frist row
        mapping.append([1./factor,1./factor]) # second row
        mapping.append([1./factor,1.])        # ...
        mapping.append([factor,1.])           # ..
        mapping.append([1.,1.])               # .
        mapping.append([1.,factor])       
        mapping.append([1.,1./factor])       
        mapping.append([factor,factor])       
        mapping.append([factor,1./factor])       
        return mapping
#}}}    
#{{{ def: convert_to_independent_scales(self,file_path)
    def convert_to_independent_scales(self,file_path):
        # this routine changes the first column of a rate file from muR=muF to independent muR and muF column
        firsttime = True
        new_data = self.readin_file(file_path) # reads space-separated matrix from file
        factor = 1/float(new_data[0][0]) # this is the factor with respect to the central scale
        if firsttime: # initialize data array
            data = [[0 for col in range(len(new_data[0])+1)] for row in range(len(new_data))] # add one column to split up muR and muF
            firsttime = False
        for row in range(len(new_data)):
            for column in range(len(new_data[row])):
                if column == 0:
                    if len(new_data) == 7: # 7-point variation
                        data[row][column]   = self.seven_point_mapping(factor)[row][0] # muR
                        data[row][column+1] = self.seven_point_mapping(factor)[row][1] # muF
                    elif len(new_data) == 9: # 9-point variation1
                        data[row][column]   = self.nine_point_mapping(factor)[row][0] # muR
                        data[row][column+1] = self.nine_point_mapping(factor)[row][1] # muF
                elif column == 1:
                    data[row][column+1] = float(new_data[row][column])
                elif column == 2:
                    data[row][column+1] = float(new_data[row][column])
                else:
                    out.print_error("The rate file converted in convert_to_independent_scales contains more than 3 columns")
        with open(file_path+".replace",'w') as f:
            for row in range(len(data)):
                line = ""
                for column in range(len(data[row])):
                    if column in [0,1]:
                        length = 10
                        line = line+" "*(length-len("%.4g" % data[row][column]))+"%.4g" % data[row][column]
                    else:
                        length = 15
                        line = line+" "*(length-len("%.8g" % data[row][column]))+"%.8g" % data[row][column]
                f.write(line+"\n")
        os.remove(file_path)
        shutil.move(file_path+".replace",file_path)
#}}}    
#{{{ def: extrapolate_qT_dependence(self,file_path)
    def extrapolate_qT_dependence(self,file_path,max_qt_value = 99):
        # this routine reads in a qT_cut-dependence file, extrapolates the central result (either 7 or 9 point variation) to rcut=0 and returns an uncertainty (difference between extrapolated result and at lowest qT_cut value)
        relative_uncertainty = 0 

        firsttime = True
        new_data = self.readin_file(file_path) # reads space-separated matrix from file

        # prepare date to for 7- or 9-point variation
        data_set = [[float(row[0]),float(row[(len(new_data[0])-1)/2])] for row in new_data if abs(float(row[0])) <= abs(max_qt_value)]
        # calculate coefficients of linear fit
        linear_fit = data_point_fitter(data_set)
        b0, b1 = linear_fit.coefficients()
        # take as measure of uncertainty: scope of linear regression times missing interval from lowest rcut value (usually rcut=0.15) to rcut=0 
        # normalized to cross section at lowerst rcut value; in percent; extra factor of 2 to be conservative
        relative_uncertainty = abs(b1/data_set[0][1]*data_set[0][0] * 100) * 2
 
        return relative_uncertainty
#}}}
#{{{ def: readin_and_perform_matching_for_radish(self)
    def readin_and_perform_matching_for_radish(self,parameter_list):
        # determine if resummation scale variations have been used
        parameter_file = open(pjoin(fold.run_folder_path,"file_parameter.dat"))
        resvariations = 0
        onlyscentralscale = 0
        joint = False
        if (parameter_list.get("damping")) == "1":
            do_damping = True
        else:
            do_damping = False
        damping_scale = float(parameter_list.get("damping_scale", "1.e99"))
        while True:
            line = parameter_file.readline()
            if line.startswith("user_switch scale_variation_res"):
                resvariations = int(str.split(line)[3])
            if line.startswith("variation_factor_CV"):
                if ( int(str.split(line)[2]) == 1 ):
                    onlyscentralscale = 1
            if line.startswith("user_switch RadISH_observable"):
                obs = int(str.split(line)[3])
                observable = "pt"
                if (obs == 3):
                    observable = "phi"
                elif (obs == 4 or obs == 5):
                    joint = True
            if not line:
                break

        if parameter_list.get("matching_scheme","1") == "0":
            scheme = "add"
        elif parameter_list.get("matching_scheme","1") == "1":
            scheme = "mult"
        else:
            out.print_error("Unknown matching scheme.")
        # readin resummed and fixed order results and perform the matching
        for radish_folder in glob.glob(pjoin(fold.run_folder_path,"result","result.*_dist","*","*.RadISH")):
            matched_folder = radish_folder.replace("/NLL.RadISH","/NLO+NLL.matched").replace("/NNLL.RadISH","/NNLO+NNLL.matched").replace("/N3LL.RadISH","/NNLO+N3LL.matched")
            # create folder with matched results
            try:
                os.makedirs(matched_folder)
                os.makedirs(pjoin(matched_folder,"MATRIX+RadISH"))
                os.makedirs(pjoin(matched_folder,"MATRIX+RadISH","cumulant"))
            except:
                pass
            # determine matching order (NLO or NNLO) taken from MATRIX
#            print radish_folder
            if radish_folder.endswith("/NLL.RadISH"):
                matching_order = "NLO.QCD"
                matched_ending = "..NLO+NLL.dat"
                resummed_ending = "..NLL.dat"
                ro = 1
                fo = 1
            elif radish_folder.endswith("/NNLL.RadISH"):
                matching_order = "NNLO.QCD"
                matched_ending = "..NNLO+NNLL.dat"
                resummed_ending = "..NNLL.dat"
                ro = 2
                fo = 2
            elif radish_folder.endswith("/N3LL.RadISH"):
                matching_order = "NNLO.QCD"
                matched_ending = "..NNLO+N3LL.dat"
                resummed_ending = "..N3LL.dat"
                ro = 3
                fo = 2
            else:
                out.print_error("Cannot determine order for matching.")
            nscales_radish = 0
            for scale in sorted(glob.glob(pjoin(radish_folder,"scale.*"))):
              nscales_radish = nscales_radish + 1
            for scale in sorted(glob.glob(pjoin(radish_folder,"scale.*"))):
              scale_radish = os.path.basename(scale)
              try:
                  os.makedirs(pjoin(matched_folder,scale_radish))
                  os.makedirs(pjoin(matched_folder,scale_radish,"cumulant"))
              except:
                  pass
              for radish_file in glob.glob(pjoin(radish_folder,scale,"*")):
                if not "--lum" in radish_file:
                    file_name = os.path.basename(radish_file)
                    matched_file = radish_file.replace(file_name,"plot."+file_name).replace(radish_folder,matched_folder).replace(".dat",matched_ending)
                    resummed_file = radish_file.replace(file_name,"plot."+file_name).replace(radish_folder,matched_folder).replace(".dat",resummed_ending)
                    expansion_file = radish_file.replace(file_name,"plot."+file_name).replace(radish_folder,matched_folder).replace(".dat","..exp."+matching_order+".dat")
                    fixed_order_file = radish_file.replace(file_name,"plot."+file_name).replace(radish_folder,matched_folder).replace(".dat",".."+matching_order+".dat")
                    matched_cumulant_file = radish_file.replace(file_name,"cumulant/cumulant."+file_name).replace(radish_folder,matched_folder).replace(".dat",matched_ending)
                    resummed_cumulant_file = radish_file.replace(file_name,"cumulant/cumulant."+file_name).replace(radish_folder,matched_folder).replace(".dat",resummed_ending)
                    expansion_cumulant_file = radish_file.replace(file_name,"cumulant/cumulant."+file_name).replace(radish_folder,matched_folder).replace(".dat","..exp."+matching_order+".dat")
                    fixed_order_cumulant_file = radish_file.replace(file_name,"cumulant/cumulant."+file_name).replace(radish_folder,matched_folder).replace(".dat",".."+matching_order+".dat")
                    # determine other relevant file paths (lumi file, and fixed order)
                    radish_lum_file = radish_file.replace(".dat","--lum.dat")
                    scale_number_matrix = str(int(scale_radish[-1])-1)
                    if (resvariations == 1 ):
                        if ( (nscales_radish == 9) and (int(scale_radish[-1]) > 7) ):
                            scale_number_matrix = "3"
                        elif ( (nscales_radish == 11) and (int(scale_radish[-1]) > 9) ):
                            scale_number_matrix = "4"
                    scale_matrix  = "scale."+scale_number_matrix+"."+scale_number_matrix
                    if matching_order == "NLO.QCD":
                        LO_cross_section_file = sorted(glob.glob(pjoin(os.path.dirname(os.path.dirname(os.path.dirname(radish_file))),scale_matrix,"NLO.*","plot.total_rate.NLO.*.dat")))[0]
                        NLO_cross_section_file = glob.glob(pjoin(os.path.dirname(os.path.dirname(os.path.dirname(radish_file))),scale_matrix,"NLO.CS.*","plot.total_rate.NLO.CS.*.dat"))[0]
                        fixed_order_distribution_file_NLO = glob.glob(pjoin(os.path.dirname(os.path.dirname(os.path.dirname(radish_file))),scale_matrix,"NLO.CS.*","plot."+os.path.basename(radish_file)).replace(".dat",".NLO.CS.*.dat"))[0]
                    elif matching_order == "NNLO.QCD":
                        LO_cross_section_file = sorted(glob.glob(pjoin(os.path.dirname(os.path.dirname(os.path.dirname(radish_file))),scale_matrix,"NNLO.*","plot.total_rate.NNLO.*.dat")))[0]
                        NLO_cross_section_file = glob.glob(pjoin(os.path.dirname(os.path.dirname(os.path.dirname(radish_file))),scale_matrix,"NNLO.CS.*","plot.total_rate.NNLO.CS.*.dat"))[0]
                        NNLO_cross_section_file = glob.glob(pjoin(os.path.dirname(os.path.dirname(os.path.dirname(radish_file))),scale_matrix,"NNLO.QT-CS.*","plot.total_rate.NNLO.QT-CS.*.dat"))[0]
                        fixed_order_distribution_file_NLO = glob.glob(pjoin(os.path.dirname(os.path.dirname(os.path.dirname(radish_file))),scale_matrix,"NNLO.CS.*","plot."+os.path.basename(radish_file)).replace(".dat",".NNLO.CS.*.dat"))[0]
                        fixed_order_distribution_file_NNLO = glob.glob(pjoin(os.path.dirname(os.path.dirname(os.path.dirname(radish_file))),scale_matrix,"NNLO.QT-CS.*","plot."+os.path.basename(radish_file)).replace(".dat",".NNLO.QT-CS.*.dat"))[0]
                    else:
                        out.print_error("Undetermined order for matching.")

                    # reading radish files
                    radish_data = self.readin_file_float(radish_file) # reads space-separated matrix from file
                    radish_lum_data = self.readin_file_float(radish_lum_file) # reads space-separated matrix from file
                    fixed_order_distribution_data_NLO = self.readin_file_float(fixed_order_distribution_file_NLO)
                    if matching_order == "NNLO.QCD":
                        fixed_order_distribution_data_NNLO = self.readin_file_float(fixed_order_distribution_file_NNLO)

                    LO                = self.readin_file_float(LO_cross_section_file)[0][1]
                    LO_err            = self.readin_file_float(LO_cross_section_file)[0][2]
                    deltaNLO          = self.readin_file_float(NLO_cross_section_file)[0][1]
                    deltaNLO_err      = self.readin_file_float(NLO_cross_section_file)[0][2]

                    try:
                        deltaNNLO     = self.readin_file_float(NNLO_cross_section_file)[0][1]
                        deltaNNLO_err = self.readin_file_float(NNLO_cross_section_file)[0][2]
                    except:
                        deltaNNLO = 0.0
                        deltaNNLO_err = 0.0
                        pass

                    if radish_lum_data[1][2] == 0:
                        rescale_factor_LO = 1.
                    else:
                        rescale_factor_LO = LO/radish_lum_data[1][2]

                    pt_left            = self.get_column_from_table(radish_data,0)
                    pt                 = self.get_column_from_table(radish_data,1)
                    resummed           = [x*rescale_factor_LO for x in self.get_column_from_table(radish_data,2)]
                    resummed_err       = [x*rescale_factor_LO for x in self.get_column_from_table(radish_data,3)]
                    res_expanded_1     = [x*rescale_factor_LO for x in self.get_column_from_table(radish_data,4)]
                    res_expanded_1_err = [x*rescale_factor_LO for x in self.get_column_from_table(radish_data,5)]
                    res_expanded_2     = [x*rescale_factor_LO for x in self.get_column_from_table(radish_data,6)]
                    res_expanded_2_err = [x*rescale_factor_LO for x in self.get_column_from_table(radish_data,7)]
                    res_expanded_3     = [x*rescale_factor_LO for x in self.get_column_from_table(radish_data,8)]
                    res_expanded_3_err = [x*rescale_factor_LO for x in self.get_column_from_table(radish_data,9)]
                    radish_lum_data    = [x*rescale_factor_LO for x in radish_lum_data[1]]

                    sigma        = [LO, deltaNLO, deltaNNLO, 0.0]
                    sigma_err    = [LO_err, deltaNLO_err, deltaNNLO_err, 0.0]

                    # for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NLO,1)[:-1]):
                    #     print pt[index+1]-pt_left[index+1]

                    if matching_order == "NLO.QCD":
                        sigma_1     = [value * (pt[index+1]-pt_left[index+1]) for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NLO,1)[1:-1])]
                        sigma_1_err = [value * (pt[index+1]-pt_left[index+1]) for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NLO,2)[1:-1])]
                        if (pt_left[0] == 0):
                            sigma_1.insert(0,self.get_column_from_table(fixed_order_distribution_data_NLO,1)[0])
                            sigma_1_err.insert(0,self.get_column_from_table(fixed_order_distribution_data_NLO,2)[0])
                        else:
                            # this shouldn't happen!!!
                            sigma_1.insert(0,self.get_column_from_table(fixed_order_distribution_data_NLO,1)[0]*(pt[1]-pt_left[1]))
                            sigma_1_err.insert(0,self.get_column_from_table(fixed_order_distribution_data_NLO,2)[0]*(pt[1]-pt_left[1]))
                        sigma_2     = [0] * len(pt)
                        sigma_2_err = [0] * len(pt)
                        sigma_3     = [0] * len(pt)
                        sigma_3_err = [0] * len(pt)
                        sigmabar_1     = [-value * (pt[index+1]-pt_left[index+1]) for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NLO,1)[:-1])]
                        sigmabar_1_err = [-value * (pt[index+1]-pt_left[index+1]) for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NLO,2)[:-1])]
                        sigmabar_2     = [0] * len(pt)
                        sigmabar_2_err = [0] * len(pt)
                        sigmabar_3     = [0] * len(pt)
                        sigmabar_3_err = [0] * len(pt)
                    elif matching_order == "NNLO.QCD":
                        sigma_1     = [value * (pt[index+1]-pt_left[index+1]) for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NLO,1)[1:-1])]
                        sigma_1_err = [value * (pt[index+1]-pt_left[index+1]) for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NLO,2)[1:-1])]
                        if (pt_left[0] == 0): 
                            sigma_1.insert(0,self.get_column_from_table(fixed_order_distribution_data_NLO,1)[0])
                            sigma_1_err.insert(0,self.get_column_from_table(fixed_order_distribution_data_NLO,2)[0])
                        else:
                            # this shouldn't happen!!!
                            sigma_1.insert(0,self.get_column_from_table(fixed_order_distribution_data_NLO,1)[0]*(pt[1]-pt_left[1]))
                            sigma_1_err.insert(0,self.get_column_from_table(fixed_order_distribution_data_NLO,2)[0]*(pt[1]-pt_left[1]))
                        sigma_2     = [value * (pt[index+1]-pt_left[index+1]) for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NNLO,1)[1:-1])]
                        sigma_2_err = [value * (pt[index+1]-pt_left[index+1]) for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NNLO,2)[1:-1])]
                        if (pt_left[0] == 0):
                            sigma_2.insert(0,self.get_column_from_table(fixed_order_distribution_data_NNLO,1)[0])
                            sigma_2_err.insert(0,self.get_column_from_table(fixed_order_distribution_data_NNLO,2)[0])
                        else:
                            # this shouldn't happen!!!
                            sigma_2.insert(0,self.get_column_from_table(fixed_order_distribution_data_NNLO,1)[0]*(pt[1]-pt_left[1]))
                            sigma_2_err.insert(0,self.get_column_from_table(fixed_order_distribution_data_NNLO,2)[0]*(pt[1]-pt_left[1]))
                        sigma_3     = [0] * len(pt)
                        sigma_3_err = [0] * len(pt)
                        sigmabar_1     = [-value * (pt[index+1]-pt_left[index+1]) for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NLO,1)[:-1])]
                        sigmabar_1_err = [-value * (pt[index+1]-pt_left[index+1]) for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NLO,2)[:-1])]
                        sigmabar_2     = [-value * (pt[index+1]-pt_left[index+1]) for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NNLO,1)[:-1])]
                        sigmabar_2_err = [-value * (pt[index+1]-pt_left[index+1]) for index, value in enumerate(self.get_column_from_table(fixed_order_distribution_data_NNLO,2)[:-1])]
                        sigmabar_3     = [0] * len(pt)
                        sigmabar_3_err = [0] * len(pt)

                    sigmasum_1 = sum(sigma_1)
                    sigmabarsum_1 = sum(sigmabar_1)
                    sigmasum_1_err = sum(sigmabar_1_err) # this is completely meaningless!

                    sigmasum_2 = sum(sigma_2)
                    sigmabarsum_2 = sum(sigmabar_2)
                    sigmasum_2_err = sum(sigmabar_2_err) # this is completely meaningless!

                    sigmasum_3 = sum(sigma_3)
                    sigmabarsum_3 = sum(sigmabar_3)
                    sigmasum_3_err = sum(sigmabar_3_err) # this is completely meaningless!

                    sigma_1     = accumulate(sigma_1)
                    sigma_1_err = accumulate(sigma_1_err)
                    sigma_2     = accumulate(sigma_2)
                    sigma_2_err = accumulate(sigma_2_err)
                    sigma_3     = accumulate(sigma_3)
                    sigma_3_err = accumulate(sigma_3_err)

                    lumi_asy     = [radish_lum_data[2],radish_lum_data[4],radish_lum_data[6],radish_lum_data[8]]
                    lumi_asy_err = [radish_lum_data[3],radish_lum_data[5],radish_lum_data[7],radish_lum_data[9]]

                    if pt[0] == 0.0:
                        pt.pop(0)
                        resummed.pop(0)
                        resummed_err.pop(0)
                        res_expanded_1.pop(0)
                        res_expanded_1_err.pop(0)
                        res_expanded_2.pop(0)
                        res_expanded_2_err.pop(0)
                        res_expanded_3.pop(0)
                        res_expanded_3_err.pop(0)
                    
                    # set final inputs for matching routine
                    nbins = len(pt) # number of bins

                    # print "sigma = ", sigma
                    # print ""
                    # print "lumi = ", lumi_asy
                    # print ""
                    # print "sigmabar_1 = ", sigmabar_1
                    # print ""
                    # print "sigmabar_2 = ", sigmabar_2
                    # print ""
                    # print "sigmabar_3 = ", sigmabar_3
                    # print ""
                    # print "res_expanded_1 = ", res_expanded_1
                    # print ""
                    # print "res_expanded_2 = ", res_expanded_2
                    # print ""
                    # print "res_expanded_3 = ", res_expanded_3
                    # print ""
                    # print "resummed = ", resummed

                    if matching_order == "NLO.QCD":
                        expansion_cumulant = [lumi_asy[0]+res_expanded_1[index] for index, value in enumerate(res_expanded_1)]
                    elif matching_order == "NNLO.QCD":
                        expansion_cumulant = [lumi_asy[0]+res_expanded_1[index]+res_expanded_2[index] for index, value in enumerate(res_expanded_1)]
                    else:
                        out.print_error("Unknown matching_order!")
                    # print "------------------------------------", matching_order
                    # print lumi_asy[0]
                    # print res_expanded_1
                    # print res_expanded_2
                    # # expansion_LO = [lumi_asy[0]+res_expanded_1[index] for index, value in enumerate(res_expanded_1)]
                    # # print expansion_LO
                    # print "========= my cumulant:"
                    # print expansion_cumulant

                    # call matching routine
                    if (joint):
                        matched_cumulant_NLO = []
                        matched_cumulant_NNLO = []
                        # These routines are not needed. They are here to reproduce alternative options for the matching of the joint.
                        #dummy = []
                        #matched_cumulant_NLO, dummy, dummy, dummy, dummy, dummy = do_matching_matrix(scheme, nbins, 1, ro, sigma, sigma, lumi_asy, lumi_asy, sigma_1, sigma_1, sigma_2, sigma_2, sigma_3, sigma_3, resummed, resummed, res_expanded_1, res_expanded_1, res_expanded_2, res_expanded_2, res_expanded_3, res_expanded_3, False)
                        #matched_cumulant_NNLO, dummy, dummy, dummy, dummy, dummy = do_matching_matrix(scheme, nbins, 2, ro, sigma, sigma, lumi_asy, lumi_asy, sigma_1, sigma_1, sigma_2, sigma_2, sigma_3, sigma_3, resummed, resummed, res_expanded_1, res_expanded_1, res_expanded_2, res_expanded_2, res_expanded_3, res_expanded_3, False)                     
                        #deltaNLO = matched_cumulant_NLO[-1] - LO
                        #if (fo >= 2):
                        #    deltaNNLO = matched_cumulant_NNLO[-1] - matched_cumulant_NLO[-1]
                        #else:
                        #    deltaNNLO = 0
                        #sigmares = [LO, deltaNLO, deltaNNLO, 0.0]
                        sigmabar_1.pop(0)
                        sigmabar_2.pop(0)
                        sigmabar_3.pop(0)
                        sigmabar_1.append(-(sigma[1]+sigmabarsum_1))
                        sigmabar_2.append(-(sigma[2]+sigmabarsum_2))
                        sigmabar_3.append(-(sigma[3]+sigmabarsum_3))
                        sigmabar_1_err.pop(0)
                        sigmabar_1_err.append(-(sigma_err[1]+sigmasum_1_err))
                        sigmabar_2_err.pop(0)
                        sigmabar_2_err.append(-(sigma_err[2]+sigmasum_2_err))
                        sigmabar_3_err.pop(0)
                        sigmabar_3_err.append(-(sigma_err[3]+sigmasum_3_err))
                        sigmabar_1     = accumulate(sigmabar_1[::-1])[::-1]
                        sigmabar_1_err = accumulate(sigmabar_1_err[::-1])[::-1]
                        sigmabar_2     = accumulate(sigmabar_2[::-1])[::-1]
                        sigmabar_2_err = accumulate(sigmabar_2_err[::-1])[::-1]
                        sigmabar_3     = accumulate(sigmabar_3[::-1])[::-1]
                        sigmabar_3_err = accumulate(sigmabar_3_err[::-1])[::-1]
                        # This corresponds to what is done in the joint resummation letter
                        # matched_cumulant, matched_cumulant_err, fixed_order_cumulant, fixed_order_cumulant_err = do_matching(scheme, nbins, fo, ro, sigmares, sigmares, lumi_asy, lumi_asy, sigmabar_1, sigmabar_1, sigmabar_2, sigmabar_2, sigmabar_3, sigmabar_3, resummed, resummed, res_expanded_1, res_expanded_1, res_expanded_2, res_expanded_2, res_expanded_3, res_expanded_3, False)
                        # This uses the correct fixed order xsec. Note that using sigmabar_i is equivalent to using sigma_i, but this routine is more stable around the Sudakov Shoulder. 
                        # One can use the default routine but the shoulder becomes slightly unstable (investigate why)
                        matched_cumulant, matched_cumulant_err, fixed_order_cumulant, fixed_order_cumulant_err, expansion, expansion_err = do_matching(scheme, nbins, fo, ro, sigma, sigma, lumi_asy, lumi_asy, pt, sigmabar_1, sigmabar_1, sigmabar_2, sigmabar_2, sigmabar_3, sigmabar_3, resummed, resummed, res_expanded_1, res_expanded_1, res_expanded_2, res_expanded_2, res_expanded_3, res_expanded_3, False)
                    else:
                        # matched_cumulant, matched_cumulant_err, fixed_order_cumulant, fixed_order_cumulant_err = do_matching_matrix(scheme, nbins, fo, ro, sigma, sigma, lumi_asy, lumi_asy, sigma_1, sigma_1, sigma_2, sigma_2, sigma_3, sigma_3, resummed, resummed, res_expanded_1, res_expanded_1, res_expanded_2, res_expanded_2, res_expanded_3, res_expanded_3, False)
                        sigmabar_1.pop(0)
                        sigmabar_2.pop(0)
                        sigmabar_3.pop(0)
                        sigmabar_1.append(-(sigma[1]+sigmabarsum_1))
                        sigmabar_2.append(-(sigma[2]+sigmabarsum_2))
                        sigmabar_3.append(-(sigma[3]+sigmabarsum_3))
                        sigmabar_1_err.pop(0)
                        sigmabar_1_err.append(-(sigma_err[1]+sigmasum_1_err))
                        sigmabar_2_err.pop(0)
                        sigmabar_2_err.append(-(sigma_err[2]+sigmasum_2_err))
                        sigmabar_3_err.pop(0)
                        sigmabar_3_err.append(-(sigma_err[3]+sigmasum_3_err))
                        sigmabar_1     = accumulate(sigmabar_1[::-1])[::-1]
                        sigmabar_1_err = accumulate(sigmabar_1_err[::-1])[::-1]
                        sigmabar_2     = accumulate(sigmabar_2[::-1])[::-1]
                        sigmabar_2_err = accumulate(sigmabar_2_err[::-1])[::-1]
                        sigmabar_3     = accumulate(sigmabar_3[::-1])[::-1]
                        sigmabar_3_err = accumulate(sigmabar_3_err[::-1])[::-1]
                        matched_cumulant, matched_cumulant_err, fixed_order_cumulant, fixed_order_cumulant_err, expansion, expansion_err = do_matching(scheme, nbins, fo, ro, sigma, sigma, lumi_asy, lumi_asy, pt, sigmabar_1, sigmabar_1, sigmabar_2, sigmabar_2, sigmabar_3, sigmabar_3, resummed, resummed, res_expanded_1, res_expanded_1, res_expanded_2, res_expanded_2, res_expanded_3, res_expanded_3, False, do_damping, observable, damping_scale)
                    # print ""
                    # print "matched_cumulant = ", list(matched_cumulant)
                    # print ""
                    # print "fixed_order_cumulant = ", list(fixed_order_cumulant)

                    # print "========= output cumulant:"
                    # print list(expansion)
                    # add zeros to get back distribution
                    pt.insert(0,0)
                    matched_cumulant = list(matched_cumulant)
                    matched_cumulant.insert(0,0)
                    matched_cumulant_err = list(matched_cumulant_err)
                    matched_cumulant_err.insert(0,0)
                    resummed_cumulant = list(resummed)
                    resummed_cumulant.insert(0,0)
                    resummed_cumulant_err = list(resummed_err)
                    resummed_cumulant_err.insert(0,0)
                    fixed_order_cumulant = list(fixed_order_cumulant)
                    fixed_order_cumulant.insert(0,0)
                    fixed_order_cumulant_err = list(fixed_order_cumulant_err)
                    fixed_order_cumulant_err.insert(0,0)
                    expansion_cumulant = list(expansion)
                    expansion_cumulant.insert(0,0)
                    expansion_cumulant_err = list(expansion_err)
                    expansion_cumulant_err.insert(0,0)
                    matched_cumulant_array = []
                    matched_distribution_array = []
                    resummed_cumulant_array = []
                    resummed_distribution_array = []
                    fixed_order_cumulant_array = []
                    fixed_order_distribution_array = []
                    expansion_cumulant_array = []
                    expansion_distribution_array = []
                    # print matched_file
                    # print "pt, matched, fixed_order, resummed, expanded1, expanded2, expanded3"
                    for i in range(0,len(pt)-1):
                        matched_cumulant_array.append([pt[i+1], matched_cumulant[i+1], matched_cumulant_err[i+1]])
                        matched_distribution_array.append([pt[i], (matched_cumulant[i+1]-matched_cumulant[i])/(pt[i+1]-pt[i]), math.sqrt(matched_cumulant_err[i+1]**2+matched_cumulant_err[i]**2)/(pt[i+1]-pt[i])])
                        resummed_cumulant_array.append([pt[i+1], resummed_cumulant[i+1], resummed_cumulant_err[i+1]])
                        resummed_distribution_array.append([pt[i], (resummed_cumulant[i+1]-resummed_cumulant[i])/(pt[i+1]-pt[i]), math.sqrt(resummed_cumulant_err[i+1]**2+resummed_cumulant_err[i]**2)/(pt[i+1]-pt[i])])
                        fixed_order_cumulant_array.append([pt[i+1], fixed_order_cumulant[i+1], fixed_order_cumulant_err[i+1]])
                        fixed_order_distribution_array.append([pt[i], (fixed_order_cumulant[i+1]-fixed_order_cumulant[i])/(pt[i+1]-pt[i]), math.sqrt(fixed_order_cumulant_err[i+1]**2+fixed_order_cumulant_err[i]**2)/(pt[i+1]-pt[i])])
                        expansion_cumulant_array.append([pt[i+1], expansion_cumulant[i+1], expansion_cumulant_err[i+1]])
                        expansion_distribution_array.append([pt[i], (expansion_cumulant[i+1]-expansion_cumulant[i])/(pt[i+1]-pt[i]), math.sqrt(expansion_cumulant_err[i+1]**2+expansion_cumulant_err[i]**2)/(pt[i+1]-pt[i])])
                        # print(pt[i+1], matched_cumulant[i+1], fixed_order_cumulant[i+1], resummed[i], res_expanded_1[i], res_expanded_2[i], res_expanded_3[i])

                    matched_distribution_array.append([pt[len(pt)-1], (matched_cumulant[len(pt)-1]-matched_cumulant[len(pt)-2])/(pt[len(pt)-1]-pt[len(pt)-2]), math.sqrt(matched_cumulant_err[len(pt)-1]**2+matched_cumulant_err[len(pt)-2]**2)/(pt[len(pt)-1]-pt[len(pt)-2])])
                    resummed_distribution_array.append([pt[len(pt)-1], (resummed_cumulant[len(pt)-1]-resummed_cumulant[len(pt)-2])/(pt[len(pt)-1]-pt[len(pt)-2]), math.sqrt(resummed_cumulant_err[len(pt)-1]**2+resummed_cumulant_err[len(pt)-2]**2)/(pt[len(pt)-1]-pt[len(pt)-2])])
                    fixed_order_distribution_array.append([pt[len(pt)-1], (fixed_order_cumulant[len(pt)-1]-fixed_order_cumulant[len(pt)-2])/(pt[len(pt)-1]-pt[len(pt)-2]), math.sqrt(fixed_order_cumulant_err[len(pt)-1]**2+fixed_order_cumulant_err[len(pt)-2]**2)/(pt[len(pt)-1]-pt[len(pt)-2])])
                    expansion_distribution_array.append([pt[len(pt)-1], (expansion_cumulant[len(pt)-1]-expansion_cumulant[len(pt)-2])/(pt[len(pt)-1]-pt[len(pt)-2]), math.sqrt(expansion_cumulant_err[len(pt)-1]**2+expansion_cumulant_err[len(pt)-2]**2)/(pt[len(pt)-1]-pt[len(pt)-2])])

                    # print matched_distribution_array
                    # print matched_cumulant_array
                    # print fixed_order_distribution_array
                    # print fixed_order_cumulant_array
                    # exit(0)

                    # print to files
                    self.print_table_to_file(matched_distribution_array,matched_file)
                    self.print_table_to_file(resummed_distribution_array,resummed_file)
                    self.print_table_to_file(fixed_order_distribution_array,fixed_order_file)
                    self.print_table_to_file(expansion_distribution_array,expansion_file)
                    self.print_table_to_file(matched_cumulant_array,matched_cumulant_file)
                    self.print_table_to_file(resummed_cumulant_array,resummed_cumulant_file)
                    self.print_table_to_file(fixed_order_cumulant_array,fixed_order_cumulant_file)
                    self.print_table_to_file(expansion_cumulant_array,expansion_cumulant_file)

            # create min max 7/9 point
            for radish_file in glob.glob(pjoin(matched_folder,"scale.1","*.dat")):
                radish_file = os.path.basename(radish_file)
                out_file = pjoin(matched_folder,"MATRIX+RadISH",radish_file.replace("plot.","").replace("..","__").replace(".","_").replace("_dat",".dat"))
                file_list_tmp = glob.glob(pjoin(matched_folder,"scale.*",radish_file))
                if len(file_list_tmp) == 7:
                    if resvariations == 0:
                        central_nr = 4
                    else:
                        out.print_error("Mismatch in the number of scales. Don't know how to handle results in RadISH combination. Exiting...")
                elif len(file_list_tmp) == 9:
                    if resvariations == 1:
                        central_nr = 4
                    elif resvariations == 0:
                        central_nr = 5
                    else:
                        out.print_error("Mismatch in the number of scales. Don't know how to handle results in RadISH combination. Exiting...")
                elif len(file_list_tmp) == 11:
                    if resvariations == 1:
                        central_nr = 5
                    else:
                        out.print_error("Mismatch in the number of scales. Don't know how to handle results in RadISH combination. Exiting...")
                elif len(file_list_tmp) == 1:
                    central_nr = 1
                else:
                    out.print_error("Number of scale variation neither 1,7,9. Don't know how to handle results in RadISH combination. Exiting...")
                if onlyscentralscale == 1:
                    central_nr = 1
                file_list = []
                for file_name in file_list_tmp:
                    if "scale."+str(central_nr) == os.path.basename(os.path.dirname(file_name)):
                        file_list.insert(0,file_name)
                    else:
                        file_list.append(file_name)
                self.get_central_min_max_from_distribution_files(file_list,out_file)
                out_file = pjoin(matched_folder,"MATRIX+RadISH","cumulant",radish_file.replace("plot.","cumulant.").replace("..","__").replace(".","_").replace("_dat",".dat"))
                file_list_tmp = glob.glob(pjoin(matched_folder,"scale.*","cumulant",radish_file.replace("plot.","cumulant.")))
                file_list = []
                for file_name in file_list_tmp:
                    if "scale."+str(central_nr) == os.path.basename(os.path.dirname(os.path.dirname(file_name))):
                        file_list.insert(0,file_name)
                    else:
                        file_list.append(file_name)
                self.get_central_min_max_from_distribution_files(file_list,out_file)
        # copy MATRIX+RadISH folder for each order to MATRIX result folder
        for radish_folder in glob.glob(pjoin(fold.run_folder_path,"result","result.*LO_dist","*","*.RadISH")):
            matched_folder = radish_folder.replace("/NLL.RadISH","/NLO+NLL.matched").replace("/NNLL.RadISH","/NNLO+NNLL.matched").replace("/N3LL.RadISH","/NNLO+N3LL.matched")
            if radish_folder.endswith("/NLL.RadISH"):
                run_order = "NLO-run"
            elif radish_folder.endswith("/NNLL.RadISH") or  radish_folder.endswith("/N3LL.RadISH"):
                run_order = "NNLO-run"
            else:
                out.print_error("Cannot determine order of run in matchin routine with RadISH.")
            # create folder with matched results
#            print pjoin(matched_folder,"MATRIX+RadISH"),pjoin(fold.result_folder_path,run_order,"MATRIX+RadISH")
#            print matched_folder,run_order
            try:
                shutil.rmtree(pjoin(fold.result_folder_path,run_order,"MATRIX+RadISH"))
            except:
                pass
            shutil.copytree(pjoin(matched_folder,"MATRIX+RadISH"),pjoin(fold.result_folder_path,run_order,"MATRIX+RadISH"))
#}}}
#{{{ def: print_table_to_file(self, matched, fixed_order)
    def print_table_to_file(self, array, out_file):
        # this routine prints out an array to a file
        with open(out_file,'w') as f:
            for row in range(len(array)):
                line = ""
                for column in range(len(array[row])):
                    length = 15
                    line = line+" "*(length-len("%.8g" % array[row][column]))+"%.8g" % array[row][column]
                f.write(line+"\n")
            f.close()
#}}}
#}}}
#{{{ class: citations()
class citations(): # class to deal with the results, adding files, copying them to the results folder etc.
#{{{ def: __init__(self,process_in)
    def __init__(self,process_in):
        self.physics_first_list = ["ppemxnmnex04","ppemexnmx04","ppeexmxnm04","ppeeexnex04","ppeexexne04","ppeexnenex04","ppzz02","ppwxw02","ppeeexex04","ppemexmx04","ppeexa03","ppnenexa03","ppenexa03","ppexnea03"]
        self.process = process_in
        self.banner_start = """%          /-------------------------------------------------------------------\\
%          |           __  __     ___     ____    ___         _     _          |
%          |          | _\/_ |   / _ \   |____|  / _ \   ||   \\\\   //          |
%          |          || \/ ||  | |_| |    ||    ||_||   ||    \\\\ //           |
%          |          ||    ||  | ___ |    ||    ||\\\\    ||    // \\\\           |
%          |          ||    ||  ||   ||    ||    || \\\\   ||   //   \\\\          |
%          |                                                                   |
%          |-------------------------------------------------------------------|
%          | MATRIX is based on a number of different computations and tools   |
%          | from various people and groups. Please acknowledge their efforts  |
%          | by citing the references for this run, which are given below.     |
%          \-------------------------------------------------------------------/
"""
        self.banner_end = """
%          /-------------------------------------------------------------------\\
%          | MATRIX is based on a number of different computations and tools   |
%          | from various people and groups. Please acknowledge their efforts  |
%          | by citing the references for this run, which are given above.     |
%          \-------------------------------------------------------------------/
"""
        self.citation_list_standard = self.get_citation_list_standard()
        self.citation_list_process  = self.get_citation_list_process()
        self.citation_list_amplitudes  = self.get_citation_list_amplitudes()
        self.parameter_list = ""
#}}}
#{{{ def: write_citations(self,file_path, citation_list_run, citation_list_standard = self.citation_list_standard,citation_list_process = self.citation_list_process)
    def write_citations(self,file_path, parameter_list, citation_list_run, citation_list_standard = None, citation_list_process = None, citation_list_amplitudes = None):
        # this routine writes out the relevant citations for a process
        self.parameter_list = parameter_list
        
        if citation_list_standard is None:
            citation_list_standard = self.citation_list_standard
        if citation_list_process is None:
            citation_list_process = self.citation_list_process
        if citation_list_amplitudes is None:
            citation_list_amplitudes = self.citation_list_amplitudes
        if parameter_list.get("add_RadISH_NLL") == "1" or parameter_list.get("add_RadISH_NNLL") == "1" or parameter_list.get("add_RadISH_N3LL") == "1":
            self.use_matrix_plus_radish_banner()
            citation_list_matrix_plus_radish = self.get_citation_list_matrix_plus_radish()
            citation_list_radish = self.get_citation_list_radish()
        else:
            citation_list_matrix_plus_radish = {}
            citation_list_radish = {}
        with open(file_path,'w') as cite_file:
                cite_file.write(self.banner_start)
                cite_file.write("\n")
                cite_file.write("%-------------\\\n")
                cite_file.write("% Spires keys |\n")
                cite_file.write("%-------------/\n")
                if citation_list_matrix_plus_radish:
                    cite_file.write("% MATRIX+RadISH publication:\n")
                    for spireskey, bibtex in citation_list_matrix_plus_radish.iteritems():
                        cite_file.write("%% - %s\n" % spireskey)
                if citation_list_matrix_plus_radish:
                    cite_file.write("%\n")
                if self.process in self.physics_first_list:
                    if citation_list_process:
                        cite_file.write("% Physics publications:\n")
                    else:
                        out.print_warning("No physics publications available for process \"%s\". Please check yourself wether relevant publications exist." % self.process)
                    for spireskey, bibtex in citation_list_process.iteritems():
                        cite_file.write("%% - %s\n" % spireskey)
                    cite_file.write("%\n")
                    cite_file.write("% MATRIX release publication:\n")
                    for spireskey, bibtex in citation_list_standard.iteritems():
                        cite_file.write("%% - %s\n" % spireskey)                    
                else:
                    cite_file.write("% MATRIX release publication:\n")
                    for spireskey, bibtex in citation_list_standard.iteritems():
                        cite_file.write("%% - %s\n" % spireskey)                    
                    if citation_list_process:
                        cite_file.write("%\n")
                        cite_file.write("% Physics publications:\n")
                    else:
                        out.print_warning("No physics publications available for process \"%s\". Please check yourself wether relevant publications exist." % self.process)
                    for spireskey, bibtex in citation_list_process.iteritems():
                        cite_file.write("%% - %s\n" % spireskey)
                if citation_list_radish:
                    cite_file.write("%\n")
                    cite_file.write("% RadISH resummation citations:\n")
                    for spireskey, bibtex in citation_list_radish.iteritems():
                        cite_file.write("%% - %s\n" % spireskey)                    
                if citation_list_amplitudes:
                    cite_file.write("%\n")
                    cite_file.write("% Amplitude citations:\n")
                for spireskey, bibtex in citation_list_amplitudes.iteritems():
                    cite_file.write("%% - %s\n" % spireskey)                    
                if citation_list_run:
                    cite_file.write("%\n")
                    cite_file.write("% Subtraction-method citations:\n")
                for spireskey, bibtex in citation_list_run.iteritems():
                    cite_file.write("%% - %s\n" % spireskey)                    
                cite_file.write("\n")
                cite_file.write("\n")
                cite_file.write("%----------------\\\n")
                cite_file.write("% BibTeX entries |\n")
                cite_file.write("%----------------/\n")
                if citation_list_matrix_plus_radish:
                    cite_file.write("% MATRIX+RadISH publication:\n")
                    for spireskey, bibtex in citation_list_matrix_plus_radish.iteritems():
                       cite_file.write("%s\n" % bibtex)
                       cite_file.write("\n")
                if citation_list_matrix_plus_radish:
                    cite_file.write("\n") 
                if self.process in self.physics_first_list:
                    if citation_list_process:
                        cite_file.write("% Physics publications:\n")
                    for spireskey, bibtex in citation_list_process.iteritems():
                        cite_file.write("%s\n" % bibtex)                    
                        cite_file.write("\n")
                    cite_file.write("\n")
                    cite_file.write("% MATRIX release publication:\n")
                    for spireskey, bibtex in citation_list_standard.iteritems():
                        cite_file.write("%s\n" % bibtex)                    
                        cite_file.write("\n")
                else:
                    cite_file.write("% MATRIX release publication:\n")
                    for spireskey, bibtex in citation_list_standard.iteritems():
                        cite_file.write("%s\n" % bibtex)                    
                        cite_file.write("\n")
                    if citation_list_process:
                        cite_file.write("\n")
                        cite_file.write("% Physics publications:\n")
                    for spireskey, bibtex in citation_list_process.iteritems():
                        cite_file.write("%s\n" % bibtex)                    
                        cite_file.write("\n")
                if citation_list_radish:
                    cite_file.write("\n")
                    cite_file.write("% RadISH resummation citations:\n")
                    for spireskey, bibtex in citation_list_radish.iteritems():
                        cite_file.write("%s\n" % bibtex)                    
                        cite_file.write("\n")
                if citation_list_amplitudes:
                    cite_file.write("\n")
                    cite_file.write("% Amplitude citations:\n")
                for spireskey, bibtex in citation_list_amplitudes.iteritems():
                    cite_file.write("%s\n" % bibtex)                    
                    cite_file.write("\n")
                if citation_list_run:
                    cite_file.write("\n")
                    cite_file.write("% Subtraction-method citations:\n")
                for spireskey, bibtex in citation_list_run.iteritems():
                    cite_file.write("%s\n" % bibtex)                    
                    cite_file.write("\n")
                cite_file.write(self.banner_end)
#}}}        
#{{{ def: use_matrix_plus_radish_banner(self)
    def use_matrix_plus_radish_banner(self):
        self.banner_start = """%          /-------------------------------------------------------------------\\
%          |        __  __            _______   _____    _____  __    __       |
%          |       |  \/  |     /\\   |__   __| |  __ \\  |_   _| \ \\  / /       |
%          |       | \  / |    /  \\     | |    | |__) |   | |    \ \\/ /        |
%          |       | |\/| |   / /\ \\    | |    |  _  /    | |     |  |         |
%          |       | |  | |  / ____ \\   | |    | | \ \   _| |_   / /\\ \        |
%          |       |_|  |_| /_/    \\_\\  |_|    |_|  \_\\ |_____| /_/  \\_\\       |
%          |                                 _                                 |
%          |                               _| |_                               |
%          |                              |_   _|                              |
%          |                                |_|                                |
%          |          _____                _   _____    _____   _    _         |
%          |         |  __ \              | | |_   _|  / ____| | |  | |        |
%          |         | |__) |   __ _    __| |   | |   | (___   | |__| |        |
%          |         |  _  /   / _` |  / _` |   | |    \___ \  |  __  |        |
%          |         | | \ \  | (_| | | (_| |  _| |_   ____) | | |  | |        |
%          |         |_|  \_\  \__,_|  \__,_| |_____| |_____/  |_|  |_|        |
%          |                                                                   |
%          |-------------------------------------------------------------------|
%          | MATRIX+RadISH is based on many different computations and tools   |
%          | from various people and groups. Please acknowledge their efforts  |
%          | by citing the references for this run, which are given below.     |
%          \-------------------------------------------------------------------/
"""
        self.banner_end = """
%          /-------------------------------------------------------------------\\
%          | MATRIX+RadISH is based on many different computations and tools   |
%          | from various people and groups. Please acknowledge their efforts  |
%          | by citing the references for this run, which are given above.     |
%          \-------------------------------------------------------------------/
"""
#}}}
#{{{ def: get_citation_list_standard(self)
    def get_citation_list_standard(self):
        standard_list = {}
        standard_list["Grazzini:2017mhc"] = """
@article{Grazzini:2017mhc,
      author         = "Grazzini, Massimiliano and Kallweit, Stefan and
                        Wiesemann, Marius",
      title          = "{Fully differential NNLO computations with MATRIX}",
      journal        = "Eur. Phys. J.",
      volume         = "C78",
      year           = "2018",
      number         = "7",
      pages          = "537",
      doi            = "10.1140/epjc/s10052-018-5771-7",
      eprint         = "1711.06631",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-30-17, CERN-TH-2017-232, ZU-TH 30/17",
      SLACcitation   = "%%CITATION = ARXIV:1711.06631;%%"
}"""
        return standard_list
#}}}        
#{{{ def: get_citation_list_process(self, process = self.process)
    def get_citation_list_process(self, process = None):
        if process is None:
            process = self.process

        Z_list = {}
        Z_list["Catani:2009sm"] = """
@article{Catani:2009sm,
      author         = "Catani, Stefano and Cieri, Leandro and Ferrera, Giancarlo
                        and de Florian, Daniel and Grazzini, Massimiliano",
      title          = "{Vector boson production at hadron colliders: a fully
                        exclusive QCD calculation at NNLO}",
      journal        = "Phys. Rev. Lett.",
      volume         = "103",
      year           = "2009",
      pages          = "082001",
      doi            = "10.1103/PhysRevLett.103.082001",
      eprint         = "0903.2120",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      SLACcitation   = "%%CITATION = ARXIV:0903.2120;%%"
}"""
        W_list = Z_list

        HH_list = {}
        HH_list["deFlorian:2016uhr"] = """
@article{deFlorian:2016uhr,
      author         = "de Florian, Daniel and Grazzini, Massimiliano and Hanga,
                        Catalin and Kallweit, Stefan and Lindert, Jonas M. and
                        Maierh\"ofer, Philipp and Mazzitelli, Javier and Rathlev,
                        Dirk",
      title          = "{Differential Higgs Boson Pair Production at
                        Next-to-Next-to-Leading Order in QCD}",
      journal        = "JHEP",
      volume         = "09",
      year           = "2016",
      pages          = "151",
      doi            = "10.1007/JHEP09(2016)151",
      eprint         = "1606.09519",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "DESY-16-107, FR-PHENO-2016-007, ICAS-08-16, MITP-16-061,
                        ZU-TH-20-16",
      SLACcitation   = "%%CITATION = ARXIV:1606.09519;%%"
}"""

        gammagamma_list = {}
        gammagamma_list["Catani:2011qz"] = """
@article{Catani:2011qz,
      author         = "Catani, Stefano and Cieri, Leandro and de Florian, Daniel
                        and Ferrera, Giancarlo and Grazzini, Massimiliano",
      title          = "{Diphoton production at hadron colliders: a
                        fully-differential QCD calculation at NNLO}",
      journal        = "Phys. Rev. Lett.",
      volume         = "108",
      year           = "2012",
      pages          = "072001",
      doi            = "10.1103/PhysRevLett.108.072001,
                        10.1103/PhysRevLett.117.089901",
      note           = "[Erratum: Phys. Rev. Lett.117,no.8,089901(2016)]",
      eprint         = "1110.2375",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-18-11, IFUM-984-FT",
      SLACcitation   = "%%CITATION = ARXIV:1110.2375;%%"
}"""

        Zgamma_list = {}
        Zgamma_list["Grazzini:2013bna"] = """
@article{Grazzini:2013bna,
      author         = "Grazzini, Massimiliano and Kallweit, Stefan and Rathlev,
                        Dirk and Torre, Alessandro",
      title          = "{$Z\gamma$ production at hadron colliders in NNLO QCD}",
      journal        = "Phys. Lett.",
      volume         = "B731",
      year           = "2014",
      pages          = "204-207",
      doi            = "10.1016/j.physletb.2014.02.037",
      eprint         = "1309.7000",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-21-13",
      SLACcitation   = "%%CITATION = ARXIV:1309.7000;%%"
}"""
        Zgamma_list["Grazzini:2015nwa"] = """
@article{Grazzini:2015nwa,
      author         = "Grazzini, Massimiliano and Kallweit, Stefan and Rathlev,
                        Dirk",
      title          = "{$W\gamma$ and $Z\gamma$ production at the LHC in NNLO
                        QCD}",
      journal        = "JHEP",
      volume         = "07",
      year           = "2015",
      pages          = "085",
      doi            = "10.1007/JHEP07(2015)085",
      eprint         = "1504.01330",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-04-15, MITP-15-021",
      SLACcitation   = "%%CITATION = ARXIV:1504.01330;%%"
}"""

        Wgamma_list = {}
        Wgamma_list["Grazzini:2015nwa"] = """
@article{Grazzini:2015nwa,
      author         = "Grazzini, Massimiliano and Kallweit, Stefan and Rathlev,
                        Dirk",
      title          = "{$W\gamma$ and $Z\gamma$ production at the LHC in NNLO
                        QCD}",
      journal        = "JHEP",
      volume         = "07",
      year           = "2015",
      pages          = "085",
      doi            = "10.1007/JHEP07(2015)085",
      eprint         = "1504.01330",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-04-15, MITP-15-021",
      SLACcitation   = "%%CITATION = ARXIV:1504.01330;%%"
}"""

        ZZ_list = {}
        ZZ_list["Cascioli:2014yka"] = """
@article{Cascioli:2014yka,
      author         = "Cascioli, F. and Gehrmann, T. and Grazzini, M. and
                        Kallweit, S. and Maierh\"ofer, P. and von Manteuffel, A.
                        and Pozzorini, S. and Rathlev, D. and Tancredi, L. and
                        Weihs, E.",
      title          = "{ZZ production at hadron colliders in NNLO QCD}",
      journal        = "Phys. Lett.",
      volume         = "B735",
      year           = "2014",
      pages          = "311-313",
      doi            = "10.1016/j.physletb.2014.06.056",
      eprint         = "1405.2219",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-19-14, MITP-14-033",
      SLACcitation   = "%%CITATION = ARXIV:1405.2219;%%"
}"""
        ZZ_list["Grazzini:2015hta"] = """
@article{Grazzini:2015hta,
      author         = "Grazzini, Massimiliano and Kallweit, Stefan and Rathlev,
                        Dirk",
      title          = "{ZZ production at the LHC: fiducial cross sections and
                        distributions in NNLO QCD}",
      journal        = "Phys. Lett.",
      volume         = "B750",
      year           = "2015",
      pages          = "407-410",
      doi            = "10.1016/j.physletb.2015.09.055",
      eprint         = "1507.06257",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      SLACcitation   = "%%CITATION = ARXIV:1507.06257;%%"
}"""
        ZZ_list["Kallweit:2018nyv"] = """
@article{Kallweit:2018nyv,
      author         = "Kallweit, Stefan and Wiesemann, Marius",
      title          = "{$ZZ$ production at the LHC: NNLO predictions for
                        $2\ell2\nu$ and $4\ell$ signatures}",
      year           = "2018",
      eprint         = "1806.05941",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "CERN-TH-2018-127, CERN-TH-2018-127",
      SLACcitation   = "%%CITATION = ARXIV:1806.05941;%%"
}
"""


        WW_list = {}
        WW_list["Gehrmann:2014fva"] = """
@article{Gehrmann:2014fva,
      author         = "Gehrmann, T. and Grazzini, M. and Kallweit, S. and
                        Maierh\"ofer, P. and von Manteuffel, A. and Pozzorini, S.
                        and Rathlev, D. and Tancredi, L.",
      title          = "{$W^+W^-$ Production at Hadron Colliders in Next to Next
                        to Leading Order QCD}",
      journal        = "Phys. Rev. Lett.",
      volume         = "113",
      year           = "2014",
      number         = "21",
      pages          = "212001",
      doi            = "10.1103/PhysRevLett.113.212001",
      eprint         = "1408.5243",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-29-14, MITP-14-053",
      SLACcitation   = "%%CITATION = ARXIV:1408.5243;%%"
}"""
        WW_list["Grazzini:2016ctr"] = """
@article{Grazzini:2016ctr,
      author         = "Grazzini, Massimiliano and Kallweit, Stefan and
                        Pozzorini, Stefano and Rathlev, Dirk and Wiesemann,
                        Marius",
      title          = "{$W^{+}W^{-}$ production at the LHC: fiducial cross
                        sections and distributions in NNLO QCD}",
      journal        = "JHEP",
      volume         = "08",
      year           = "2016",
      pages          = "140",
      doi            = "10.1007/JHEP08(2016)140",
      eprint         = "1605.02716",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-16-16, MITP-16-038, NSF-KITP-16-047, DESY-16-075",
      SLACcitation   = "%%CITATION = ARXIV:1605.02716;%%"
}"""
        WZ_list = {}
        WZ_list["Grazzini:2016swo"] = """
@article{Grazzini:2016swo,
      author         = "Grazzini, Massimiliano and Kallweit, Stefan and Rathlev,
                        Dirk and Wiesemann, Marius",
      title          = "{$W^{\pm}Z$ production at hadron colliders in NNLO QCD}",
      journal        = "Phys. Lett.",
      volume         = "B761",
      year           = "2016",
      pages          = "179-183",
      doi            = "10.1016/j.physletb.2016.08.017",
      eprint         = "1604.08576",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-15-16, MITP-16-037, NSF-KITP-16-046, DESY-16-074",
      SLACcitation   = "%%CITATION = ARXIV:1604.08576;%%"
}"""
        WZ_list["Grazzini:2017ckn"] = """
@article{Grazzini:2017ckn,
      author         = "Grazzini, Massimiliano and Kallweit, Stefan and Rathlev,
                        Dirk and Wiesemann, Marius",
      title          = "{$W^\pm Z$ production at the LHC: fiducial cross sections
                        and distributions in NNLO QCD}",
      year           = "2017",
      eprint         = "1703.09065",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-06-17, CERN-TH-2017-065",
      SLACcitation   = "%%CITATION = ARXIV:1703.09065;%%"
}"""
        ZZplusWW_list = ZZ_list.copy()
        ZZplusWW_list.update(WW_list)

        citation_list = {}
#        citation_list["pph21"] = H_list
        citation_list["ppz01"] = Z_list
        citation_list["ppw01"] = W_list
        citation_list["ppwx01nockm"] = W_list
        citation_list["ppw01nockm"] = W_list
        citation_list["ppwx01"] = W_list
        citation_list["ppeex02"] = Z_list
        citation_list["ppnenex02"] = Z_list
        citation_list["ppexne02nockm"] = W_list
        citation_list["ppenex02nockm"] = W_list
        citation_list["ppexne02"] = W_list
        citation_list["ppenex02"] = W_list
        citation_list["pphh22"] = HH_list
        citation_list["ppaa02"] = gammagamma_list
        citation_list["ppeexa03"] = Zgamma_list
        citation_list["ppnenexa03"] = Zgamma_list
        citation_list["ppexnea03"] = Wgamma_list
        citation_list["ppenexa03"] = Wgamma_list
        citation_list["ppzz02"] = ZZ_list
        citation_list["ppwxw02"] = WW_list
        citation_list["ppeeexex04"] = ZZ_list
        citation_list["ppemexmx04"] = ZZ_list
        citation_list["ppeexnenex04"] = ZZplusWW_list
        citation_list["ppeexnmnmx04"] = ZZ_list
        citation_list["ppemxnmnex04"] = WW_list
        citation_list["ppemexnmx04"] = WZ_list
        citation_list["ppeexmxnm04"] = WZ_list
        citation_list["ppeeexnex04"] = WZ_list
        citation_list["ppeexexne04"] = WZ_list
        return citation_list.get(process,{})
#}}}        
#{{{ def: get_citation_list_matrix_plus_radish(self, process = self.process)
    def get_citation_list_matrix_plus_radish(self, process = None):
        matrix_plus_radish_list = {}
        matrix_plus_radish_list["Kallweit:2020gva"] = """
@article{Kallweit:2020gva,
      author         = "Kallweit, Stefan and Re, Emanuele and Rottoli, Luca and
                        Wiesemann, Marius",
      title          = "{Accurate single- and double-differential resummation of
                        colour-singlet processes with MATRIX+RadISH: $W^+W^-$
                        production at the LHC}",
      year           = "2020",
      eprint         = "2004.07720",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "MPP-2020-53, LAPTH-012/20",
      SLACcitation   = "%%CITATION = ARXIV:2004.07720;%%"
}"""
        return matrix_plus_radish_list
#}}}        
#{{{ def: get_citation_list_amplitudes(self, process = self.process)
    def get_citation_list_amplitudes(self, process = None):
        if process is None:
            process = self.process
        amplitude_list = {}
        amplitude_list["Cascioli:2011va"] = """
@article{Cascioli:2011va,
      author         = "Cascioli, Fabio and Maierh\"ofer, Philipp and Pozzorini,
                        Stefano",
      title          = "{Scattering Amplitudes with Open Loops}",
      journal        = "Phys. Rev. Lett.",
      volume         = "108",
      year           = "2012",
      pages          = "111601",
      doi            = "10.1103/PhysRevLett.108.111601",
      eprint         = "1111.5206",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-23-11, LPN11-66",
      SLACcitation   = "%%CITATION = ARXIV:1111.5206;%%"
}"""
        amplitude_list["Denner:2016kdg"] = """
@article{Denner:2016kdg,
      author         = "Denner, Ansgar and Dittmaier, Stefan and Hofer, Lars",
      title          = "{Collier: a fortran-based Complex One-Loop LIbrary in
                        Extended Regularizations}",
      journal        = "Comput. Phys. Commun.",
      volume         = "212",
      year           = "2017",
      pages          = "220-238",
      doi            = "10.1016/j.cpc.2016.10.013",
      eprint         = "1604.06792",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "FR-PHENO-2016-003, ICCUB-16-016",
      SLACcitation   = "%%CITATION = ARXIV:1604.06792;%%"
}"""
        if process == "pph21":
            amplitude_list["Harlander:2000mg"] = """
@article{Harlander:2000mg,
      author         = "Harlander, Robert V.",
      title          = "{Virtual corrections to g g ---> H to two loops in the
                        heavy top limit}",
      journal        = "Phys. Lett.",
      volume         = "B492",
      year           = "2000",
      pages          = "74-80",
      doi            = "10.1016/S0370-2693(00)01042-X",
      eprint         = "hep-ph/0007289",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "BNL-HET-00-29",
      SLACcitation   = "%%CITATION = HEP-PH/0007289;%%"
}"""
        elif process in ["ppw01","ppwx01","ppnenex02","ppexne02nockm","ppenex02nockm","ppexne02","ppenex02"]:
            amplitude_list["Matsuura:1988sm"] = """
@article{Matsuura:1988sm,
      author         = "Matsuura, T. and van der Marck, S. C. and van Neerven, W.
                        L.",
      title          = "{The Calculation of the Second Order Soft and Virtual
                        Contributions to the Drell-Yan Cross-Section}",
      journal        = "Nucl. Phys.",
      volume         = "B319",
      year           = "1989",
      pages          = "570-622",
      doi            = "10.1016/0550-3213(89)90620-2",
      reportNumber   = "Print-88-0798 (LEIDEN)",
      SLACcitation   = "%%CITATION = NUPHA,B319,570;%%"
}"""
        elif process == "pphh22":
            amplitude_list["deFlorian:2013uza"] = """
@article{deFlorian:2013uza,
      author         = "de Florian, Daniel and Mazzitelli, Javier",
      title          = "{Two-loop virtual corrections to Higgs pair production}",
      journal        = "Phys. Lett.",
      volume         = "B724",
      year           = "2013",
      pages          = "306-309",
      doi            = "10.1016/j.physletb.2013.06.046",
      eprint         = "1305.5206",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      SLACcitation   = "%%CITATION = ARXIV:1305.5206;%%"
}"""
        elif process == "ppaa02":
            amplitude_list["Anastasiou:2002zn"] = """
@article{Anastasiou:2002zn,
      author         = "Anastasiou, C. and Glover, E. W. Nigel and
                        Tejeda-Yeomans, M. E.",
      title          = "{Two loop QED and QCD corrections to massless fermion
                        boson scattering}",
      journal        = "Nucl. Phys.",
      volume         = "B629",
      year           = "2002",
      pages          = "255-289",
      doi            = "10.1016/S0550-3213(02)00140-2",
      eprint         = "hep-ph/0201274",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "SLAC-PUB-9130, DCPT-02-10, IPPP-02-05, YITP-SB-02-04",
      SLACcitation   = "%%CITATION = HEP-PH/0201274;%%"
}"""
        elif process in ["ppeexa03","ppnenexa03","ppexnea03","ppenexa03"]:
            amplitude_list["Gehrmann:2011ab"] = """
@article{Gehrmann:2011ab,
      author         = "Gehrmann, Thomas and Tancredi, Lorenzo",
      title          = "{Two-loop QCD helicity amplitudes for $q\bar q \to W^\pm
                        \gamma$ and $q\bar q \to Z^0 \gamma$}",
      journal        = "JHEP",
      volume         = "02",
      year           = "2012",
      pages          = "004",
      doi            = "10.1007/JHEP02(2012)004",
      eprint         = "1112.1531",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-26-11",
      SLACcitation   = "%%CITATION = ARXIV:1112.1531;%%"
}"""
        elif process in ["ppzz02","ppwxw02","ppeeexex04","ppemexmx04","ppeexnenex04","ppeexnmnmx04","ppemxnmnex04","ppemexnmx04","ppeexmxnm04","ppeeexnex04","ppeexexne04"]:
            amplitude_list["Gehrmann:2015ora"] = """
@article{Gehrmann:2015ora,
      author         = "Gehrmann, Thomas and von Manteuffel, Andreas and
                        Tancredi, Lorenzo",
      title          = "{The two-loop helicity amplitudes for $
                        q\overline{q}^{\prime}\to {V}_1{V}_2\to 4 $ leptons}",
      journal        = "JHEP",
      volume         = "09",
      year           = "2015",
      pages          = "128",
      doi            = "10.1007/JHEP09(2015)128",
      eprint         = "1503.04812",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-03-15, MITP-15-011, TTP15-011",
      SLACcitation   = "%%CITATION = ARXIV:1503.04812;%%"
}"""
        return amplitude_list
#}}}        
#{{{ def: get_citation_list_radish(self, process = self.process)
    def get_citation_list_radish(self, process = None):
        radish_list = {}
        if int(self.parameter_list.get("RadISH_observable")) in [4,5]:
               radish_list["Monni:2019yyr"] = """
@article{Monni:2019yyr,
      author         = "Monni, Pier Francesco and Rottoli, Luca and Torrielli,
                        Paolo",
      title          = "{Higgs transverse momentum with a jet veto: a
                        double-differential resummation}",
      year           = "2019",
      eprint         = "1909.04704",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "CERN-TH-2019-101",
      SLACcitation   = "%%CITATION = ARXIV:1909.04704;%%"
}"""
        radish_list["Bizon:2017rah"] = """
@article{Bizon:2017rah,
      author         = "Bizon, Wojciech and Monni, Pier Francesco and Re,
                        Emanuele and Rottoli, Luca and Torrielli, Paolo",
      title          = "{Momentum-space resummation for transverse observables
                        and the Higgs p$_{\perp}$ at N$^{3}$LL+NNLO}",
      journal        = "JHEP",
      volume         = "02",
      year           = "2018",
      pages          = "108",
      doi            = "10.1007/JHEP02(2018)108",
      eprint         = "1705.09127",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "OUTP-17-05P, CERN-TH-2017-111, LAPTH-018-17",
      SLACcitation   = "%%CITATION = ARXIV:1705.09127;%%"
}"""
        radish_list["Monni:2016ktx"] = """
@article{Monni:2016ktx,
      author         = "Monni, Pier Francesco and Re, Emanuele and Torrielli,
                        Paolo",
      title          = "{Higgs Transverse-Momentum Resummation in Direct Space}",
      journal        = "Phys. Rev. Lett.",
      volume         = "116",
      year           = "2016",
      number         = "24",
      pages          = "242001",
      doi            = "10.1103/PhysRevLett.116.242001",
      eprint         = "1604.02191",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "LAPTH-017-16, OUTP-16-07P",
      SLACcitation   = "%%CITATION = ARXIV:1604.02191;%%"
}"""
        return radish_list
#}}}        
#{{{ def: get_citation_list_run(self, order, NLO_subtraction_method, parameter_list)
    def get_citation_list_run(self, order, NLO_subtraction_method, parameter_list):
        run_list = OrderedDict()
        if parameter_list.get("add_NLL","0") == "1" or parameter_list.get("add_NNLL","0") == "1":            
            run_list["Grazzini:2015wpa"] = """
@article{Grazzini:2015wpa,
      author         = "Grazzini, Massimiliano and Kallweit, Stefan and Rathlev,
                        Dirk and Wiesemann, Marius",
      title          = "{Transverse-momentum resummation for vector-boson pair
                        production at NNLL+NNLO}",
      journal        = "JHEP",
      volume         = "08",
      year           = "2015",
      pages          = "154",
      doi            = "10.1007/JHEP08(2015)154",
      eprint         = "1507.02565",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-20-15, MITP-15-051",
      SLACcitation   = "%%CITATION = ARXIV:1507.02565;%%"
}"""
            run_list["deFlorian:2012mx"] = """
@article{deFlorian:2012mx,
      author         = "de Florian, D. and Ferrera, G. and Grazzini, M. and
                        Tommasini, D.",
      title          = "{Higgs boson production at the LHC: transverse momentum
                        resummation effects in the H->2gamma, H->WW->lnu lnu and
                        H->ZZ->4l decay modes}",
      journal        = "JHEP",
      volume         = "06",
      year           = "2012",
      pages          = "132",
      doi            = "10.1007/JHEP06(2012)132",
      eprint         = "1203.6321",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-04-12, IFUM-993-FT",
      SLACcitation   = "%%CITATION = ARXIV:1203.6321;%%"
}"""
            run_list["Bozzi:2005wk"] = """
@article{Bozzi:2005wk,
      author         = "Bozzi, Giuseppe and Catani, Stefano and de Florian,
                        Daniel and Grazzini, Massimiliano",
      title          = "{Transverse-momentum resummation and the spectrum of the
                        Higgs boson at the LHC}",
      journal        = "Nucl. Phys.",
      volume         = "B737",
      year           = "2006",
      pages          = "73-120",
      doi            = "10.1016/j.nuclphysb.2005.12.022",
      eprint         = "hep-ph/0508068",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      SLACcitation   = "%%CITATION = HEP-PH/0508068;%%"
}"""

        if "NNLO" in order or ("NLO" in order and not NLO_subtraction_method == "1"):
            run_list["Catani:2007vq"] = """
@article{Catani:2007vq,
      author         = "Catani, Stefano and Grazzini, Massimiliano",
      title          = "{An NNLO subtraction formalism in hadron collisions and
                        its application to Higgs boson production at the LHC}",
      journal        = "Phys. Rev. Lett.",
      volume         = "98",
      year           = "2007",
      pages          = "222002",
      doi            = "10.1103/PhysRevLett.98.222002",
      eprint         = "hep-ph/0703012",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      SLACcitation   = "%%CITATION = HEP-PH/0703012;%%"
}"""
        if "NNLO" in order:
            if self.process in ["pph21","pphh22"]:
                run_list["Catani:2011kr"] = """
@article{Catani:2011kr,
      author         = "Catani, S. and Grazzini, M.",
      title          = "{Higgs Boson Production at Hadron Colliders:
                        Hard-Collinear Coefficients at the NNLO}",
      journal        = "Eur. Phys. J.",
      volume         = "C72",
      year           = "2012",
      pages          = "2013",
      doi            = "10.1140/epjc/s10052-012-2013-2,
                        10.1140/epjc/s10052-012-2132-9",
      note           = "[Erratum: Eur. Phys. J.C72,2132(2012)]",
      eprint         = "1106.4652",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-12-11",
      SLACcitation   = "%%CITATION = ARXIV:1106.4652;%%"
}"""
            else:
                run_list["Catani:2012qa"] = """
@article{Catani:2012qa,
      author         = "Catani, Stefano and Cieri, Leandro and de Florian, Daniel
                        and Ferrera, Giancarlo and Grazzini, Massimiliano",
      title          = "{Vector boson production at hadron colliders:
                        hard-collinear coefficients at the NNLO}",
      journal        = "Eur. Phys. J.",
      volume         = "C72",
      year           = "2012",
      pages          = "2195",
      doi            = "10.1140/epjc/s10052-012-2195-7",
      eprint         = "1209.0158",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      reportNumber   = "ZU-TH-16-12",
      SLACcitation   = "%%CITATION = ARXIV:1209.0158;%%"
}"""


#         if "NNLO" in order:
#             run_list["Catani:2013tia"] = """
# @article{Catani:2013tia,
#       author         = "Catani, Stefano and Cieri, Leandro and de Florian, Daniel
#                         and Ferrera, Giancarlo and Grazzini, Massimiliano",
#       title          = "{Universality of transverse-momentum resummation and hard
#                         factors at the NNLO}",
#       journal        = "Nucl. Phys.",
#       volume         = "B881",
#       year           = "2014",
#       pages          = "414-443",
#       doi            = "10.1016/j.nuclphysb.2014.02.011",
#       eprint         = "1311.1654",
#       archivePrefix  = "arXiv",
#       primaryClass   = "hep-ph",
#       reportNumber   = "ZU-TH-25-13",
#       SLACcitation   = "%%CITATION = ARXIV:1311.1654;%%"
# }
# """
#         if "NNLO" in order or ("NLO" in order and NLO_subtraction_method == "1"):            
#             run_list["Catani:1996vz"] = """
# @article{Catani:1996vz,
#       author         = "Catani, S. and Seymour, M. H.",
#       title          = "{A General algorithm for calculating jet cross-sections
#                         in NLO QCD}",
#       journal        = "Nucl. Phys.",
#       volume         = "B485",
#       year           = "1997",
#       pages          = "291-419",
#       doi            = "10.1016/S0550-3213(96)00589-5,
#                         10.1016/S0550-3213(98)81022-5",
#       note           = "[Erratum: Nucl. Phys.B510,503(1998)]",
#       eprint         = "hep-ph/9605323",
#       archivePrefix  = "arXiv",
#       primaryClass   = "hep-ph",
#       reportNumber   = "CERN-TH-96-029, CERN-TH-96-29",
#       SLACcitation   = "%%CITATION = HEP-PH/9605323;%%"
# }
# """
        return run_list
#}}}
#}}}        
#{{{ class: data_point_fitter()
class data_point_fitter(): # class to deal with the fitting of data points, first implementation: linear regression
#{{{ def: __init__(self,data_points)
    def __init__(self,data_points):
        self.data_points = data_points
#}}}
#{{{ def: mean(values)
    def mean(self,values):
    # Calculate the mean value of a list of numbers
        return sum(values) / float(len(values))
#}}}
#{{{ def: covariance(x, mean_x, y, mean_y)
    def covariance(self,x, mean_x, y, mean_y):
    # Calculate covariance between x and y
        covar = 0.0
        for i in range(len(x)):
            covar += (x[i] - mean_x) * (y[i] - mean_y)
        return covar
#}}}
#{{{ def: variance(values, mean)
    def variance(self,values, mean):
    # Calculate the variance of a list of numbers
        return sum([(x-mean)**2 for x in values])
#}}}
#{{{ def: coefficients(dataset)
    def coefficients(self,mode="linear"):
    # Calculate coefficients
        if mode == "linear":
            x = [row[0] for row in self.data_points]
            y = [row[1] for row in self.data_points]
            x_mean, y_mean = self.mean(x), self.mean(y)
            b1 = self.covariance(x, x_mean, y, y_mean) / self.variance(x, x_mean)
            b0 = y_mean - b1 * x_mean
        else:
            out.print_error("Other fitting procedures than \"linear\" not implemented...")
        return [b0, b1]
#}}}
#}}}        
