#ifdef MORE

#include "../include/classes.cxx"
#include "header/more.h"

void ConvertToFortran(char* fstring, std::size_t fstring_len,
                      const char* cstring)
{

    std::size_t inlen = std::strlen(cstring);
    std::size_t cpylen = std::min(inlen, fstring_len);

    if (inlen > fstring_len)
    {
        std::cout << "ConvertToFortran: illegal sizes (" << inlen << ", " << fstring_len << ")" << std::endl;
        exit(1);
    }

    std::copy(cstring, cstring + cpylen, fstring);
    std::fill(fstring + cpylen, fstring + fstring_len, ' ');
}

void initMore(int resorder, int pertorder, observable_set & oset) {
  resorder_.resorder=resorder;
  resorder_.pertorder=pertorder;

  resumchan_.pdf_content_modify=-1; //FIXME oset.pdf_content_modify;

  lopowers_.pcf=0;
  lopowers_.kcf=0;

  flaginitres_.doinitres=1;

  char test[64]="MSTW2008nnlo90cl.LHgrid               "; //FIXME
  for (int i=0; i<oset.LHAPDFname.size(); i++) {
                test[i]=oset.LHAPDFname.c_str()[i];
  }
  for (int i=oset.LHAPDFname.size(); i<63; i++) {
          test[i]=' ';
  }
  char fstring[64];
  ConvertToFortran(fstring,sizeof test,test);

  for (int i=0; i<64; i++) {
          pdfhres_.name[i]=fstring[i];
  }

  pdfhres_.member=1;
  pdfhres_.ih1=1;
  pdfhres_.ih2=1;

  nf_.nf = oset.N_f;

  pdfset_();

  somescale_.startscale=oset.var_mu_fact;
}

// boosts system to the specified transverse momentum. the azimuth is generated uniformly
void performQTBoost(double qt2,double Q,double y,vector<vector<vector<particle> > > &p) {
  // boost final state system
  fourvector q_lab;
//  double Q = sqrt(xbs_all[0][0]);
//  double y = log(sqrt(x_pdf[1]/x_pdf[2]));

//  q_lab = osi_particle_event[24][0][0].momentum()+osi_particle_event[24][0][1].momentum();
  double pt = 0.5*Q*(exp(y)+exp(-y));
  double pz = 0.5*Q*(exp(y)-exp(-y));


//  cout << "q_lab = " << q_lab << endl;
  q_lab = fourvector(pt,0,0,pz);
//  cout << "q_lab = " << q_lab << ", y = " << q_lab.rapidity() << ", eta = " << q_lab.eta() << endl;

//  cout << "pz=" << pz << ", " << q_lab.x3() << "; pt=" << pt << ", " << q_lab.x0() << endl;
  double mT = sqrt(Q*Q+qt2);

  // boost to the correct z
  double B = pt*pz/(pt*pt+mT*mT/Q/Q*pz*pz);
  double C = (pz*pz-mT*mT/Q/Q*pz*pz)/(pt*pt+mT*mT/Q/Q*pz*pz);
  double beta_z = B-sqrt(B*B-C);

  if (pz<0) {
    beta_z = B+sqrt(B*B-C);
  }

//  cout << beta_z << endl;
//  if (beta_z<0) {
//    beta_z = B+sqrt(B*B-C);
//  }
  q_lab = q_lab.zboost(beta_z);
//  cout << "correctness: " << 0.5*mT*(exp(y)-exp(-y)) << ", " << (1.0/sqrt((1-beta_z*beta_z)))*(pz-beta_z*pt) << endl;

//  cout << beta_z << endl;
//  cout << q_lab << "; " << q_lab.m() << endl;
//  cout << "pz=" << q_lab.x3() << ", " << 0.5*mT*(exp(y)-exp(-y)) << "; pt=" << q_lab.x0() << ", " << 0.5*mT*(exp(y)+exp(-y)) << endl;

  // boost to the correct qT
  double beta_x = -sqrt(qt2/(q_lab.x0()*q_lab.x0()+qt2));

  if (q_lab.x0()<0)
    beta_x *= -1.0;

  q_lab = q_lab.xboost(beta_x);
//  cout << "pz=" << q_lab.x3() << ", " << 0.5*mT*(exp(y)-exp(-y)) << "; pt=" << q_lab.x0() << ", " << 0.5*mT*(exp(y)+exp(-y)) << endl;

  // generate angle for qT
  double qT_phi = 2*M_PI*double(rand())/RAND_MAX;
//  cout << qT_phi << endl;
  q_lab = q_lab.rot(qT_phi,1);
//  cout << "q_lab = " << q_lab << "; " << q_lab.pT() << endl;

  // modify all final state momenta according to the prescription above
//  cout << "beta_z, beta_x, qT_phi = " << beta_z << ", " << beta_x << ", " << qT_phi << endl;
  for (int j1=0; j1<p.size(); j1++) {
    for (int i_a=0; i_a<p[j1].size(); i_a++) {
      for (int i_p=0; i_p<p[j1][i_a].size(); i_p++) {
          // event vector might have unused entries; skip these
          if (p[j1][i_a][i_p].momentum == physconst::nullvector) {
            continue;
          }
          //cout << j1 << ", " << i_a << ", " << i_p << endl;
          //cout << p[j1][i_a][i_p].momentum << endl;

          q_lab = p[j1][i_a][i_p].momentum;

          q_lab = q_lab.zboost(beta_z);
          q_lab = q_lab.xboost(beta_x);
          q_lab = q_lab.rot(qT_phi,1);

          p[j1][i_a][i_p] = particle(q_lab);
          //cout << p[j1][i_a][i_p].momentum << endl;
      }
    }
  }
}

// boosts system to the specified transverse momentum. the azimuth is generated uniformly
void performQTBoost_cms(double qt2,double Q,double y,vector<vector<fourvector> > &p) {
  // if we are boosting fourvectors in the cms system, we first have to transform to the lab system
  // and then transform back at the very end
  double boost_cms=0;
  boost_cms = (exp(y)-exp(-y))/(exp(y)+exp(-y));
  //cout << "boost_cms = " << boost_cms << endl;
//  boost = (x_pdf[1] - x_pdf[2]) / (x_pdf[1] + x_pdf[2]);


  // boost final state system
  fourvector q_lab;
//  double Q = sqrt(xbs_all[0][0]);
//  double y = log(sqrt(x_pdf[1]/x_pdf[2]));

//  q_lab = osi_particle_event[24][0][0].momentum()+osi_particle_event[24][0][1].momentum();
  double pt = 0.5*Q*(exp(y)+exp(-y));
  double pz = 0.5*Q*(exp(y)-exp(-y));


//  cout << "q_lab = " << q_lab << endl;
  q_lab = fourvector(pt,0,0,pz);
//  cout << "q_lab = " << q_lab << ", y = " << q_lab.rapidity() << ", eta = " << q_lab.eta() << endl;

//  cout << "pz=" << pz << ", " << q_lab.x3() << "; pt=" << pt << ", " << q_lab.x0() << endl;
  double mT = sqrt(Q*Q+qt2);

  // boost to the correct z
  double B = pt*pz/(pt*pt+mT*mT/Q/Q*pz*pz);
  double C = (pz*pz-mT*mT/Q/Q*pz*pz)/(pt*pt+mT*mT/Q/Q*pz*pz);
  double beta_z = B-sqrt(B*B-C);

  if (pz<0) {
    beta_z = B+sqrt(B*B-C);
  }

//  cout << beta_z << endl;
//  if (beta_z<0) {
//    beta_z = B+sqrt(B*B-C);
//  }
  q_lab = q_lab.zboost(beta_z);
//  cout << "correctness: " << 0.5*mT*(exp(y)-exp(-y)) << ", " << (1.0/sqrt((1-beta_z*beta_z)))*(pz-beta_z*pt) << endl;

//  cout << beta_z << endl;
//  cout << q_lab << "; " << q_lab.m() << endl;
//  cout << "pz=" << q_lab.x3() << ", " << 0.5*mT*(exp(y)-exp(-y)) << "; pt=" << q_lab.x0() << ", " << 0.5*mT*(exp(y)+exp(-y)) << endl;

  // boost to the correct qT
  double beta_x = -sqrt(qt2/(q_lab.x0()*q_lab.x0()+qt2));

  if (q_lab.x0()<0)
    beta_x *= -1.0;

  q_lab = q_lab.xboost(beta_x);
//  cout << "pz=" << q_lab.x3() << ", " << 0.5*mT*(exp(y)-exp(-y)) << "; pt=" << q_lab.x0() << ", " << 0.5*mT*(exp(y)+exp(-y)) << endl;

  // generate angle for qT
  double qT_phi = 2*M_PI*double(rand())/RAND_MAX;
//  cout << qT_phi << endl;
  q_lab = q_lab.rot(qT_phi,1);
//  cout << "q_lab = " << q_lab << "; " << q_lab.pT() << endl;

  // modify all final state momenta according to the prescription above
//  cout << "beta_z, beta_x, qT_phi = " << beta_z << ", " << beta_x << ", " << qT_phi << endl;
    for (int i_a=0; i_a<p.size(); i_a++) {
      for (int i_p=0; i_p<p[i_a].size(); i_p++) {
          // event vector might have unused entries; skip these
          if (p[i_a][i_p]== physconst::nullvector) {
            continue;
          }
          //cout << i_a << ", " << i_p << endl;
          //cout << p[i_a][i_p]<< endl;

          q_lab = p[i_a][i_p];

          q_lab = q_lab.zboost(-boost_cms);

          q_lab = q_lab.zboost(beta_z);
          q_lab = q_lab.xboost(beta_x);
          q_lab = q_lab.rot(qT_phi,1);

          q_lab = q_lab.zboost(boost_cms);

          p[i_a][i_p] = q_lab;
          //cout << p[i_a][i_p] << endl;
      }
  }
}

void set_PDF_mellin_coefficients() {
  for (int kkm1=0; kkm1<50; kkm1++) {
  lopowers_.pcf=0;
    pdfmellin_.A1UV_in[kkm1] = 0.;  pdfmellin_.A2UV_in[kkm1] = 0.;  pdfmellin_.A3UV_in[kkm1] = 0.;  pdfmellin_.A4UV_in[kkm1] = 0.;  pdfmellin_.A5UV_in[kkm1] = 0.;  pdfmellin_.A6UV_in[kkm1] = 0.;  pdfmellin_.A7UV_in[kkm1] = 0.;  pdfmellin_.A8UV_in[kkm1] = 0.;
    pdfmellin_.A1DV_in[kkm1] = 0.;  pdfmellin_.A2DV_in[kkm1] = 0.;  pdfmellin_.A3DV_in[kkm1] = 0.;  pdfmellin_.A4DV_in[kkm1] = 0.;  pdfmellin_.A5DV_in[kkm1] = 0.;  pdfmellin_.A6DV_in[kkm1] = 0.;  pdfmellin_.A7DV_in[kkm1] = 0.;  pdfmellin_.A8DV_in[kkm1] = 0.;
    pdfmellin_.A1US_in[kkm1] = 0.;  pdfmellin_.A2US_in[kkm1] = 0.;  pdfmellin_.A3US_in[kkm1] = 0.;  pdfmellin_.A4US_in[kkm1] = 0.;  pdfmellin_.A5US_in[kkm1] = 0.;  pdfmellin_.A6US_in[kkm1] = 0.;  pdfmellin_.A7US_in[kkm1] = 0.;  pdfmellin_.A8US_in[kkm1] = 0.;
    pdfmellin_.A1DS_in[kkm1] = 0.;  pdfmellin_.A2DS_in[kkm1] = 0.;  pdfmellin_.A3DS_in[kkm1] = 0.;  pdfmellin_.A4DS_in[kkm1] = 0.;  pdfmellin_.A5DS_in[kkm1] = 0.;  pdfmellin_.A6DS_in[kkm1] = 0.;  pdfmellin_.A7DS_in[kkm1] = 0.;  pdfmellin_.A8DS_in[kkm1] = 0.;
    pdfmellin_.A1SS_in[kkm1] = 0.;  pdfmellin_.A2SS_in[kkm1] = 0.;  pdfmellin_.A3SS_in[kkm1] = 0.;  pdfmellin_.A4SS_in[kkm1] = 0.;  pdfmellin_.A5SS_in[kkm1] = 0.;  pdfmellin_.A6SS_in[kkm1] = 0.;  pdfmellin_.A7SS_in[kkm1] = 0.;  pdfmellin_.A8SS_in[kkm1] = 0.;
    pdfmellin_.A1GL_in[kkm1] = 0.;  pdfmellin_.A2GL_in[kkm1] = 0.;  pdfmellin_.A3GL_in[kkm1] = 0.;  pdfmellin_.A4GL_in[kkm1] = 0.;  pdfmellin_.A5GL_in[kkm1] = 0.;  pdfmellin_.A6GL_in[kkm1] = 0.;  pdfmellin_.A7GL_in[kkm1] = 0.;  pdfmellin_.A8GL_in[kkm1] = 0.;
    pdfmellin_.A1CH_in[kkm1] = 0.;  pdfmellin_.A2CH_in[kkm1] = 0.;  pdfmellin_.A3CH_in[kkm1] = 0.;  pdfmellin_.A4CH_in[kkm1] = 0.;  pdfmellin_.A5CH_in[kkm1] = 0.;  pdfmellin_.A6CH_in[kkm1] = 0.;  pdfmellin_.A7CH_in[kkm1] = 0.;  pdfmellin_.A8CH_in[kkm1] = 0.;
    pdfmellin_.A1BO_in[kkm1] = 0.;  pdfmellin_.A2BO_in[kkm1] = 0.;  pdfmellin_.A3BO_in[kkm1] = 0.;  pdfmellin_.A4BO_in[kkm1] = 0.;  pdfmellin_.A5BO_in[kkm1] = 0.;  pdfmellin_.A6BO_in[kkm1] = 0.;  pdfmellin_.A7BO_in[kkm1] = 0.;  pdfmellin_.A8BO_in[kkm1] = 0.;
    pdfmellin_.A1UVp_in[kkm1] = 0.; pdfmellin_.A2UVp_in[kkm1] = 0.; pdfmellin_.A3UVp_in[kkm1] = 0.; pdfmellin_.A4UVp_in[kkm1] = 0.; pdfmellin_.A5UVp_in[kkm1] = 0.; pdfmellin_.A6UVp_in[kkm1] = 0.; pdfmellin_.A7UVp_in[kkm1] = 0.; pdfmellin_.A8UVp_in[kkm1] = 0.;
    pdfmellin_.A1DVp_in[kkm1] = 0.; pdfmellin_.A2DVp_in[kkm1] = 0.; pdfmellin_.A3DVp_in[kkm1] = 0.; pdfmellin_.A4DVp_in[kkm1] = 0.; pdfmellin_.A5DVp_in[kkm1] = 0.; pdfmellin_.A6DVp_in[kkm1] = 0.; pdfmellin_.A7DVp_in[kkm1] = 0.; pdfmellin_.A8DVp_in[kkm1] = 0.;
    pdfmellin_.A1USp_in[kkm1] = 0.; pdfmellin_.A2USp_in[kkm1] = 0.; pdfmellin_.A3USp_in[kkm1] = 0.; pdfmellin_.A4USp_in[kkm1] = 0.; pdfmellin_.A5USp_in[kkm1] = 0.; pdfmellin_.A6USp_in[kkm1] = 0.; pdfmellin_.A7USp_in[kkm1] = 0.; pdfmellin_.A8USp_in[kkm1] = 0.;
    pdfmellin_.A1DSp_in[kkm1] = 0.; pdfmellin_.A2DSp_in[kkm1] = 0.; pdfmellin_.A3DSp_in[kkm1] = 0.; pdfmellin_.A4DSp_in[kkm1] = 0.; pdfmellin_.A5DSp_in[kkm1] = 0.; pdfmellin_.A6DSp_in[kkm1] = 0.; pdfmellin_.A7DSp_in[kkm1] = 0.; pdfmellin_.A8DSp_in[kkm1] = 0.;
    pdfmellin_.A1SSp_in[kkm1] = 0.; pdfmellin_.A2SSp_in[kkm1] = 0.; pdfmellin_.A3SSp_in[kkm1] = 0.; pdfmellin_.A4SSp_in[kkm1] = 0.; pdfmellin_.A5SSp_in[kkm1] = 0.; pdfmellin_.A6SSp_in[kkm1] = 0.; pdfmellin_.A7SSp_in[kkm1] = 0.; pdfmellin_.A8SSp_in[kkm1] = 0.;
    pdfmellin_.A1GLp_in[kkm1] = 0.; pdfmellin_.A2GLp_in[kkm1] = 0.; pdfmellin_.A3GLp_in[kkm1] = 0.; pdfmellin_.A4GLp_in[kkm1] = 0.; pdfmellin_.A5GLp_in[kkm1] = 0.; pdfmellin_.A6GLp_in[kkm1] = 0.; pdfmellin_.A7GLp_in[kkm1] = 0.; pdfmellin_.A8GLp_in[kkm1] = 0.;
    pdfmellin_.A1CHp_in[kkm1] = 0.; pdfmellin_.A2CHp_in[kkm1] = 0.; pdfmellin_.A3CHp_in[kkm1] = 0.; pdfmellin_.A4CHp_in[kkm1] = 0.; pdfmellin_.A5CHp_in[kkm1] = 0.; pdfmellin_.A6CHp_in[kkm1] = 0.; pdfmellin_.A7CHp_in[kkm1] = 0.; pdfmellin_.A8CHp_in[kkm1] = 0.;
    pdfmellin_.A1BOp_in[kkm1] = 0.; pdfmellin_.A2BOp_in[kkm1] = 0.; pdfmellin_.A3BOp_in[kkm1] = 0.; pdfmellin_.A4BOp_in[kkm1] = 0.; pdfmellin_.A5BOp_in[kkm1] = 0.; pdfmellin_.A6BOp_in[kkm1] = 0.; pdfmellin_.A7BOp_in[kkm1] = 0.; pdfmellin_.A8BOp_in[kkm1] = 0.;
  }

 pdfmellin_.A1UV_in[0] =   0.53182008693990623;
 pdfmellin_.A2UV_in[0] =   0.25385510801108202;
 pdfmellin_.A3UV_in[0] =    4.5598731752055262;
 pdfmellin_.A4UV_in[0] =    7.5443035002262633;
 pdfmellin_.A5UV_in[0] =    3.9607612507892753;
 pdfmellin_.A6UV_in[0] =   -13.386861890746502;
 pdfmellin_.A7UV_in[0] =    17.089350998276430;
 pdfmellin_.A8UV_in[0] =   -8.7769116613552427;
 pdfmellin_.A1DV_in[0] =   0.43077675581514940;
 pdfmellin_.A2DV_in[0] =   0.34109614609453964;
 pdfmellin_.A3DV_in[0] =    4.8210777290567783;
 pdfmellin_.A4DV_in[0] =   -2.3807064538577194;
 pdfmellin_.A5DV_in[0] =    4.0548284640925658;
 pdfmellin_.A6DV_in[0] =    1.2850091387489186;
 pdfmellin_.A7DV_in[0] =   -6.3401038266721343;
 pdfmellin_.A8DV_in[0] =    4.3542597728671382;
 pdfmellin_.A1US_in[0] =    9.1237220297165328E-002 ;
 pdfmellin_.A2US_in[0] =  -0.39726652041271809;
 pdfmellin_.A3US_in[0] =    8.0940661576539163;
 pdfmellin_.A4US_in[0] =  -0.76842129600290221;
 pdfmellin_.A5US_in[0] =   0.13097443545694887;
 pdfmellin_.A6US_in[0] =   -4.1276760779174992;
 pdfmellin_.A7US_in[0] =    7.9777571965894367;
 pdfmellin_.A8US_in[0] =    8.8566358666501941;
 pdfmellin_.A1DS_in[0] =   0.11617384202746615;
 pdfmellin_.A2DS_in[0] =  -0.36298798756458789;
 pdfmellin_.A3DS_in[0] =    15.631053958465792;
 pdfmellin_.A4DS_in[0] =    4.9779914224761335;
 pdfmellin_.A5DS_in[0] =   -9.2179847416097971E-002 ;
 pdfmellin_.A6DS_in[0] =    5.8484174208135613;
 pdfmellin_.A7DS_in[0] =    39.592146370348445;
 pdfmellin_.A8DS_in[0] =    89.222690512770015;
 pdfmellin_.A1SS_in[0] =   0.17390507735324748;
 pdfmellin_.A2SS_in[0] =  -0.32443592671136395;
 pdfmellin_.A3SS_in[0] =    13.819339474762693;
 pdfmellin_.A4SS_in[0] =    24.727767464921918;
 pdfmellin_.A5SS_in[0] =   -5.5184640910435974;
 pdfmellin_.A6SS_in[0] =   -59.013990708785499;
 pdfmellin_.A7SS_in[0] =    76.648879415278714;
 pdfmellin_.A8SS_in[0] =   -10.367838571327056;
 pdfmellin_.A1GL_in[0] =    1.6229925469922737;
 pdfmellin_.A2GL_in[0] =  -0.43039554401551466;
 pdfmellin_.A3GL_in[0] =    9.7202803486702969;
 pdfmellin_.A4GL_in[0] =    3.0438116617037672;
 pdfmellin_.A5GL_in[0] =   -2.9435736050872805;
 pdfmellin_.A6GL_in[0] =    24.117976849148050;
 pdfmellin_.A7GL_in[0] =   -82.279162819882856;
 pdfmellin_.A8GL_in[0] =    87.061293646810839;
 pdfmellin_.A1CH_in[0] =    5.6412066373300325E-002 ;
 pdfmellin_.A2CH_in[0] =  -0.44240082703377964;
 pdfmellin_.A3CH_in[0] =    10.888408433603251;
 pdfmellin_.A4CH_in[0] =   -12.092725902670242;
 pdfmellin_.A5CH_in[0] =    7.6265165441299132E-002 ;
 pdfmellin_.A6CH_in[0] =    37.717175902625712;
 pdfmellin_.A7CH_in[0] =   -37.498907180297479;
 pdfmellin_.A8CH_in[0] =    19.461440810690505;
 pdfmellin_.A1BO_in[0] =    4.5827337719813531E-002 ;
 pdfmellin_.A2BO_in[0] =  -0.44054432825377604;
 pdfmellin_.A3BO_in[0] =    14.174805812915565;
 pdfmellin_.A4BO_in[0] =    5.5009848791778495;
 pdfmellin_.A5BO_in[0] =   -2.7742393944502481;
 pdfmellin_.A6BO_in[0] =    4.1562082014654607;
 pdfmellin_.A7BO_in[0] =   -36.468951035095685;
 pdfmellin_.A8BO_in[0] =    79.121421391486564;
 pdfmellin_.A1UVp_in[0] =   0.49027644610728283;
 pdfmellin_.A2UVp_in[0] =   0.23453214546659007;
 pdfmellin_.A3UVp_in[0] =    5.3698156134531061;
 pdfmellin_.A4UVp_in[0] =    13.839174277221019;
 pdfmellin_.A5UVp_in[0] =    3.2660649803689950;
 pdfmellin_.A6UVp_in[0] =   -13.295080289095679;
 pdfmellin_.A7UVp_in[0] =   0.37532216188564860;
 pdfmellin_.A8UVp_in[0] =    27.288921919385388;
 pdfmellin_.A1DVp_in[0] =   0.60497816276916594;
 pdfmellin_.A2DVp_in[0] =   0.38371539491749962;
 pdfmellin_.A3DVp_in[0] =    6.2015915109354331;
 pdfmellin_.A4DVp_in[0] =    3.8333700755124447;
 pdfmellin_.A5DVp_in[0] =    1.7719406886700213;
 pdfmellin_.A6DVp_in[0] =   -2.8937665785466464;
 pdfmellin_.A7DVp_in[0] =   -4.3811125515175320;
 pdfmellin_.A8DVp_in[0] =    13.179166754565287;
 pdfmellin_.A1USp_in[0] =   0.18536143809274100;
 pdfmellin_.A2USp_in[0] =  -0.31338099178510143;
 pdfmellin_.A3USp_in[0] =    9.3593051078426566;
 pdfmellin_.A4USp_in[0] =    20.940215979270903;
 pdfmellin_.A5USp_in[0] =   -4.2675366908073968;
 pdfmellin_.A6USp_in[0] =   -49.185592366887001;
 pdfmellin_.A7USp_in[0] =    43.331810268021059;
 pdfmellin_.A8USp_in[0] =    7.3823349080496143;
 pdfmellin_.A1DSp_in[0] =   0.13600221827452896;
 pdfmellin_.A2DSp_in[0] =  -0.34679998115317012;
 pdfmellin_.A3DSp_in[0] =    15.879795019842440;
 pdfmellin_.A4DSp_in[0] =    13.397796312675062;
 pdfmellin_.A5DSp_in[0] =   -1.5301087100919801;
 pdfmellin_.A6DSp_in[0] =   -19.272560145868269;
 pdfmellin_.A7DSp_in[0] =    59.010204859396197;
 pdfmellin_.A8DSp_in[0] =    89.983423616605876;
 pdfmellin_.A1SSp_in[0] =   -3.5167476188920563E-003 ;
 pdfmellin_.A2SSp_in[0] =  -0.79273982553475320;
 pdfmellin_.A3SSp_in[0] =    10.090641566770206;
 pdfmellin_.A4SSp_in[0] =    159.50147439813136;
 pdfmellin_.A5SSp_in[0] =   -59.999997679373188;
 pdfmellin_.A6SSp_in[0] =   -89.999842212718391;
 pdfmellin_.A7SSp_in[0] =   -89.991387530096048;
 pdfmellin_.A8SSp_in[0] =   -89.998227511676220;
 pdfmellin_.A1GLp_in[0] =    2.0968962352673368;
 pdfmellin_.A2GLp_in[0] =  -0.39960123338586678;
 pdfmellin_.A3GLp_in[0] =    10.715826295517303;
 pdfmellin_.A4GLp_in[0] =    13.541770491782422;
 pdfmellin_.A5GLp_in[0] =   -4.4805582972049365;
 pdfmellin_.A6GLp_in[0] =   -4.2525362253343673;
 pdfmellin_.A7GLp_in[0] =   -52.790332146157951;
 pdfmellin_.A8GLp_in[0] =    84.710071606860879;
 pdfmellin_.A1CHp_in[0] =   0.11121294152183556;
 pdfmellin_.A2CHp_in[0] =  -0.36380119128778232;
 pdfmellin_.A3CHp_in[0] =    14.139680466442476;
 pdfmellin_.A4CHp_in[0] =    18.994205188221322;
 pdfmellin_.A5CHp_in[0] =   -4.6218095655184115;
 pdfmellin_.A6CHp_in[0] =   -35.323773849036293;
 pdfmellin_.A7CHp_in[0] =    6.6273644414921904;
 pdfmellin_.A8CHp_in[0] =    65.247129231919956;
 pdfmellin_.A1BOp_in[0] =    7.5513975335841893E-002 ;
 pdfmellin_.A2BOp_in[0] =  -0.37129459279105043;
 pdfmellin_.A3BOp_in[0] =    24.977057235923535;
 pdfmellin_.A4BOp_in[0] =    7.5932390850366005;
 pdfmellin_.A5BOp_in[0] =   -3.6979995498561209;
 pdfmellin_.A6BOp_in[0] =    21.038655906056604;
 pdfmellin_.A7BOp_in[0] =    25.890278837151953;
 pdfmellin_.A8BOp_in[0] =    18.115343529128197;
 pdfmellin_.A1UV_in[1] =   0.71727726399152714;
 pdfmellin_.A2UV_in[1] =   0.28834904552249085;
 pdfmellin_.A3UV_in[1] =    4.6295665919134663;
 pdfmellin_.A4UV_in[1] =    6.5339067518293525;
 pdfmellin_.A5UV_in[1] =    2.3094537585471713;
 pdfmellin_.A6UV_in[1] =   -6.8650155428908022;
 pdfmellin_.A7UV_in[1] =    3.5373262564872334;
 pdfmellin_.A8UV_in[1] =   0.44003299976876065;
 pdfmellin_.A1DV_in[1] =   0.45180043636065115;
 pdfmellin_.A2DV_in[1] =   0.30778249293637994;
 pdfmellin_.A3DV_in[1] =    5.1269875176610054;
 pdfmellin_.A4DV_in[1] =    4.5388858341096991;
 pdfmellin_.A5DV_in[1] =    1.7207775453772669;
 pdfmellin_.A6DV_in[1] =   -1.1066070667837522;
 pdfmellin_.A7DV_in[1] =   -15.459292857973693;
 pdfmellin_.A8DV_in[1] =    13.829497414820068;
 pdfmellin_.A1US_in[1] =    5.0001070523676409E-002 ;
 pdfmellin_.A2US_in[1] =  -0.48934276062732135;
 pdfmellin_.A3US_in[1] =    9.8338527529936801;
 pdfmellin_.A4US_in[1] =    12.479025857916724;
 pdfmellin_.A5US_in[1] =    1.2043543381251069;
 pdfmellin_.A6US_in[1] =   -49.595760121517543;
 pdfmellin_.A7US_in[1] =    38.784377334081057;
 pdfmellin_.A8US_in[1] =    67.295046289233824;
 pdfmellin_.A1DS_in[1] =   0.20540101272329231;
 pdfmellin_.A2DS_in[1] =  -0.28341443818231116;
 pdfmellin_.A3DS_in[1] =    15.688225888908205;
 pdfmellin_.A4DS_in[1] =    7.7361576407591670;
 pdfmellin_.A5DS_in[1] =   -2.0482768264573608;
 pdfmellin_.A6DS_in[1] =  -0.53321476702394932;
 pdfmellin_.A7DS_in[1] =    37.569803654900710;
 pdfmellin_.A8DS_in[1] =    33.488259074691825;
 pdfmellin_.A1SS_in[1] =    6.0545763299873517E-002 ;
 pdfmellin_.A2SS_in[1] =  -0.47059670785975188;
 pdfmellin_.A3SS_in[1] =    5.9607701310491592;
 pdfmellin_.A4SS_in[1] =    4.5961858111773388;
 pdfmellin_.A5SS_in[1] =   -1.7580425443532590;
 pdfmellin_.A6SS_in[1] =   -31.147033944527983;
 pdfmellin_.A7SS_in[1] =    83.820822283374525;
 pdfmellin_.A8SS_in[1] =   -71.289434785178358;
 pdfmellin_.A1GL_in[1] =   0.82625596933119516;
 pdfmellin_.A2GL_in[1] =  -0.52026942740606763;
 pdfmellin_.A3GL_in[1] =    5.9870109178471047;
 pdfmellin_.A4GL_in[1] =   -11.285074447643691;
 pdfmellin_.A5GL_in[1] =   0.18274855107974020;
 pdfmellin_.A6GL_in[1] =    31.761104771200053;
 pdfmellin_.A7GL_in[1] =   -34.819034360117563;
 pdfmellin_.A8GL_in[1] =    14.672231213086917;
 pdfmellin_.A1CH_in[1] =    7.2005833293381810E-002 ;
 pdfmellin_.A2CH_in[1] =  -0.42226793861628292;
 pdfmellin_.A3CH_in[1] =    9.7795767988052731;
 pdfmellin_.A4CH_in[1] =    10.755149939000525;
 pdfmellin_.A5CH_in[1] =   -2.9614526343429688;
 pdfmellin_.A6CH_in[1] =   -42.026435007310475;
 pdfmellin_.A7CH_in[1] =    88.751004679373153;
 pdfmellin_.A8CH_in[1] =   -60.750277683665203;
 pdfmellin_.A1BO_in[1] =    5.7318318755047830E-002 ;
 pdfmellin_.A2BO_in[1] =  -0.41753821790805318;
 pdfmellin_.A3BO_in[1] =    13.938480898132815;
 pdfmellin_.A4BO_in[1] =    14.651479661432091;
 pdfmellin_.A5BO_in[1] =   -4.3821415644777773;
 pdfmellin_.A6BO_in[1] =   -15.144889573554593;
 pdfmellin_.A7BO_in[1] =   -29.951154263638877;
 pdfmellin_.A8BO_in[1] =    84.832020154402926;
 pdfmellin_.A1UVp_in[1] =   0.51685244834104105;
 pdfmellin_.A2UVp_in[1] =   0.24996760215134528;
 pdfmellin_.A3UVp_in[1] =    5.5038112267316484;
 pdfmellin_.A4UVp_in[1] =    12.779693065231193;
 pdfmellin_.A5UVp_in[1] =    3.5269345867673181;
 pdfmellin_.A6UVp_in[1] =   -14.510447141449546;
 pdfmellin_.A7UVp_in[1] =    6.3229102197679623;
 pdfmellin_.A8UVp_in[1] =    25.846938552369238;
 pdfmellin_.A1DVp_in[1] =   0.61221307167317984;
 pdfmellin_.A2DVp_in[1] =   0.38866735920551077;
 pdfmellin_.A3DVp_in[1] =    6.6096758958491719;
 pdfmellin_.A4DVp_in[1] =    3.8895470653386539;
 pdfmellin_.A5DVp_in[1] =    1.9088277724394374;
 pdfmellin_.A6DVp_in[1] =   -2.6607750588581496;
 pdfmellin_.A7DVp_in[1] =   -3.4322532178971699;
 pdfmellin_.A8DVp_in[1] =    17.886094754201011;
 pdfmellin_.A1USp_in[1] =    9.0551853569609403E-002 ;
 pdfmellin_.A2USp_in[1] =  -0.39656993809137248;
 pdfmellin_.A3USp_in[1] =    7.7297798414345076;
 pdfmellin_.A4USp_in[1] =   -12.280922057256017;
 pdfmellin_.A5USp_in[1] =    1.0105636444104462;
 pdfmellin_.A6USp_in[1] =    46.694189531966117;
 pdfmellin_.A7USp_in[1] =   -86.344351584573303;
 pdfmellin_.A8USp_in[1] =    68.197920981679999;
 pdfmellin_.A1DSp_in[1] =   0.10361510564476581;
 pdfmellin_.A2DSp_in[1] =  -0.37719915223187717;
 pdfmellin_.A3DSp_in[1] =    14.802526890507330;
 pdfmellin_.A4DSp_in[1] =   -3.3553879705648768;
 pdfmellin_.A5DSp_in[1] =   0.92194077013741804;
 pdfmellin_.A6DSp_in[1] =    33.972775981573349;
 pdfmellin_.A7DSp_in[1] =    18.776599168648659;
 pdfmellin_.A8DSp_in[1] =    32.404593438454214;
 pdfmellin_.A1SSp_in[1] =    9.1713318307325778E-002 ;
 pdfmellin_.A2SSp_in[1] =  -0.39512828599398936;
 pdfmellin_.A3SSp_in[1] =    11.205725587520682;
 pdfmellin_.A4SSp_in[1] =   -16.861333243266550;
 pdfmellin_.A5SSp_in[1] =  -0.28703013520736675;
 pdfmellin_.A6SSp_in[1] =    72.097742779658233;
 pdfmellin_.A7SSp_in[1] =   -87.730915297977361;
 pdfmellin_.A8SSp_in[1] =    33.706764450336578;
 pdfmellin_.A1GLp_in[1] =    2.3868119781862136;
 pdfmellin_.A2GLp_in[1] =  -0.38665877477296029;
 pdfmellin_.A3GLp_in[1] =    9.0561357663846263;
 pdfmellin_.A4GLp_in[1] =    25.010556444938871;
 pdfmellin_.A5GLp_in[1] =   -5.7505012186605384;
 pdfmellin_.A6GLp_in[1] =   -60.172090717624343;
 pdfmellin_.A7GLp_in[1] =    70.938212362788761;
 pdfmellin_.A8GLp_in[1] =   -24.320585408658786;
 pdfmellin_.A1CHp_in[1] =   0.14176247358993510;
 pdfmellin_.A2CHp_in[1] =  -0.33522752368493841;
 pdfmellin_.A3CHp_in[1] =    15.217087972995246;
 pdfmellin_.A4CHp_in[1] =    30.060645672818900;
 pdfmellin_.A5CHp_in[1] =   -6.1098215127090043;
 pdfmellin_.A6CHp_in[1] =   -67.231811485572507;
 pdfmellin_.A7CHp_in[1] =    37.456808026689004;
 pdfmellin_.A8CHp_in[1] =    72.689216013356599;
 pdfmellin_.A1BOp_in[1] =    6.6952147915966376E-002 ;
 pdfmellin_.A2BOp_in[1] =  -0.39581598300179088;
 pdfmellin_.A3BOp_in[1] =    14.233534281718255;
 pdfmellin_.A4BOp_in[1] =    17.348514250482886;
 pdfmellin_.A5BOp_in[1] =   -4.8985613660023475;
 pdfmellin_.A6BOp_in[1] =   -22.473931889843598;
 pdfmellin_.A7BOp_in[1] =   -19.822213828672332;
 pdfmellin_.A8BOp_in[1] =    78.546652943311528;
 pdfmellin_.A1UV_in[2] =    1.2899654186846481;
 pdfmellin_.A2UV_in[2] =   0.38427813095022700;
 pdfmellin_.A3UV_in[2] =    4.8142780035678694;
 pdfmellin_.A4UV_in[2] =    5.0834092365064301;
 pdfmellin_.A5UV_in[2] =   0.46431208825390513;
 pdfmellin_.A6UV_in[2] =   -2.1887682992041277;
 pdfmellin_.A7UV_in[2] =   -5.0356330238049765;
 pdfmellin_.A8UV_in[2] =    6.5862017849834444;
 pdfmellin_.A1DV_in[2] =   0.65755171434834103;
 pdfmellin_.A2DV_in[2] =   0.36692763907862025;
 pdfmellin_.A3DV_in[2] =    4.9965210638698023;
 pdfmellin_.A4DV_in[2] =    4.4947061349220228;
 pdfmellin_.A5DV_in[2] =   0.48362766864251938;
 pdfmellin_.A6DV_in[2] =   -3.2356085430582482;
 pdfmellin_.A7DV_in[2] =   -9.3700906157165917;
 pdfmellin_.A8DV_in[2] =    9.3706840168242991;
 pdfmellin_.A1US_in[2] =   0.15941055291907674;
 pdfmellin_.A2US_in[2] =  -0.32826381243917824;
 pdfmellin_.A3US_in[2] =    9.5899447001247804;
 pdfmellin_.A4US_in[2] =    9.0840340174900795;
 pdfmellin_.A5US_in[2] =   -2.7966724052117655;
 pdfmellin_.A6US_in[2] =   -3.8348441100402368;
 pdfmellin_.A7US_in[2] =   -35.881872420869207;
 pdfmellin_.A8US_in[2] =    62.104213261966009;
 pdfmellin_.A1DS_in[2] =   0.33392275057137027;
 pdfmellin_.A2DS_in[2] =  -0.14759345980620697;
 pdfmellin_.A3DS_in[2] =    17.283414284331776;
 pdfmellin_.A4DS_in[2] =   -10.195312735229386;
 pdfmellin_.A5DS_in[2] =   0.34555442093645183;
 pdfmellin_.A6DS_in[2] =    38.502464064192964;
 pdfmellin_.A7DS_in[2] =   -1.4262352765298942;
 pdfmellin_.A8DS_in[2] =    88.091501181859883;
 pdfmellin_.A1SS_in[2] =    8.3575014946671994E-002 ;
 pdfmellin_.A2SS_in[2] =  -0.41871533157122359;
 pdfmellin_.A3SS_in[2] =    13.478229348879941;
 pdfmellin_.A4SS_in[2] =    4.2853333125714492;
 pdfmellin_.A5SS_in[2] =   -1.8421612409552068;
 pdfmellin_.A6SS_in[2] =    3.8546112397431926;
 pdfmellin_.A7SS_in[2] =   -27.992747265450049;
 pdfmellin_.A8SS_in[2] =    85.663409044904199;
 pdfmellin_.A1GL_in[2] =   0.68928847943155358;
 pdfmellin_.A2GL_in[2] =  -0.56215310374858385;
 pdfmellin_.A3GL_in[2] =    8.0487352592550963;
 pdfmellin_.A4GL_in[2] =  -0.29710583131302926;
 pdfmellin_.A5GL_in[2] =  -0.45401149145612862;
 pdfmellin_.A6GL_in[2] =   -1.1418421319627186;
 pdfmellin_.A7GL_in[2] =    3.3266117663770274;
 pdfmellin_.A8GL_in[2] =    9.8951025770590064;
 pdfmellin_.A1CH_in[2] =   0.13194897655047841;
 pdfmellin_.A2CH_in[2] =  -0.34002191956550532;
 pdfmellin_.A3CH_in[2] =    11.583249235673760;
 pdfmellin_.A4CH_in[2] =    23.954589422795848;
 pdfmellin_.A5CH_in[2] =   -5.4806819275649801;
 pdfmellin_.A6CH_in[2] =   -64.298527221289646;
 pdfmellin_.A7CH_in[2] =    89.223020901126745;
 pdfmellin_.A8CH_in[2] =   -37.207528785951851;
 pdfmellin_.A1BO_in[2] =    4.1835773882232481E-002 ;
 pdfmellin_.A2BO_in[2] =  -0.44237396077255481;
 pdfmellin_.A3BO_in[2] =    12.867898960862986;
 pdfmellin_.A4BO_in[2] =    6.5940570239890341;
 pdfmellin_.A5BO_in[2] =   -2.2103040757816785;
 pdfmellin_.A6BO_in[2] =   -26.277601717356355;
 pdfmellin_.A7BO_in[2] =    58.255223900871101;
 pdfmellin_.A8BO_in[2] =   -18.786877109040020;
 pdfmellin_.A1UVp_in[2] =   0.57450065355042967;
 pdfmellin_.A2UVp_in[2] =   0.28165687444648668;
 pdfmellin_.A3UVp_in[2] =    4.7272155034606822;
 pdfmellin_.A4UVp_in[2] =    4.5647751239050436;
 pdfmellin_.A5UVp_in[2] =    4.6831350159729368;
 pdfmellin_.A6UVp_in[2] =   -13.261344153093006;
 pdfmellin_.A7UVp_in[2] =    27.539925197781258;
 pdfmellin_.A8UVp_in[2] =   -16.647264790569125;
 pdfmellin_.A1DVp_in[2] =   0.63758601610350907;
 pdfmellin_.A2DVp_in[2] =   0.39293264750677692;
 pdfmellin_.A3DVp_in[2] =    6.6889774279213423;
 pdfmellin_.A4DVp_in[2] =    4.0716007127975935;
 pdfmellin_.A5DVp_in[2] =    1.6851531043236392;
 pdfmellin_.A6DVp_in[2] =   -2.5990193117991822;
 pdfmellin_.A7DVp_in[2] =   -2.5707206515893688;
 pdfmellin_.A8DVp_in[2] =    17.066438831434965;
 pdfmellin_.A1USp_in[2] =   0.11944877227975309;
 pdfmellin_.A2USp_in[2] =  -0.36600584310836604;
 pdfmellin_.A3USp_in[2] =    6.1657674524997894;
 pdfmellin_.A4USp_in[2] =    1.5114830755940787;
 pdfmellin_.A5USp_in[2] =   -1.3790412816346063;
 pdfmellin_.A6USp_in[2] =   -4.3074716577522025;
 pdfmellin_.A7USp_in[2] =    5.2650206495300864;
 pdfmellin_.A8USp_in[2] =   0.91217206994252820;
 pdfmellin_.A1DSp_in[2] =   0.15779770875508328;
 pdfmellin_.A2DSp_in[2] =  -0.33203236553008381;
 pdfmellin_.A3DSp_in[2] =    5.4482720376473628;
 pdfmellin_.A4DSp_in[2] =    9.2079477859223360;
 pdfmellin_.A5DSp_in[2] =   -2.7651675516581236;
 pdfmellin_.A6DSp_in[2] =   -12.446399742719635;
 pdfmellin_.A7DSp_in[2] =   -5.9819676703778697;
 pdfmellin_.A8DSp_in[2] =    11.415550973444397;
 pdfmellin_.A1SSp_in[2] =   0.18732616070061009;
 pdfmellin_.A2SSp_in[2] =  -0.31521052081729234;
 pdfmellin_.A3SSp_in[2] =    14.078826382256949;
 pdfmellin_.A4SSp_in[2] =    28.156110034361980;
 pdfmellin_.A5SSp_in[2] =   -5.9546048657313975;
 pdfmellin_.A6SSp_in[2] =   -69.380658791672516;
 pdfmellin_.A7SSp_in[2] =    85.883388436943108;
 pdfmellin_.A8SSp_in[2] =   -5.9153913144382528;
 pdfmellin_.A1GLp_in[2] =    2.3713388969539380;
 pdfmellin_.A2GLp_in[2] =  -0.38825565317695832;
 pdfmellin_.A3GLp_in[2] =    9.5731698513077497;
 pdfmellin_.A4GLp_in[2] =    25.869177345914125;
 pdfmellin_.A5GLp_in[2] =   -5.7961673111362515;
 pdfmellin_.A6GLp_in[2] =   -61.973868043524192;
 pdfmellin_.A7GLp_in[2] =    70.067036681708458;
 pdfmellin_.A8GLp_in[2] =   -18.568278273642946;
 pdfmellin_.A1CHp_in[2] =    8.2411409614327580E-002 ;
 pdfmellin_.A2CHp_in[2] =  -0.39765529790153503;
 pdfmellin_.A3CHp_in[2] =    11.389012185276908;
 pdfmellin_.A4CHp_in[2] =    5.2866640608639059;
 pdfmellin_.A5CHp_in[2] =   -2.5362684495202288;
 pdfmellin_.A6CHp_in[2] =   -21.779402980188408;
 pdfmellin_.A7CHp_in[2] =    71.647008712354790;
 pdfmellin_.A8CHp_in[2] =   -63.276943581480545;
 pdfmellin_.A1BOp_in[2] =    6.4265929882696499E-002 ;
 pdfmellin_.A2BOp_in[2] =  -0.40001824612892217;
 pdfmellin_.A3BOp_in[2] =    13.237041194393649;
 pdfmellin_.A4BOp_in[2] =    11.733061840345595;
 pdfmellin_.A5BOp_in[2] =   -4.4553252512456609;
 pdfmellin_.A6BOp_in[2] =  -0.18323690630180067;
 pdfmellin_.A7BOp_in[2] =   -53.190834674970979;
 pdfmellin_.A8BOp_in[2] =    84.416738488996060;
 pdfmellin_.A1UV_in[3] =    1.1223773183455772;
 pdfmellin_.A2UV_in[3] =   0.34330462864301126;
 pdfmellin_.A3UV_in[3] =    4.7314665292517617;
 pdfmellin_.A4UV_in[3] =    5.7855824714827762;
 pdfmellin_.A5UV_in[3] =   0.35703647865567945;
 pdfmellin_.A6UV_in[3] =  -0.48612055622362504;
 pdfmellin_.A7UV_in[3] =   -9.3562723994230765;
 pdfmellin_.A8UV_in[3] =    8.6284929094567246;
 pdfmellin_.A1DV_in[3] =   0.72591864865854272;
 pdfmellin_.A2DV_in[3] =   0.39996965099153026;
 pdfmellin_.A3DV_in[3] =    5.1016775083349204;
 pdfmellin_.A4DV_in[3] =    4.1183500458449949;
 pdfmellin_.A5DV_in[3] =   0.52151800196992326;
 pdfmellin_.A6DV_in[3] =   -3.4064311696132563;
 pdfmellin_.A7DV_in[3] =   -7.9747688975980537;
 pdfmellin_.A8DV_in[3] =    8.5552432092028852;
 pdfmellin_.A1US_in[3] =   0.35685901674069043;
 pdfmellin_.A2US_in[3] =  -0.24692903940896427;
 pdfmellin_.A3US_in[3] =    9.6758916977736558;
 pdfmellin_.A4US_in[3] =    13.130719772638251;
 pdfmellin_.A5US_in[3] =   -5.2877734462544126;
 pdfmellin_.A6US_in[3] =    6.4610721835600913;
 pdfmellin_.A7US_in[3] =   -67.535239341888072;
 pdfmellin_.A8US_in[3] =    74.477932469473927;
 pdfmellin_.A1DS_in[3] =   0.43172715105593773;
 pdfmellin_.A2DS_in[3] =   -9.9218843083924702E-003 ;
 pdfmellin_.A3DS_in[3] =    18.283271479202561;
 pdfmellin_.A4DS_in[3] =   -11.068894167726004;
 pdfmellin_.A5DS_in[3] =    2.5669598769169468;
 pdfmellin_.A6DS_in[3] =    5.6691100800478011E-002 ;
 pdfmellin_.A7DS_in[3] =    86.292950896976492;
 pdfmellin_.A8DS_in[3] =    87.650788982567917;
 pdfmellin_.A1SS_in[3] =   0.48948779880194937;
 pdfmellin_.A2SS_in[3] =   0.20691712109406635;
 pdfmellin_.A3SS_in[3] =    17.432055517006631;
 pdfmellin_.A4SS_in[3] =   -18.473849137933378;
 pdfmellin_.A5SS_in[3] =    4.6664036366017427;
 pdfmellin_.A6SS_in[3] =   -19.517758727048900;
 pdfmellin_.A7SS_in[3] =    77.876269912577726;
 pdfmellin_.A8SS_in[3] =    72.341819996487544;
 pdfmellin_.A1GL_in[3] =   0.35324201223387625;
 pdfmellin_.A2GL_in[3] =  -0.68367841226115322;
 pdfmellin_.A3GL_in[3] =    7.9351086119097864;
 pdfmellin_.A4GL_in[3] =   -1.1904842377175520;
 pdfmellin_.A5GL_in[3] =    1.2088500020878499;
 pdfmellin_.A6GL_in[3] =   -2.8112782121114748;
 pdfmellin_.A7GL_in[3] =  -0.71784690634507342;
 pdfmellin_.A8GL_in[3] =    25.539852931582431;
 pdfmellin_.A1CH_in[3] =   0.48123847484279852;
 pdfmellin_.A2CH_in[3] =    8.7868022952396374E-002 ;
 pdfmellin_.A3CH_in[3] =    14.898567314589089;
 pdfmellin_.A4CH_in[3] =   0.68461346068585271;
 pdfmellin_.A5CH_in[3] =   -2.2339859487616138;
 pdfmellin_.A6CH_in[3] =   -1.8149448622129682;
 pdfmellin_.A7CH_in[3] =    4.0689632503227102;
 pdfmellin_.A8CH_in[3] =    23.513474641428360;
 pdfmellin_.A1BO_in[3] =   0.40417097720832035;
 pdfmellin_.A2BO_in[3] =   0.24949956183679300;
 pdfmellin_.A3BO_in[3] =    19.118807908443998;
 pdfmellin_.A4BO_in[3] =   0.46568754313604188;
 pdfmellin_.A5BO_in[3] =   0.24288364603947343;
 pdfmellin_.A6BO_in[3] =   -49.076854779413907;
 pdfmellin_.A7BO_in[3] =    83.345864987227401;
 pdfmellin_.A8BO_in[3] =    61.779251165304856;
 pdfmellin_.A1UVp_in[3] =   0.61859216591310839;
 pdfmellin_.A2UVp_in[3] =   0.28097923019224225;
 pdfmellin_.A3UVp_in[3] =    4.6958301322922038;
 pdfmellin_.A4UVp_in[3] =    6.6819599346342073;
 pdfmellin_.A5UVp_in[3] =    3.4870935436515254;
 pdfmellin_.A6UVp_in[3] =   -11.577203606521721;
 pdfmellin_.A7UVp_in[3] =    14.599557874147735;
 pdfmellin_.A8UVp_in[3] =   -5.7088981288157896;
 pdfmellin_.A1DVp_in[3] =   0.63275648736883416;
 pdfmellin_.A2DVp_in[3] =   0.39098185317811440;
 pdfmellin_.A3DVp_in[3] =    6.7302683525479239;
 pdfmellin_.A4DVp_in[3] =    4.0691171708794123;
 pdfmellin_.A5DVp_in[3] =    1.6964298410317440;
 pdfmellin_.A6DVp_in[3] =   -2.3982380140101469;
 pdfmellin_.A7DVp_in[3] =   -2.2449545959599533;
 pdfmellin_.A8DVp_in[3] =    16.983459216362888;
 pdfmellin_.A1USp_in[3] =   0.11809450408378608;
 pdfmellin_.A2USp_in[3] =  -0.36732513776499059;
 pdfmellin_.A3USp_in[3] =    7.1076357311799301;
 pdfmellin_.A4USp_in[3] =    2.9098931496500668;
 pdfmellin_.A5USp_in[3] =   -1.3558043315952588;
 pdfmellin_.A6USp_in[3] =   -8.2295606428051684;
 pdfmellin_.A7USp_in[3] =    10.025193607483899;
 pdfmellin_.A8USp_in[3] =   0.98699665621893473;
 pdfmellin_.A1DSp_in[3] =   0.12178466334012099;
 pdfmellin_.A2DSp_in[3] =  -0.36149270784288634;
 pdfmellin_.A3DSp_in[3] =    8.4099010617486094;
 pdfmellin_.A4DSp_in[3] =    1.4927475908665571;
 pdfmellin_.A5DSp_in[3] =  -0.83246890977913779;
 pdfmellin_.A6DSp_in[3] =    5.5855209415199596;
 pdfmellin_.A7DSp_in[3] =    11.325965355412421;
 pdfmellin_.A8DSp_in[3] =   -43.542530896406078;
 pdfmellin_.A1SSp_in[3] =   0.20961045462654909;
 pdfmellin_.A2SSp_in[3] =  -0.30272512067502505;
 pdfmellin_.A3SSp_in[3] =    15.841503780368635;
 pdfmellin_.A4SSp_in[3] =    33.762027464284685;
 pdfmellin_.A5SSp_in[3] =   -6.6821459435303012;
 pdfmellin_.A6SSp_in[3] =   -74.450712290606191;
 pdfmellin_.A7SSp_in[3] =    44.178221992152430;
 pdfmellin_.A8SSp_in[3] =    89.892955024066652;
 pdfmellin_.A1GLp_in[3] =    2.2213492739429892;
 pdfmellin_.A2GLp_in[3] =  -0.39509810395297718;
 pdfmellin_.A3GLp_in[3] =    11.071981660201056;
 pdfmellin_.A4GLp_in[3] =    19.034001162405644;
 pdfmellin_.A5GLp_in[3] =   -5.0987842448834897;
 pdfmellin_.A6GLp_in[3] =   -23.076443026897820;
 pdfmellin_.A7GLp_in[3] =   -25.928765555585073;
 pdfmellin_.A8GLp_in[3] =    74.244344627633041;
 pdfmellin_.A1CHp_in[3] =    6.5801183239976427E-002 ;
 pdfmellin_.A2CHp_in[3] =  -0.41913130520115699;
 pdfmellin_.A3CHp_in[3] =    10.387351455139697;
 pdfmellin_.A4CHp_in[3] =   -19.990993441067047;
 pdfmellin_.A5CHp_in[3] =   0.22952292200550772;
 pdfmellin_.A6CHp_in[3] =    73.056684338566328;
 pdfmellin_.A7CHp_in[3] =   -89.966394426215601;
 pdfmellin_.A8CHp_in[3] =    35.732463069829592;
 pdfmellin_.A1BOp_in[3] =    6.5900644612779585E-002 ;
 pdfmellin_.A2BOp_in[3] =  -0.39828521681909229;
 pdfmellin_.A3BOp_in[3] =    17.360533223241362;
 pdfmellin_.A4BOp_in[3] =    24.707764272171694;
 pdfmellin_.A5BOp_in[3] =   -5.0878993490876994;
 pdfmellin_.A6BOp_in[3] =   -59.897987971653841;
 pdfmellin_.A7BOp_in[3] =    56.199021083576099;
 pdfmellin_.A8BOp_in[3] =    67.381062940550436;
 pdfmellin_.A1UV_in[4] =    1.3222599130343413;
 pdfmellin_.A2UV_in[4] =   0.39652010343379840;
 pdfmellin_.A3UV_in[4] =    4.4614557553113308;
 pdfmellin_.A4UV_in[4] =    4.2562465436452293;
 pdfmellin_.A5UV_in[4] =   0.59340432576595958;
 pdfmellin_.A6UV_in[4] =   -2.8719944206652031;
 pdfmellin_.A7UV_in[4] =   -3.0718117806949152;
 pdfmellin_.A8UV_in[4] =    3.0522487563984839;
 pdfmellin_.A1DV_in[4] =   0.79445368222792057;
 pdfmellin_.A2DV_in[4] =   0.50998728964779350;
 pdfmellin_.A3DV_in[4] =    5.2823968277729447;
 pdfmellin_.A4DV_in[4] =    3.9815896248014866;
 pdfmellin_.A5DV_in[4] =    1.6752751306510589;
 pdfmellin_.A6DV_in[4] =   -6.1035893806633510;
 pdfmellin_.A7DV_in[4] =   -11.319624754705160;
 pdfmellin_.A8DV_in[4] =    15.601118076691165;
 pdfmellin_.A1US_in[4] =   0.45439442507869221;
 pdfmellin_.A2US_in[4] =   0.55477240801689365;
 pdfmellin_.A3US_in[4] =    10.848625950417723;
 pdfmellin_.A4US_in[4] =   -19.945846072413215;
 pdfmellin_.A5US_in[4] =    12.525794931484455;
 pdfmellin_.A6US_in[4] =   -55.221439353968741;
 pdfmellin_.A7US_in[4] =    88.260548032846629;
 pdfmellin_.A8US_in[4] =    27.716370547755034;
 pdfmellin_.A1DS_in[4] =    7.3611881398835330E-002 ;
 pdfmellin_.A2DS_in[4] =   -8.3560297246592197E-002 ;
 pdfmellin_.A3DS_in[4] =    7.4581575127090582;
 pdfmellin_.A4DS_in[4] =   -12.902479025757934;
 pdfmellin_.A5DS_in[4] =    12.315637564816456;
 pdfmellin_.A6DS_in[4] =   -41.560166315713865;
 pdfmellin_.A7DS_in[4] =    78.061083760125683;
 pdfmellin_.A8DS_in[4] =   -56.848143267650997;
 pdfmellin_.A1SS_in[4] =   0.92390495241538506;
 pdfmellin_.A2SS_in[4] =   0.18450992140373068;
 pdfmellin_.A3SS_in[4] =    8.4413267353895360;
 pdfmellin_.A4SS_in[4] =    14.115545148646646;
 pdfmellin_.A5SS_in[4] =   -4.6732417516378177;
 pdfmellin_.A6SS_in[4] =   -44.586611948766226;
 pdfmellin_.A7SS_in[4] =    89.602941620735749;
 pdfmellin_.A8SS_in[4] =   -66.615937888180412;
 pdfmellin_.A1GL_in[4] =   0.65925682022707066;
 pdfmellin_.A2GL_in[4] =  -0.52293354536244352;
 pdfmellin_.A3GL_in[4] =    8.2507347505423851;
 pdfmellin_.A4GL_in[4] =   -2.1372125975393139;
 pdfmellin_.A5GL_in[4] =   0.56770263801271881;
 pdfmellin_.A6GL_in[4] =   -1.3221981519935468;
 pdfmellin_.A7GL_in[4] =    2.1653020805604797;
 pdfmellin_.A8GL_in[4] =    16.920478330977843;
 pdfmellin_.A1CH_in[4] =   0.63911901153025497;
 pdfmellin_.A2CH_in[4] =   0.32820193072571602;
 pdfmellin_.A3CH_in[4] =    13.439113914913287;
 pdfmellin_.A4CH_in[4] =   0.67303147187832835;
 pdfmellin_.A5CH_in[4] =   -1.8467150081113104;
 pdfmellin_.A6CH_in[4] =   -2.7347120617359622;
 pdfmellin_.A7CH_in[4] =    1.5745261551205658;
 pdfmellin_.A8CH_in[4] =    17.184556623209872;
 pdfmellin_.A1BO_in[4] =   0.50797287164813254;
 pdfmellin_.A2BO_in[4] =    1.0918943613163172;
 pdfmellin_.A3BO_in[4] =    20.905633797254577;
 pdfmellin_.A4BO_in[4] =    2.4354253936017685;
 pdfmellin_.A5BO_in[4] =   0.81285783072733864;
 pdfmellin_.A6BO_in[4] =    43.877885710692738;
 pdfmellin_.A7BO_in[4] =    8.3963736267461542;
 pdfmellin_.A8BO_in[4] =    89.928488413781537;
 pdfmellin_.A1UVp_in[4] =   0.66132868045082427;
 pdfmellin_.A2UVp_in[4] =   0.28891255910238689;
 pdfmellin_.A3UVp_in[4] =    4.7742301245272927;
 pdfmellin_.A4UVp_in[4] =    6.1271638905535326;
 pdfmellin_.A5UVp_in[4] =    3.1377040989818070;
 pdfmellin_.A6UVp_in[4] =   -7.5729723314940145;
 pdfmellin_.A7UVp_in[4] =    6.5507764669784194;
 pdfmellin_.A8UVp_in[4] =   0.49347889657130395;
 pdfmellin_.A1DVp_in[4] =   0.63874212694063248;
 pdfmellin_.A2DVp_in[4] =   0.39283352148177819;
 pdfmellin_.A3DVp_in[4] =    6.7258258390570154;
 pdfmellin_.A4DVp_in[4] =    4.1447371477965582;
 pdfmellin_.A5DVp_in[4] =    1.6661408408769134;
 pdfmellin_.A6DVp_in[4] =   -2.6170459814664184;
 pdfmellin_.A7DVp_in[4] =   -2.4399766542318133;
 pdfmellin_.A8DVp_in[4] =    17.307137190155686;
 pdfmellin_.A1USp_in[4] =   0.11580589772622425;
 pdfmellin_.A2USp_in[4] =  -0.36884036663038400;
 pdfmellin_.A3USp_in[4] =    7.1953718628111565;
 pdfmellin_.A4USp_in[4] =    1.4889982144061094;
 pdfmellin_.A5USp_in[4] =   -1.1235428609225480;
 pdfmellin_.A6USp_in[4] =   -4.2805131298788837;
 pdfmellin_.A7USp_in[4] =    5.4697863197540926;
 pdfmellin_.A8USp_in[4] =    2.9770784046838372;
 pdfmellin_.A1DSp_in[4] =   0.12780494164652725;
 pdfmellin_.A2DSp_in[4] =  -0.35593515323844155;
 pdfmellin_.A3DSp_in[4] =    8.4860326069560070;
 pdfmellin_.A4DSp_in[4] =    3.6412188100275138;
 pdfmellin_.A5DSp_in[4] =   -1.1972735471567972;
 pdfmellin_.A6DSp_in[4] =  -0.90586072498176406;
 pdfmellin_.A7DSp_in[4] =    20.699626763114779;
 pdfmellin_.A8DSp_in[4] =   -48.251892426232139;
 pdfmellin_.A1SSp_in[4] =   0.17708766093296663;
 pdfmellin_.A2SSp_in[4] =  -0.32204191629679452;
 pdfmellin_.A3SSp_in[4] =    15.147127183361503;
 pdfmellin_.A4SSp_in[4] =    25.874762907577797;
 pdfmellin_.A5SSp_in[4] =   -5.6150668074117291;
 pdfmellin_.A6SSp_in[4] =   -53.371933366427740;
 pdfmellin_.A7SSp_in[4] =    31.582181033831446;
 pdfmellin_.A8SSp_in[4] =    72.443192431323354;
 pdfmellin_.A1GLp_in[4] =    2.3797502534945596;
 pdfmellin_.A2GLp_in[4] =  -0.38786169206755144;
 pdfmellin_.A3GLp_in[4] =    9.9162753267187966;
 pdfmellin_.A4GLp_in[4] =    25.951969765317187;
 pdfmellin_.A5GLp_in[4] =   -5.7989158416004614;
 pdfmellin_.A6GLp_in[4] =   -60.944452129923235;
 pdfmellin_.A7GLp_in[4] =    65.441635998729481;
 pdfmellin_.A8GLp_in[4] =   -11.923848855769592;
 pdfmellin_.A1CHp_in[4] =    9.3127921658263557E-002 ;
 pdfmellin_.A2CHp_in[4] =  -0.38058475743032821;
 pdfmellin_.A3CHp_in[4] =    9.5001184399522387;
 pdfmellin_.A4CHp_in[4] =   -3.6358595203072142;
 pdfmellin_.A5CHp_in[4] =   -2.5765929335140427;
 pdfmellin_.A6CHp_in[4] =    39.954944420669591;
 pdfmellin_.A7CHp_in[4] =   -87.280822088260763;
 pdfmellin_.A8CHp_in[4] =    67.009595366277239;
 pdfmellin_.A1BOp_in[4] =    6.5704196959312303E-002 ;
 pdfmellin_.A2BOp_in[4] =  -0.39688027180260982;
 pdfmellin_.A3BOp_in[4] =    12.478233013368252;
 pdfmellin_.A4BOp_in[4] =    11.510377582634039;
 pdfmellin_.A5BOp_in[4] =   -4.5247514603797896;
 pdfmellin_.A6BOp_in[4] =  -0.73106557855928145;
 pdfmellin_.A7BOp_in[4] =   -46.949856484960975;
 pdfmellin_.A8BOp_in[4] =    70.253455732370213;
 pdfmellin_.A1UV_in[5] =    1.4434973729408163;
 pdfmellin_.A2UV_in[5] =   0.41453025234870844;
 pdfmellin_.A3UV_in[5] =    4.2884359836064334;
 pdfmellin_.A4UV_in[5] =    4.3049252287324116;
 pdfmellin_.A5UV_in[5] =   0.26933431115236317;
 pdfmellin_.A6UV_in[5] =   -2.5665298550234041;
 pdfmellin_.A7UV_in[5] =   -4.9942691869670597;
 pdfmellin_.A8UV_in[5] =    3.9905463894171334;
 pdfmellin_.A1DV_in[5] =   0.72369627323952934;
 pdfmellin_.A2DV_in[5] =   0.52895223146457337;
 pdfmellin_.A3DV_in[5] =    4.6352786479669454;
 pdfmellin_.A4DV_in[5] =    3.3995418062388154;
 pdfmellin_.A5DV_in[5] =    2.1317062063439636;
 pdfmellin_.A6DV_in[5] =   -7.5757189545501262;
 pdfmellin_.A7DV_in[5] =   -11.625882873826328;
 pdfmellin_.A8DV_in[5] =    15.559529151276692;
 pdfmellin_.A1US_in[5] =   0.95840329790228140;
 pdfmellin_.A2US_in[5] =   0.91953371609179890;
 pdfmellin_.A3US_in[5] =    10.008828737354523;
 pdfmellin_.A4US_in[5] =   -11.282314378322955;
 pdfmellin_.A5US_in[5] =    7.1902333830433633;
 pdfmellin_.A6US_in[5] =   -28.077001542727338;
 pdfmellin_.A7US_in[5] =    37.926558917524218;
 pdfmellin_.A8US_in[5] =    16.307367679100381;
 pdfmellin_.A1DS_in[5] =   0.22339775680212481;
 pdfmellin_.A2DS_in[5] =   0.20118795311920024;
 pdfmellin_.A3DS_in[5] =    7.7312014652499830;
 pdfmellin_.A4DS_in[5] =  -0.17527624770551498;
 pdfmellin_.A5DS_in[5] =    3.8822120205256709;
 pdfmellin_.A6DS_in[5] =   -23.321620685576832;
 pdfmellin_.A7DS_in[5] =    21.049951739149037;
 pdfmellin_.A8DS_in[5] =   -9.1582710048185021;
 pdfmellin_.A1SS_in[5] =   0.65086575454148299;
 pdfmellin_.A2SS_in[5] =    1.4894465309754186;
 pdfmellin_.A3SS_in[5] =    9.2666108994158840;
 pdfmellin_.A4SS_in[5] =    5.8100474664805581;
 pdfmellin_.A5SS_in[5] =    1.2563272356966664;
 pdfmellin_.A6SS_in[5] =    3.5418698220375404;
 pdfmellin_.A7SS_in[5] =   -5.8022966341391822;
 pdfmellin_.A8SS_in[5] =   -39.931753485402197;
 pdfmellin_.A1GL_in[5] =   0.65263542582955336;
 pdfmellin_.A2GL_in[5] =  -0.54561949426187817;
 pdfmellin_.A3GL_in[5] =    8.2196309898019919;
 pdfmellin_.A4GL_in[5] =   -2.1911814964873848;
 pdfmellin_.A5GL_in[5] =   0.41301974749995907;
 pdfmellin_.A6GL_in[5] =  -0.91000629166325098;
 pdfmellin_.A7GL_in[5] =    3.4700999991042067;
 pdfmellin_.A8GL_in[5] =    14.499818868773346;
 pdfmellin_.A1CH_in[5] =   0.61191580765090370;
 pdfmellin_.A2CH_in[5] =   0.94883445898358509;
 pdfmellin_.A3CH_in[5] =    12.064231965757651;
 pdfmellin_.A4CH_in[5] =    2.7275773385230870;
 pdfmellin_.A5CH_in[5] =   -1.1317563776545327;
 pdfmellin_.A6CH_in[5] =  -0.40107767211124212;
 pdfmellin_.A7CH_in[5] =  -0.31189063024302754;
 pdfmellin_.A8CH_in[5] =   0.38967986416075462;
 pdfmellin_.A1BO_in[5] =   0.37020619055607762;
 pdfmellin_.A2BO_in[5] =    1.1865671226820154;
 pdfmellin_.A3BO_in[5] =    17.931287140542764;
 pdfmellin_.A4BO_in[5] =   -6.4481321328707377;
 pdfmellin_.A5BO_in[5] =   0.15181524000909263;
 pdfmellin_.A6BO_in[5] =    16.218985881319369;
 pdfmellin_.A7BO_in[5] =    71.670568714804119;
 pdfmellin_.A8BO_in[5] =    56.226644614436196;
 pdfmellin_.A1UVp_in[5] =   0.66050917905738515;
 pdfmellin_.A2UVp_in[5] =   0.28845531747310083;
 pdfmellin_.A3UVp_in[5] =    4.7813652435186631;
 pdfmellin_.A4UVp_in[5] =    6.1819260589626595;
 pdfmellin_.A5UVp_in[5] =    3.1253336645390064;
 pdfmellin_.A6UVp_in[5] =   -7.5077724975485722;
 pdfmellin_.A7UVp_in[5] =    6.2912145428107067;
 pdfmellin_.A8UVp_in[5] =   0.85230391006659545;
 pdfmellin_.A1DVp_in[5] =   0.62261347620667307;
 pdfmellin_.A2DVp_in[5] =   0.38830743520250754;
 pdfmellin_.A3DVp_in[5] =    6.7522240475246544;
 pdfmellin_.A4DVp_in[5] =    4.2584837050887607;
 pdfmellin_.A5DVp_in[5] =    1.7304573848926381;
 pdfmellin_.A6DVp_in[5] =   -2.6324915897797467;
 pdfmellin_.A7DVp_in[5] =   -2.5269478502486322;
 pdfmellin_.A8DVp_in[5] =    18.168797878749871;
 pdfmellin_.A1USp_in[5] =   0.11756882740680168;
 pdfmellin_.A2USp_in[5] =  -0.36645714129015033;
 pdfmellin_.A3USp_in[5] =    9.2909481294507117;
 pdfmellin_.A4USp_in[5] =    3.4683072855916208E-002 ;
 pdfmellin_.A5USp_in[5] =   -1.0026885103037273;
 pdfmellin_.A6USp_in[5] =    19.204557588156163;
 pdfmellin_.A7USp_in[5] =   -64.179107743928810;
 pdfmellin_.A8USp_in[5] =    76.481210874824427;
 pdfmellin_.A1DSp_in[5] =   0.11458702523507418;
 pdfmellin_.A2DSp_in[5] =  -0.36696153990644109;
 pdfmellin_.A3DSp_in[5] =    14.876500747449763;
 pdfmellin_.A4DSp_in[5] =    2.7292320811558248;
 pdfmellin_.A5DSp_in[5] =   -3.7236492897712026E-002 ;
 pdfmellin_.A6DSp_in[5] =    10.364584164134939;
 pdfmellin_.A7DSp_in[5] =    55.140421121374942;
 pdfmellin_.A8DSp_in[5] =    8.6684816075191975;
 pdfmellin_.A1SSp_in[5] =   0.19205898596450596;
 pdfmellin_.A2SSp_in[5] =  -0.31341449558811019;
 pdfmellin_.A3SSp_in[5] =    14.754769080179059;
 pdfmellin_.A4SSp_in[5] =    32.089154516776752;
 pdfmellin_.A5SSp_in[5] =   -6.3212298435934144;
 pdfmellin_.A6SSp_in[5] =   -80.575851721957761;
 pdfmellin_.A7SSp_in[5] =    88.768522270814771;
 pdfmellin_.A8SSp_in[5] =    17.379850258735559;
 pdfmellin_.A1GLp_in[5] =    2.3127868146197135;
 pdfmellin_.A2GLp_in[5] =  -0.39043428219487397;
 pdfmellin_.A3GLp_in[5] =    11.514486108691226;
 pdfmellin_.A4GLp_in[5] =    21.073134561242249;
 pdfmellin_.A5GLp_in[5] =   -5.3521767531480933;
 pdfmellin_.A6GLp_in[5] =   -28.062311912302810;
 pdfmellin_.A7GLp_in[5] =   -23.896019156346199;
 pdfmellin_.A8GLp_in[5] =    81.312466965900711;
 pdfmellin_.A1CHp_in[5] =   0.12589398778294347;
 pdfmellin_.A2CHp_in[5] =  -0.34847785552321264;
 pdfmellin_.A3CHp_in[5] =    13.541452117339096;
 pdfmellin_.A4CHp_in[5] =    20.938113000139488;
 pdfmellin_.A5CHp_in[5] =   -5.1916016081460299;
 pdfmellin_.A6CHp_in[5] =   -36.882321169913297;
 pdfmellin_.A7CHp_in[5] =    2.2626730782502449;
 pdfmellin_.A8CHp_in[5] =    63.351608961693984;
 pdfmellin_.A1BOp_in[5] =    6.7379127563602287E-002 ;
 pdfmellin_.A2BOp_in[5] =  -0.39365466257835841;
 pdfmellin_.A3BOp_in[5] =    13.054670004067015;
 pdfmellin_.A4BOp_in[5] =    12.005738660368657;
 pdfmellin_.A5BOp_in[5] =   -4.6066427653645263;
 pdfmellin_.A6BOp_in[5] =   0.15070762100279467;
 pdfmellin_.A7BOp_in[5] =   -53.316584691945295;
 pdfmellin_.A8BOp_in[5] =    80.204711520947683;
 pdfmellin_.A1UV_in[6] =    1.4434973729408163;
 pdfmellin_.A2UV_in[6] =   0.41453025234870844;
 pdfmellin_.A3UV_in[6] =    4.2884359836064334;
 pdfmellin_.A4UV_in[6] =    4.3049252287324116;
 pdfmellin_.A5UV_in[6] =   0.26933431115236317;
 pdfmellin_.A6UV_in[6] =   -2.5665298550234041;
 pdfmellin_.A7UV_in[6] =   -4.9942691869670597;
 pdfmellin_.A8UV_in[6] =    3.9905463894171334;
 pdfmellin_.A1DV_in[6] =   0.72369627323952934;
 pdfmellin_.A2DV_in[6] =   0.52895223146457337;
 pdfmellin_.A3DV_in[6] =    4.6352786479669454;
 pdfmellin_.A4DV_in[6] =    3.3995418062388154;
 pdfmellin_.A5DV_in[6] =    2.1317062063439636;
 pdfmellin_.A6DV_in[6] =   -7.5757189545501262;
 pdfmellin_.A7DV_in[6] =   -11.625882873826328;
 pdfmellin_.A8DV_in[6] =    15.559529151276692;
 pdfmellin_.A1US_in[6] =   0.95840329790228140;
 pdfmellin_.A2US_in[6] =   0.91953371609179890;
 pdfmellin_.A3US_in[6] =    10.008828737354523;
 pdfmellin_.A4US_in[6] =   -11.282314378322955;
 pdfmellin_.A5US_in[6] =    7.1902333830433633;
 pdfmellin_.A6US_in[6] =   -28.077001542727338;
 pdfmellin_.A7US_in[6] =    37.926558917524218;
 pdfmellin_.A8US_in[6] =    16.307367679100381;
 pdfmellin_.A1DS_in[6] =   0.22339775680212481;
 pdfmellin_.A2DS_in[6] =   0.20118795311920024;
 pdfmellin_.A3DS_in[6] =    7.7312014652499830;
 pdfmellin_.A4DS_in[6] =  -0.17527624770551498;
 pdfmellin_.A5DS_in[6] =    3.8822120205256709;
 pdfmellin_.A6DS_in[6] =   -23.321620685576832;
 pdfmellin_.A7DS_in[6] =    21.049951739149037;
 pdfmellin_.A8DS_in[6] =   -9.1582710048185021;
 pdfmellin_.A1SS_in[6] =   0.65086575454148299;
 pdfmellin_.A2SS_in[6] =    1.4894465309754186;
 pdfmellin_.A3SS_in[6] =    9.2666108994158840;
 pdfmellin_.A4SS_in[6] =    5.8100474664805581;
 pdfmellin_.A5SS_in[6] =    1.2563272356966664;
 pdfmellin_.A6SS_in[6] =    3.5418698220375404;
 pdfmellin_.A7SS_in[6] =   -5.8022966341391822;
 pdfmellin_.A8SS_in[6] =   -39.931753485402197;
 pdfmellin_.A1GL_in[6] =   0.65263542582955336;
 pdfmellin_.A2GL_in[6] =  -0.54561949426187817;
 pdfmellin_.A3GL_in[6] =    8.2196309898019919;
 pdfmellin_.A4GL_in[6] =   -2.1911814964873848;
 pdfmellin_.A5GL_in[6] =   0.41301974749995907;
 pdfmellin_.A6GL_in[6] =  -0.91000629166325098;
 pdfmellin_.A7GL_in[6] =    3.4700999991042067;
 pdfmellin_.A8GL_in[6] =    14.499818868773346;
 pdfmellin_.A1CH_in[6] =   0.61191580765090370;
 pdfmellin_.A2CH_in[6] =   0.94883445898358509;
 pdfmellin_.A3CH_in[6] =    12.064231965757651;
 pdfmellin_.A4CH_in[6] =    2.7275773385230870;
 pdfmellin_.A5CH_in[6] =   -1.1317563776545327;
 pdfmellin_.A6CH_in[6] =  -0.40107767211124212;
 pdfmellin_.A7CH_in[6] =  -0.31189063024302754;
 pdfmellin_.A8CH_in[6] =   0.38967986416075462;
 pdfmellin_.A1BO_in[6] =   0.37020619055607762;
 pdfmellin_.A2BO_in[6] =    1.1865671226820154;
 pdfmellin_.A3BO_in[6] =    17.931287140542764;
 pdfmellin_.A4BO_in[6] =   -6.4481321328707377;
 pdfmellin_.A5BO_in[6] =   0.15181524000909263;
 pdfmellin_.A6BO_in[6] =    16.218985881319369;
 pdfmellin_.A7BO_in[6] =    71.670568714804119;
 pdfmellin_.A8BO_in[6] =    56.226644614436196;
 pdfmellin_.A1UVp_in[6] =   0.66050917905738515;
 pdfmellin_.A2UVp_in[6] =   0.28845531747310083;
 pdfmellin_.A3UVp_in[6] =    4.7813652435186631;
 pdfmellin_.A4UVp_in[6] =    6.1819260589626595;
 pdfmellin_.A5UVp_in[6] =    3.1253336645390064;
 pdfmellin_.A6UVp_in[6] =   -7.5077724975485722;
 pdfmellin_.A7UVp_in[6] =    6.2912145428107067;
 pdfmellin_.A8UVp_in[6] =   0.85230391006659545;
 pdfmellin_.A1DVp_in[6] =   0.62261347620667307;
 pdfmellin_.A2DVp_in[6] =   0.38830743520250754;
 pdfmellin_.A3DVp_in[6] =    6.7522240475246544;
 pdfmellin_.A4DVp_in[6] =    4.2584837050887607;
 pdfmellin_.A5DVp_in[6] =    1.7304573848926381;
 pdfmellin_.A6DVp_in[6] =   -2.6324915897797467;
 pdfmellin_.A7DVp_in[6] =   -2.5269478502486322;
 pdfmellin_.A8DVp_in[6] =    18.168797878749871;
 pdfmellin_.A1USp_in[6] =   0.11756882740680168;
 pdfmellin_.A2USp_in[6] =  -0.36645714129015033;
 pdfmellin_.A3USp_in[6] =    9.2909481294507117;
 pdfmellin_.A4USp_in[6] =    3.4683072855916208E-002 ;
 pdfmellin_.A5USp_in[6] =   -1.0026885103037273;
 pdfmellin_.A6USp_in[6] =    19.204557588156163;
 pdfmellin_.A7USp_in[6] =   -64.179107743928810;
 pdfmellin_.A8USp_in[6] =    76.481210874824427;
 pdfmellin_.A1DSp_in[6] =   0.11458702523507418;
 pdfmellin_.A2DSp_in[6] =  -0.36696153990644109;
 pdfmellin_.A3DSp_in[6] =    14.876500747449763;
 pdfmellin_.A4DSp_in[6] =    2.7292320811558248;
 pdfmellin_.A5DSp_in[6] =   -3.7236492897712026E-002 ;
 pdfmellin_.A6DSp_in[6] =    10.364584164134939;
 pdfmellin_.A7DSp_in[6] =    55.140421121374942;
 pdfmellin_.A8DSp_in[6] =    8.6684816075191975;
 pdfmellin_.A1SSp_in[6] =   0.19205898596450596;
 pdfmellin_.A2SSp_in[6] =  -0.31341449558811019;
 pdfmellin_.A3SSp_in[6] =    14.754769080179059;
 pdfmellin_.A4SSp_in[6] =    32.089154516776752;
 pdfmellin_.A5SSp_in[6] =   -6.3212298435934144;
 pdfmellin_.A6SSp_in[6] =   -80.575851721957761;
 pdfmellin_.A7SSp_in[6] =    88.768522270814771;
 pdfmellin_.A8SSp_in[6] =    17.379850258735559;
 pdfmellin_.A1GLp_in[6] =    2.3127868146197135;
 pdfmellin_.A2GLp_in[6] =  -0.39043428219487397;
 pdfmellin_.A3GLp_in[6] =    11.514486108691226;
 pdfmellin_.A4GLp_in[6] =    21.073134561242249;
 pdfmellin_.A5GLp_in[6] =   -5.3521767531480933;
 pdfmellin_.A6GLp_in[6] =   -28.062311912302810;
 pdfmellin_.A7GLp_in[6] =   -23.896019156346199;
 pdfmellin_.A8GLp_in[6] =    81.312466965900711;
 pdfmellin_.A1CHp_in[6] =   0.12589398778294347;
 pdfmellin_.A2CHp_in[6] =  -0.34847785552321264;
 pdfmellin_.A3CHp_in[6] =    13.541452117339096;
 pdfmellin_.A4CHp_in[6] =    20.938113000139488;
 pdfmellin_.A5CHp_in[6] =   -5.1916016081460299;
 pdfmellin_.A6CHp_in[6] =   -36.882321169913297;
 pdfmellin_.A7CHp_in[6] =    2.2626730782502449;
 pdfmellin_.A8CHp_in[6] =    63.351608961693984;
 pdfmellin_.A1BOp_in[6] =    6.7379127563602287E-002 ;
 pdfmellin_.A2BOp_in[6] =  -0.39365466257835841;
 pdfmellin_.A3BOp_in[6] =    13.054670004067015;
 pdfmellin_.A4BOp_in[6] =    12.005738660368657;
 pdfmellin_.A5BOp_in[6] =   -4.6066427653645263;
 pdfmellin_.A6BOp_in[6] =   0.15070762100279467;
 pdfmellin_.A7BOp_in[6] =   -53.316584691945295;
 pdfmellin_.A8BOp_in[6] =    80.204711520947683;
 pdfmellin_.A1UV_in[7] =   0.49027644610728283;
 pdfmellin_.A2UV_in[7] =   0.23453214546659007;
 pdfmellin_.A3UV_in[7] =    5.3698156134531061;
 pdfmellin_.A4UV_in[7] =    13.839174277221019;
 pdfmellin_.A5UV_in[7] =    3.2660649803689950;
 pdfmellin_.A6UV_in[7] =   -13.295080289095679;
 pdfmellin_.A7UV_in[7] =   0.37532216188564860;
 pdfmellin_.A8UV_in[7] =    27.288921919385388;
 pdfmellin_.A1DV_in[7] =   0.60497816276916594;
 pdfmellin_.A2DV_in[7] =   0.38371539491749962;
 pdfmellin_.A3DV_in[7] =    6.2015915109354331;
 pdfmellin_.A4DV_in[7] =    3.8333700755124447;
 pdfmellin_.A5DV_in[7] =    1.7719406886700213;
 pdfmellin_.A6DV_in[7] =   -2.8937665785466464;
 pdfmellin_.A7DV_in[7] =   -4.3811125515175320;
 pdfmellin_.A8DV_in[7] =    13.179166754565287;
 pdfmellin_.A1US_in[7] =   0.18536143809274100;
 pdfmellin_.A2US_in[7] =  -0.31338099178510143;
 pdfmellin_.A3US_in[7] =    9.3593051078426566;
 pdfmellin_.A4US_in[7] =    20.940215979270903;
 pdfmellin_.A5US_in[7] =   -4.2675366908073968;
 pdfmellin_.A6US_in[7] =   -49.185592366887001;
 pdfmellin_.A7US_in[7] =    43.331810268021059;
 pdfmellin_.A8US_in[7] =    7.3823349080496143;
 pdfmellin_.A1DS_in[7] =   0.13600221827452896;
 pdfmellin_.A2DS_in[7] =  -0.34679998115317012;
 pdfmellin_.A3DS_in[7] =    15.879795019842440;
 pdfmellin_.A4DS_in[7] =    13.397796312675062;
 pdfmellin_.A5DS_in[7] =   -1.5301087100919801;
 pdfmellin_.A6DS_in[7] =   -19.272560145868269;
 pdfmellin_.A7DS_in[7] =    59.010204859396197;
 pdfmellin_.A8DS_in[7] =    89.983423616605876;
 pdfmellin_.A1SS_in[7] =   -3.5167476188920563E-003 ;
 pdfmellin_.A2SS_in[7] =  -0.79273982553475320;
 pdfmellin_.A3SS_in[7] =    10.090641566770206;
 pdfmellin_.A4SS_in[7] =    159.50147439813136;
 pdfmellin_.A5SS_in[7] =   -59.999997679373188;
 pdfmellin_.A6SS_in[7] =   -89.999842212718391;
 pdfmellin_.A7SS_in[7] =   -89.991387530096048;
 pdfmellin_.A8SS_in[7] =   -89.998227511676220;
 pdfmellin_.A1GL_in[7] =    2.0968962352673368;
 pdfmellin_.A2GL_in[7] =  -0.39960123338586678;
 pdfmellin_.A3GL_in[7] =    10.715826295517303;
 pdfmellin_.A4GL_in[7] =    13.541770491782422;
 pdfmellin_.A5GL_in[7] =   -4.4805582972049365;
 pdfmellin_.A6GL_in[7] =   -4.2525362253343673;
 pdfmellin_.A7GL_in[7] =   -52.790332146157951;
 pdfmellin_.A8GL_in[7] =    84.710071606860879;
 pdfmellin_.A1CH_in[7] =   0.11121294152183556;
 pdfmellin_.A2CH_in[7] =  -0.36380119128778232;
 pdfmellin_.A3CH_in[7] =    14.139680466442476;
 pdfmellin_.A4CH_in[7] =    18.994205188221322;
 pdfmellin_.A5CH_in[7] =   -4.6218095655184115;
 pdfmellin_.A6CH_in[7] =   -35.323773849036293;
 pdfmellin_.A7CH_in[7] =    6.6273644414921904;
 pdfmellin_.A8CH_in[7] =    65.247129231919956;
 pdfmellin_.A1BO_in[7] =    7.5513975335841893E-002 ;
 pdfmellin_.A2BO_in[7] =  -0.37129459279105043;
 pdfmellin_.A3BO_in[7] =    24.977057235923535;
 pdfmellin_.A4BO_in[7] =    7.5932390850366005;
 pdfmellin_.A5BO_in[7] =   -3.6979995498561209;
 pdfmellin_.A6BO_in[7] =    21.038655906056604;
 pdfmellin_.A7BO_in[7] =    25.890278837151953;
 pdfmellin_.A8BO_in[7] =    18.115343529128197;
 pdfmellin_.A1UVp_in[7] =   0.53182008693990623;
 pdfmellin_.A2UVp_in[7] =   0.25385510801108202;
 pdfmellin_.A3UVp_in[7] =    4.5598731752055262;
 pdfmellin_.A4UVp_in[7] =    7.5443035002262633;
 pdfmellin_.A5UVp_in[7] =    3.9607612507892753;
 pdfmellin_.A6UVp_in[7] =   -13.386861890746502;
 pdfmellin_.A7UVp_in[7] =    17.089350998276430;
 pdfmellin_.A8UVp_in[7] =   -8.7769116613552427;
 pdfmellin_.A1DVp_in[7] =   0.43077675581514940;
 pdfmellin_.A2DVp_in[7] =   0.34109614609453964;
 pdfmellin_.A3DVp_in[7] =    4.8210777290567783;
 pdfmellin_.A4DVp_in[7] =   -2.3807064538577194;
 pdfmellin_.A5DVp_in[7] =    4.0548284640925658;
 pdfmellin_.A6DVp_in[7] =    1.2850091387489186;
 pdfmellin_.A7DVp_in[7] =   -6.3401038266721343;
 pdfmellin_.A8DVp_in[7] =    4.3542597728671382;
 pdfmellin_.A1USp_in[7] =    9.1237220297165328E-002 ;
 pdfmellin_.A2USp_in[7] =  -0.39726652041271809;
 pdfmellin_.A3USp_in[7] =    8.0940661576539163;
 pdfmellin_.A4USp_in[7] =  -0.76842129600290221;
 pdfmellin_.A5USp_in[7] =   0.13097443545694887;
 pdfmellin_.A6USp_in[7] =   -4.1276760779174992;
 pdfmellin_.A7USp_in[7] =    7.9777571965894367;
 pdfmellin_.A8USp_in[7] =    8.8566358666501941;
 pdfmellin_.A1DSp_in[7] =   0.11617384202746615;
 pdfmellin_.A2DSp_in[7] =  -0.36298798756458789;
 pdfmellin_.A3DSp_in[7] =    15.631053958465792;
 pdfmellin_.A4DSp_in[7] =    4.9779914224761335;
 pdfmellin_.A5DSp_in[7] =   -9.2179847416097971E-002 ;
 pdfmellin_.A6DSp_in[7] =    5.8484174208135613;
 pdfmellin_.A7DSp_in[7] =    39.592146370348445;
 pdfmellin_.A8DSp_in[7] =    89.222690512770015;
 pdfmellin_.A1SSp_in[7] =   0.17390507735324748;
 pdfmellin_.A2SSp_in[7] =  -0.32443592671136395;
 pdfmellin_.A3SSp_in[7] =    13.819339474762693;
 pdfmellin_.A4SSp_in[7] =    24.727767464921918;
 pdfmellin_.A5SSp_in[7] =   -5.5184640910435974;
 pdfmellin_.A6SSp_in[7] =   -59.013990708785499;
 pdfmellin_.A7SSp_in[7] =    76.648879415278714;
 pdfmellin_.A8SSp_in[7] =   -10.367838571327056;
 pdfmellin_.A1GLp_in[7] =    1.6229925469922737;
 pdfmellin_.A2GLp_in[7] =  -0.43039554401551466;
 pdfmellin_.A3GLp_in[7] =    9.7202803486702969;
 pdfmellin_.A4GLp_in[7] =    3.0438116617037672;
 pdfmellin_.A5GLp_in[7] =   -2.9435736050872805;
 pdfmellin_.A6GLp_in[7] =    24.117976849148050;
 pdfmellin_.A7GLp_in[7] =   -82.279162819882856;
 pdfmellin_.A8GLp_in[7] =    87.061293646810839;
 pdfmellin_.A1CHp_in[7] =    5.6412066373300325E-002 ;
 pdfmellin_.A2CHp_in[7] =  -0.44240082703377964;
 pdfmellin_.A3CHp_in[7] =    10.888408433603251;
 pdfmellin_.A4CHp_in[7] =   -12.092725902670242;
 pdfmellin_.A5CHp_in[7] =    7.6265165441299132E-002 ;
 pdfmellin_.A6CHp_in[7] =    37.717175902625712;
 pdfmellin_.A7CHp_in[7] =   -37.498907180297479;
 pdfmellin_.A8CHp_in[7] =    19.461440810690505;
 pdfmellin_.A1BOp_in[7] =    4.5827337719813531E-002 ;
 pdfmellin_.A2BOp_in[7] =  -0.44054432825377604;
 pdfmellin_.A3BOp_in[7] =    14.174805812915565;
 pdfmellin_.A4BOp_in[7] =    5.5009848791778495;
 pdfmellin_.A5BOp_in[7] =   -2.7742393944502481;
 pdfmellin_.A6BOp_in[7] =    4.1562082014654607;
 pdfmellin_.A7BOp_in[7] =   -36.468951035095685;
 pdfmellin_.A8BOp_in[7] =    79.121421391486564;
 pdfmellin_.A1UV_in[8] =   0.51685244834104105;
 pdfmellin_.A2UV_in[8] =   0.24996760215134528;
 pdfmellin_.A3UV_in[8] =    5.5038112267316484;
 pdfmellin_.A4UV_in[8] =    12.779693065231193;
 pdfmellin_.A5UV_in[8] =    3.5269345867673181;
 pdfmellin_.A6UV_in[8] =   -14.510447141449546;
 pdfmellin_.A7UV_in[8] =    6.3229102197679623;
 pdfmellin_.A8UV_in[8] =    25.846938552369238;
 pdfmellin_.A1DV_in[8] =   0.61221307167317984;
 pdfmellin_.A2DV_in[8] =   0.38866735920551077;
 pdfmellin_.A3DV_in[8] =    6.6096758958491719;
 pdfmellin_.A4DV_in[8] =    3.8895470653386539;
 pdfmellin_.A5DV_in[8] =    1.9088277724394374;
 pdfmellin_.A6DV_in[8] =   -2.6607750588581496;
 pdfmellin_.A7DV_in[8] =   -3.4322532178971699;
 pdfmellin_.A8DV_in[8] =    17.886094754201011;
 pdfmellin_.A1US_in[8] =    9.0551853569609403E-002 ;
 pdfmellin_.A2US_in[8] =  -0.39656993809137248;
 pdfmellin_.A3US_in[8] =    7.7297798414345076;
 pdfmellin_.A4US_in[8] =   -12.280922057256017;
 pdfmellin_.A5US_in[8] =    1.0105636444104462;
 pdfmellin_.A6US_in[8] =    46.694189531966117;
 pdfmellin_.A7US_in[8] =   -86.344351584573303;
 pdfmellin_.A8US_in[8] =    68.197920981679999;
 pdfmellin_.A1DS_in[8] =   0.10361510564476581;
 pdfmellin_.A2DS_in[8] =  -0.37719915223187717;
 pdfmellin_.A3DS_in[8] =    14.802526890507330;
 pdfmellin_.A4DS_in[8] =   -3.3553879705648768;
 pdfmellin_.A5DS_in[8] =   0.92194077013741804;
 pdfmellin_.A6DS_in[8] =    33.972775981573349;
 pdfmellin_.A7DS_in[8] =    18.776599168648659;
 pdfmellin_.A8DS_in[8] =    32.404593438454214;
 pdfmellin_.A1SS_in[8] =    9.1713318307325778E-002 ;
 pdfmellin_.A2SS_in[8] =  -0.39512828599398936;
 pdfmellin_.A3SS_in[8] =    11.205725587520682;
 pdfmellin_.A4SS_in[8] =   -16.861333243266550;
 pdfmellin_.A5SS_in[8] =  -0.28703013520736675;
 pdfmellin_.A6SS_in[8] =    72.097742779658233;
 pdfmellin_.A7SS_in[8] =   -87.730915297977361;
 pdfmellin_.A8SS_in[8] =    33.706764450336578;
 pdfmellin_.A1GL_in[8] =    2.3868119781862136;
 pdfmellin_.A2GL_in[8] =  -0.38665877477296029;
 pdfmellin_.A3GL_in[8] =    9.0561357663846263;
 pdfmellin_.A4GL_in[8] =    25.010556444938871;
 pdfmellin_.A5GL_in[8] =   -5.7505012186605384;
 pdfmellin_.A6GL_in[8] =   -60.172090717624343;
 pdfmellin_.A7GL_in[8] =    70.938212362788761;
 pdfmellin_.A8GL_in[8] =   -24.320585408658786;
 pdfmellin_.A1CH_in[8] =   0.14176247358993510;
 pdfmellin_.A2CH_in[8] =  -0.33522752368493841;
 pdfmellin_.A3CH_in[8] =    15.217087972995246;
 pdfmellin_.A4CH_in[8] =    30.060645672818900;
 pdfmellin_.A5CH_in[8] =   -6.1098215127090043;
 pdfmellin_.A6CH_in[8] =   -67.231811485572507;
 pdfmellin_.A7CH_in[8] =    37.456808026689004;
 pdfmellin_.A8CH_in[8] =    72.689216013356599;
 pdfmellin_.A1BO_in[8] =    6.6952147915966376E-002 ;
 pdfmellin_.A2BO_in[8] =  -0.39581598300179088;
 pdfmellin_.A3BO_in[8] =    14.233534281718255;
 pdfmellin_.A4BO_in[8] =    17.348514250482886;
 pdfmellin_.A5BO_in[8] =   -4.8985613660023475;
 pdfmellin_.A6BO_in[8] =   -22.473931889843598;
 pdfmellin_.A7BO_in[8] =   -19.822213828672332;
 pdfmellin_.A8BO_in[8] =    78.546652943311528;
 pdfmellin_.A1UVp_in[8] =   0.71727726399152714;
 pdfmellin_.A2UVp_in[8] =   0.28834904552249085;
 pdfmellin_.A3UVp_in[8] =    4.6295665919134663;
 pdfmellin_.A4UVp_in[8] =    6.5339067518293525;
 pdfmellin_.A5UVp_in[8] =    2.3094537585471713;
 pdfmellin_.A6UVp_in[8] =   -6.8650155428908022;
 pdfmellin_.A7UVp_in[8] =    3.5373262564872334;
 pdfmellin_.A8UVp_in[8] =   0.44003299976876065;
 pdfmellin_.A1DVp_in[8] =   0.45180043636065115;
 pdfmellin_.A2DVp_in[8] =   0.30778249293637994;
 pdfmellin_.A3DVp_in[8] =    5.1269875176610054;
 pdfmellin_.A4DVp_in[8] =    4.5388858341096991;
 pdfmellin_.A5DVp_in[8] =    1.7207775453772669;
 pdfmellin_.A6DVp_in[8] =   -1.1066070667837522;
 pdfmellin_.A7DVp_in[8] =   -15.459292857973693;
 pdfmellin_.A8DVp_in[8] =    13.829497414820068;
 pdfmellin_.A1USp_in[8] =    5.0001070523676409E-002 ;
 pdfmellin_.A2USp_in[8] =  -0.48934276062732135;
 pdfmellin_.A3USp_in[8] =    9.8338527529936801;
 pdfmellin_.A4USp_in[8] =    12.479025857916724;
 pdfmellin_.A5USp_in[8] =    1.2043543381251069;
 pdfmellin_.A6USp_in[8] =   -49.595760121517543;
 pdfmellin_.A7USp_in[8] =    38.784377334081057;
 pdfmellin_.A8USp_in[8] =    67.295046289233824;
 pdfmellin_.A1DSp_in[8] =   0.20540101272329231;
 pdfmellin_.A2DSp_in[8] =  -0.28341443818231116;
 pdfmellin_.A3DSp_in[8] =    15.688225888908205;
 pdfmellin_.A4DSp_in[8] =    7.7361576407591670;
 pdfmellin_.A5DSp_in[8] =   -2.0482768264573608;
 pdfmellin_.A6DSp_in[8] =  -0.53321476702394932;
 pdfmellin_.A7DSp_in[8] =    37.569803654900710;
 pdfmellin_.A8DSp_in[8] =    33.488259074691825;
 pdfmellin_.A1SSp_in[8] =    6.0545763299873517E-002 ;
 pdfmellin_.A2SSp_in[8] =  -0.47059670785975188;
 pdfmellin_.A3SSp_in[8] =    5.9607701310491592;
 pdfmellin_.A4SSp_in[8] =    4.5961858111773388;
 pdfmellin_.A5SSp_in[8] =   -1.7580425443532590;
 pdfmellin_.A6SSp_in[8] =   -31.147033944527983;
 pdfmellin_.A7SSp_in[8] =    83.820822283374525;
 pdfmellin_.A8SSp_in[8] =   -71.289434785178358;
 pdfmellin_.A1GLp_in[8] =   0.82625596933119516;
 pdfmellin_.A2GLp_in[8] =  -0.52026942740606763;
 pdfmellin_.A3GLp_in[8] =    5.9870109178471047;
 pdfmellin_.A4GLp_in[8] =   -11.285074447643691;
 pdfmellin_.A5GLp_in[8] =   0.18274855107974020;
 pdfmellin_.A6GLp_in[8] =    31.761104771200053;
 pdfmellin_.A7GLp_in[8] =   -34.819034360117563;
 pdfmellin_.A8GLp_in[8] =    14.672231213086917;
 pdfmellin_.A1CHp_in[8] =    7.2005833293381810E-002 ;
 pdfmellin_.A2CHp_in[8] =  -0.42226793861628292;
 pdfmellin_.A3CHp_in[8] =    9.7795767988052731;
 pdfmellin_.A4CHp_in[8] =    10.755149939000525;
 pdfmellin_.A5CHp_in[8] =   -2.9614526343429688;
 pdfmellin_.A6CHp_in[8] =   -42.026435007310475;
 pdfmellin_.A7CHp_in[8] =    88.751004679373153;
 pdfmellin_.A8CHp_in[8] =   -60.750277683665203;
 pdfmellin_.A1BOp_in[8] =    5.7318318755047830E-002 ;
 pdfmellin_.A2BOp_in[8] =  -0.41753821790805318;
 pdfmellin_.A3BOp_in[8] =    13.938480898132815;
 pdfmellin_.A4BOp_in[8] =    14.651479661432091;
 pdfmellin_.A5BOp_in[8] =   -4.3821415644777773;
 pdfmellin_.A6BOp_in[8] =   -15.144889573554593;
 pdfmellin_.A7BOp_in[8] =   -29.951154263638877;
 pdfmellin_.A8BOp_in[8] =    84.832020154402926;
 pdfmellin_.A1UV_in[9] =   0.57450065355042967;
 pdfmellin_.A2UV_in[9] =   0.28165687444648668;
 pdfmellin_.A3UV_in[9] =    4.7272155034606822;
 pdfmellin_.A4UV_in[9] =    4.5647751239050436;
 pdfmellin_.A5UV_in[9] =    4.6831350159729368;
 pdfmellin_.A6UV_in[9] =   -13.261344153093006;
 pdfmellin_.A7UV_in[9] =    27.539925197781258;
 pdfmellin_.A8UV_in[9] =   -16.647264790569125;
 pdfmellin_.A1DV_in[9] =   0.63758601610350907;
 pdfmellin_.A2DV_in[9] =   0.39293264750677692;
 pdfmellin_.A3DV_in[9] =    6.6889774279213423;
 pdfmellin_.A4DV_in[9] =    4.0716007127975935;
 pdfmellin_.A5DV_in[9] =    1.6851531043236392;
 pdfmellin_.A6DV_in[9] =   -2.5990193117991822;
 pdfmellin_.A7DV_in[9] =   -2.5707206515893688;
 pdfmellin_.A8DV_in[9] =    17.066438831434965;
 pdfmellin_.A1US_in[9] =   0.11944877227975309;
 pdfmellin_.A2US_in[9] =  -0.36600584310836604;
 pdfmellin_.A3US_in[9] =    6.1657674524997894;
 pdfmellin_.A4US_in[9] =    1.5114830755940787;
 pdfmellin_.A5US_in[9] =   -1.3790412816346063;
 pdfmellin_.A6US_in[9] =   -4.3074716577522025;
 pdfmellin_.A7US_in[9] =    5.2650206495300864;
 pdfmellin_.A8US_in[9] =   0.91217206994252820;
 pdfmellin_.A1DS_in[9] =   0.15779770875508328;
 pdfmellin_.A2DS_in[9] =  -0.33203236553008381;
 pdfmellin_.A3DS_in[9] =    5.4482720376473628;
 pdfmellin_.A4DS_in[9] =    9.2079477859223360;
 pdfmellin_.A5DS_in[9] =   -2.7651675516581236;
 pdfmellin_.A6DS_in[9] =   -12.446399742719635;
 pdfmellin_.A7DS_in[9] =   -5.9819676703778697;
 pdfmellin_.A8DS_in[9] =    11.415550973444397;
 pdfmellin_.A1SS_in[9] =   0.18732616070061009;
 pdfmellin_.A2SS_in[9] =  -0.31521052081729234;
 pdfmellin_.A3SS_in[9] =    14.078826382256949;
 pdfmellin_.A4SS_in[9] =    28.156110034361980;
 pdfmellin_.A5SS_in[9] =   -5.9546048657313975;
 pdfmellin_.A6SS_in[9] =   -69.380658791672516;
 pdfmellin_.A7SS_in[9] =    85.883388436943108;
 pdfmellin_.A8SS_in[9] =   -5.9153913144382528;
 pdfmellin_.A1GL_in[9] =    2.3713388969539380;
 pdfmellin_.A2GL_in[9] =  -0.38825565317695832;
 pdfmellin_.A3GL_in[9] =    9.5731698513077497;
 pdfmellin_.A4GL_in[9] =    25.869177345914125;
 pdfmellin_.A5GL_in[9] =   -5.7961673111362515;
 pdfmellin_.A6GL_in[9] =   -61.973868043524192;
 pdfmellin_.A7GL_in[9] =    70.067036681708458;
 pdfmellin_.A8GL_in[9] =   -18.568278273642946;
 pdfmellin_.A1CH_in[9] =    8.2411409614327580E-002 ;
 pdfmellin_.A2CH_in[9] =  -0.39765529790153503;
 pdfmellin_.A3CH_in[9] =    11.389012185276908;
 pdfmellin_.A4CH_in[9] =    5.2866640608639059;
 pdfmellin_.A5CH_in[9] =   -2.5362684495202288;
 pdfmellin_.A6CH_in[9] =   -21.779402980188408;
 pdfmellin_.A7CH_in[9] =    71.647008712354790;
 pdfmellin_.A8CH_in[9] =   -63.276943581480545;
 pdfmellin_.A1BO_in[9] =    6.4265929882696499E-002 ;
 pdfmellin_.A2BO_in[9] =  -0.40001824612892217;
 pdfmellin_.A3BO_in[9] =    13.237041194393649;
 pdfmellin_.A4BO_in[9] =    11.733061840345595;
 pdfmellin_.A5BO_in[9] =   -4.4553252512456609;
 pdfmellin_.A6BO_in[9] =  -0.18323690630180067;
 pdfmellin_.A7BO_in[9] =   -53.190834674970979;
 pdfmellin_.A8BO_in[9] =    84.416738488996060;
 pdfmellin_.A1UVp_in[9] =    1.2899654186846481;
 pdfmellin_.A2UVp_in[9] =   0.38427813095022700;
 pdfmellin_.A3UVp_in[9] =    4.8142780035678694;
 pdfmellin_.A4UVp_in[9] =    5.0834092365064301;
 pdfmellin_.A5UVp_in[9] =   0.46431208825390513;
 pdfmellin_.A6UVp_in[9] =   -2.1887682992041277;
 pdfmellin_.A7UVp_in[9] =   -5.0356330238049765;
 pdfmellin_.A8UVp_in[9] =    6.5862017849834444;
 pdfmellin_.A1DVp_in[9] =   0.65755171434834103;
 pdfmellin_.A2DVp_in[9] =   0.36692763907862025;
 pdfmellin_.A3DVp_in[9] =    4.9965210638698023;
 pdfmellin_.A4DVp_in[9] =    4.4947061349220228;
 pdfmellin_.A5DVp_in[9] =   0.48362766864251938;
 pdfmellin_.A6DVp_in[9] =   -3.2356085430582482;
 pdfmellin_.A7DVp_in[9] =   -9.3700906157165917;
 pdfmellin_.A8DVp_in[9] =    9.3706840168242991;
 pdfmellin_.A1USp_in[9] =   0.15941055291907674;
 pdfmellin_.A2USp_in[9] =  -0.32826381243917824;
 pdfmellin_.A3USp_in[9] =    9.5899447001247804;
 pdfmellin_.A4USp_in[9] =    9.0840340174900795;
 pdfmellin_.A5USp_in[9] =   -2.7966724052117655;
 pdfmellin_.A6USp_in[9] =   -3.8348441100402368;
 pdfmellin_.A7USp_in[9] =   -35.881872420869207;
 pdfmellin_.A8USp_in[9] =    62.104213261966009;
 pdfmellin_.A1DSp_in[9] =   0.33392275057137027;
 pdfmellin_.A2DSp_in[9] =  -0.14759345980620697;
 pdfmellin_.A3DSp_in[9] =    17.283414284331776;
 pdfmellin_.A4DSp_in[9] =   -10.195312735229386;
 pdfmellin_.A5DSp_in[9] =   0.34555442093645183;
 pdfmellin_.A6DSp_in[9] =    38.502464064192964;
 pdfmellin_.A7DSp_in[9] =   -1.4262352765298942;
 pdfmellin_.A8DSp_in[9] =    88.091501181859883;
 pdfmellin_.A1SSp_in[9] =    8.3575014946671994E-002 ;
 pdfmellin_.A2SSp_in[9] =  -0.41871533157122359;
 pdfmellin_.A3SSp_in[9] =    13.478229348879941;
 pdfmellin_.A4SSp_in[9] =    4.2853333125714492;
 pdfmellin_.A5SSp_in[9] =   -1.8421612409552068;
 pdfmellin_.A6SSp_in[9] =    3.8546112397431926;
 pdfmellin_.A7SSp_in[9] =   -27.992747265450049;
 pdfmellin_.A8SSp_in[9] =    85.663409044904199;
 pdfmellin_.A1GLp_in[9] =   0.68928847943155358;
 pdfmellin_.A2GLp_in[9] =  -0.56215310374858385;
 pdfmellin_.A3GLp_in[9] =    8.0487352592550963;
 pdfmellin_.A4GLp_in[9] =  -0.29710583131302926;
 pdfmellin_.A5GLp_in[9] =  -0.45401149145612862;
 pdfmellin_.A6GLp_in[9] =   -1.1418421319627186;
 pdfmellin_.A7GLp_in[9] =    3.3266117663770274;
 pdfmellin_.A8GLp_in[9] =    9.8951025770590064;
 pdfmellin_.A1CHp_in[9] =   0.13194897655047841;
 pdfmellin_.A2CHp_in[9] =  -0.34002191956550532;
 pdfmellin_.A3CHp_in[9] =    11.583249235673760;
 pdfmellin_.A4CHp_in[9] =    23.954589422795848;
 pdfmellin_.A5CHp_in[9] =   -5.4806819275649801;
 pdfmellin_.A6CHp_in[9] =   -64.298527221289646;
 pdfmellin_.A7CHp_in[9] =    89.223020901126745;
 pdfmellin_.A8CHp_in[9] =   -37.207528785951851;
 pdfmellin_.A1BOp_in[9] =    4.1835773882232481E-002 ;
 pdfmellin_.A2BOp_in[9] =  -0.44237396077255481;
 pdfmellin_.A3BOp_in[9] =    12.867898960862986;
 pdfmellin_.A4BOp_in[9] =    6.5940570239890341;
 pdfmellin_.A5BOp_in[9] =   -2.2103040757816785;
 pdfmellin_.A6BOp_in[9] =   -26.277601717356355;
 pdfmellin_.A7BOp_in[9] =    58.255223900871101;
 pdfmellin_.A8BOp_in[9] =   -18.786877109040020;
 pdfmellin_.A1UV_in[10] =   0.61859216591310839;
 pdfmellin_.A2UV_in[10] =   0.28097923019224225;
 pdfmellin_.A3UV_in[10] =    4.6958301322922038;
 pdfmellin_.A4UV_in[10] =    6.6819599346342073;
 pdfmellin_.A5UV_in[10] =    3.4870935436515254;
 pdfmellin_.A6UV_in[10] =   -11.577203606521721;
 pdfmellin_.A7UV_in[10] =    14.599557874147735;
 pdfmellin_.A8UV_in[10] =   -5.7088981288157896;
 pdfmellin_.A1DV_in[10] =   0.63275648736883416;
 pdfmellin_.A2DV_in[10] =   0.39098185317811440;
 pdfmellin_.A3DV_in[10] =    6.7302683525479239;
 pdfmellin_.A4DV_in[10] =    4.0691171708794123;
 pdfmellin_.A5DV_in[10] =    1.6964298410317440;
 pdfmellin_.A6DV_in[10] =   -2.3982380140101469;
 pdfmellin_.A7DV_in[10] =   -2.2449545959599533;
 pdfmellin_.A8DV_in[10] =    16.983459216362888;
 pdfmellin_.A1US_in[10] =   0.11809450408378608;
 pdfmellin_.A2US_in[10] =  -0.36732513776499059;
 pdfmellin_.A3US_in[10] =    7.1076357311799301;
 pdfmellin_.A4US_in[10] =    2.9098931496500668;
 pdfmellin_.A5US_in[10] =   -1.3558043315952588;
 pdfmellin_.A6US_in[10] =   -8.2295606428051684;
 pdfmellin_.A7US_in[10] =    10.025193607483899;
 pdfmellin_.A8US_in[10] =   0.98699665621893473;
 pdfmellin_.A1DS_in[10] =   0.12178466334012099;
 pdfmellin_.A2DS_in[10] =  -0.36149270784288634;
 pdfmellin_.A3DS_in[10] =    8.4099010617486094;
 pdfmellin_.A4DS_in[10] =    1.4927475908665571;
 pdfmellin_.A5DS_in[10] =  -0.83246890977913779;
 pdfmellin_.A6DS_in[10] =    5.5855209415199596;
 pdfmellin_.A7DS_in[10] =    11.325965355412421;
 pdfmellin_.A8DS_in[10] =   -43.542530896406078;
 pdfmellin_.A1SS_in[10] =   0.20961045462654909;
 pdfmellin_.A2SS_in[10] =  -0.30272512067502505;
 pdfmellin_.A3SS_in[10] =    15.841503780368635;
 pdfmellin_.A4SS_in[10] =    33.762027464284685;
 pdfmellin_.A5SS_in[10] =   -6.6821459435303012;
 pdfmellin_.A6SS_in[10] =   -74.450712290606191;
 pdfmellin_.A7SS_in[10] =    44.178221992152430;
 pdfmellin_.A8SS_in[10] =    89.892955024066652;
 pdfmellin_.A1GL_in[10] =    2.2213492739429892;
 pdfmellin_.A2GL_in[10] =  -0.39509810395297718;
 pdfmellin_.A3GL_in[10] =    11.071981660201056;
 pdfmellin_.A4GL_in[10] =    19.034001162405644;
 pdfmellin_.A5GL_in[10] =   -5.0987842448834897;
 pdfmellin_.A6GL_in[10] =   -23.076443026897820;
 pdfmellin_.A7GL_in[10] =   -25.928765555585073;
 pdfmellin_.A8GL_in[10] =    74.244344627633041;
 pdfmellin_.A1CH_in[10] =    6.5801183239976427E-002 ;
 pdfmellin_.A2CH_in[10] =  -0.41913130520115699;
 pdfmellin_.A3CH_in[10] =    10.387351455139697;
 pdfmellin_.A4CH_in[10] =   -19.990993441067047;
 pdfmellin_.A5CH_in[10] =   0.22952292200550772;
 pdfmellin_.A6CH_in[10] =    73.056684338566328;
 pdfmellin_.A7CH_in[10] =   -89.966394426215601;
 pdfmellin_.A8CH_in[10] =    35.732463069829592;
 pdfmellin_.A1BO_in[10] =    6.5900644612779585E-002 ;
 pdfmellin_.A2BO_in[10] =  -0.39828521681909229;
 pdfmellin_.A3BO_in[10] =    17.360533223241362;
 pdfmellin_.A4BO_in[10] =    24.707764272171694;
 pdfmellin_.A5BO_in[10] =   -5.0878993490876994;
 pdfmellin_.A6BO_in[10] =   -59.897987971653841;
 pdfmellin_.A7BO_in[10] =    56.199021083576099;
 pdfmellin_.A8BO_in[10] =    67.381062940550436;
 pdfmellin_.A1UVp_in[10] =    1.1223773183455772;
 pdfmellin_.A2UVp_in[10] =   0.34330462864301126;
 pdfmellin_.A3UVp_in[10] =    4.7314665292517617;
 pdfmellin_.A4UVp_in[10] =    5.7855824714827762;
 pdfmellin_.A5UVp_in[10] =   0.35703647865567945;
 pdfmellin_.A6UVp_in[10] =  -0.48612055622362504;
 pdfmellin_.A7UVp_in[10] =   -9.3562723994230765;
 pdfmellin_.A8UVp_in[10] =    8.6284929094567246;
 pdfmellin_.A1DVp_in[10] =   0.72591864865854272;
 pdfmellin_.A2DVp_in[10] =   0.39996965099153026;
 pdfmellin_.A3DVp_in[10] =    5.1016775083349204;
 pdfmellin_.A4DVp_in[10] =    4.1183500458449949;
 pdfmellin_.A5DVp_in[10] =   0.52151800196992326;
 pdfmellin_.A6DVp_in[10] =   -3.4064311696132563;
 pdfmellin_.A7DVp_in[10] =   -7.9747688975980537;
 pdfmellin_.A8DVp_in[10] =    8.5552432092028852;
 pdfmellin_.A1USp_in[10] =   0.35685901674069043;
 pdfmellin_.A2USp_in[10] =  -0.24692903940896427;
 pdfmellin_.A3USp_in[10] =    9.6758916977736558;
 pdfmellin_.A4USp_in[10] =    13.130719772638251;
 pdfmellin_.A5USp_in[10] =   -5.2877734462544126;
 pdfmellin_.A6USp_in[10] =    6.4610721835600913;
 pdfmellin_.A7USp_in[10] =   -67.535239341888072;
 pdfmellin_.A8USp_in[10] =    74.477932469473927;
 pdfmellin_.A1DSp_in[10] =   0.43172715105593773;
 pdfmellin_.A2DSp_in[10] =   -9.9218843083924702E-003 ;
 pdfmellin_.A3DSp_in[10] =    18.283271479202561;
 pdfmellin_.A4DSp_in[10] =   -11.068894167726004;
 pdfmellin_.A5DSp_in[10] =    2.5669598769169468;
 pdfmellin_.A6DSp_in[10] =    5.6691100800478011E-002 ;
 pdfmellin_.A7DSp_in[10] =    86.292950896976492;
 pdfmellin_.A8DSp_in[10] =    87.650788982567917;
 pdfmellin_.A1SSp_in[10] =   0.48948779880194937;
 pdfmellin_.A2SSp_in[10] =   0.20691712109406635;
 pdfmellin_.A3SSp_in[10] =    17.432055517006631;
 pdfmellin_.A4SSp_in[10] =   -18.473849137933378;
 pdfmellin_.A5SSp_in[10] =    4.6664036366017427;
 pdfmellin_.A6SSp_in[10] =   -19.517758727048900;
 pdfmellin_.A7SSp_in[10] =    77.876269912577726;
 pdfmellin_.A8SSp_in[10] =    72.341819996487544;
 pdfmellin_.A1GLp_in[10] =   0.35324201223387625;
 pdfmellin_.A2GLp_in[10] =  -0.68367841226115322;
 pdfmellin_.A3GLp_in[10] =    7.9351086119097864;
 pdfmellin_.A4GLp_in[10] =   -1.1904842377175520;
 pdfmellin_.A5GLp_in[10] =    1.2088500020878499;
 pdfmellin_.A6GLp_in[10] =   -2.8112782121114748;
 pdfmellin_.A7GLp_in[10] =  -0.71784690634507342;
 pdfmellin_.A8GLp_in[10] =    25.539852931582431;
 pdfmellin_.A1CHp_in[10] =   0.48123847484279852;
 pdfmellin_.A2CHp_in[10] =    8.7868022952396374E-002 ;
 pdfmellin_.A3CHp_in[10] =    14.898567314589089;
 pdfmellin_.A4CHp_in[10] =   0.68461346068585271;
 pdfmellin_.A5CHp_in[10] =   -2.2339859487616138;
 pdfmellin_.A6CHp_in[10] =   -1.8149448622129682;
 pdfmellin_.A7CHp_in[10] =    4.0689632503227102;
 pdfmellin_.A8CHp_in[10] =    23.513474641428360;
 pdfmellin_.A1BOp_in[10] =   0.40417097720832035;
 pdfmellin_.A2BOp_in[10] =   0.24949956183679300;
 pdfmellin_.A3BOp_in[10] =    19.118807908443998;
 pdfmellin_.A4BOp_in[10] =   0.46568754313604188;
 pdfmellin_.A5BOp_in[10] =   0.24288364603947343;
 pdfmellin_.A6BOp_in[10] =   -49.076854779413907;
 pdfmellin_.A7BOp_in[10] =    83.345864987227401;
 pdfmellin_.A8BOp_in[10] =    61.779251165304856;
 pdfmellin_.A1UV_in[11] =   0.66132868045082427;
 pdfmellin_.A2UV_in[11] =   0.28891255910238689;
 pdfmellin_.A3UV_in[11] =    4.7742301245272927;
 pdfmellin_.A4UV_in[11] =    6.1271638905535326;
 pdfmellin_.A5UV_in[11] =    3.1377040989818070;
 pdfmellin_.A6UV_in[11] =   -7.5729723314940145;
 pdfmellin_.A7UV_in[11] =    6.5507764669784194;
 pdfmellin_.A8UV_in[11] =   0.49347889657130395;
 pdfmellin_.A1DV_in[11] =   0.63874212694063248;
 pdfmellin_.A2DV_in[11] =   0.39283352148177819;
 pdfmellin_.A3DV_in[11] =    6.7258258390570154;
 pdfmellin_.A4DV_in[11] =    4.1447371477965582;
 pdfmellin_.A5DV_in[11] =    1.6661408408769134;
 pdfmellin_.A6DV_in[11] =   -2.6170459814664184;
 pdfmellin_.A7DV_in[11] =   -2.4399766542318133;
 pdfmellin_.A8DV_in[11] =    17.307137190155686;
 pdfmellin_.A1US_in[11] =   0.11580589772622425;
 pdfmellin_.A2US_in[11] =  -0.36884036663038400;
 pdfmellin_.A3US_in[11] =    7.1953718628111565;
 pdfmellin_.A4US_in[11] =    1.4889982144061094;
 pdfmellin_.A5US_in[11] =   -1.1235428609225480;
 pdfmellin_.A6US_in[11] =   -4.2805131298788837;
 pdfmellin_.A7US_in[11] =    5.4697863197540926;
 pdfmellin_.A8US_in[11] =    2.9770784046838372;
 pdfmellin_.A1DS_in[11] =   0.12780494164652725;
 pdfmellin_.A2DS_in[11] =  -0.35593515323844155;
 pdfmellin_.A3DS_in[11] =    8.4860326069560070;
 pdfmellin_.A4DS_in[11] =    3.6412188100275138;
 pdfmellin_.A5DS_in[11] =   -1.1972735471567972;
 pdfmellin_.A6DS_in[11] =  -0.90586072498176406;
 pdfmellin_.A7DS_in[11] =    20.699626763114779;
 pdfmellin_.A8DS_in[11] =   -48.251892426232139;
 pdfmellin_.A1SS_in[11] =   0.17708766093296663;
 pdfmellin_.A2SS_in[11] =  -0.32204191629679452;
 pdfmellin_.A3SS_in[11] =    15.147127183361503;
 pdfmellin_.A4SS_in[11] =    25.874762907577797;
 pdfmellin_.A5SS_in[11] =   -5.6150668074117291;
 pdfmellin_.A6SS_in[11] =   -53.371933366427740;
 pdfmellin_.A7SS_in[11] =    31.582181033831446;
 pdfmellin_.A8SS_in[11] =    72.443192431323354;
 pdfmellin_.A1GL_in[11] =    2.3797502534945596;
 pdfmellin_.A2GL_in[11] =  -0.38786169206755144;
 pdfmellin_.A3GL_in[11] =    9.9162753267187966;
 pdfmellin_.A4GL_in[11] =    25.951969765317187;
 pdfmellin_.A5GL_in[11] =   -5.7989158416004614;
 pdfmellin_.A6GL_in[11] =   -60.944452129923235;
 pdfmellin_.A7GL_in[11] =    65.441635998729481;
 pdfmellin_.A8GL_in[11] =   -11.923848855769592;
 pdfmellin_.A1CH_in[11] =    9.3127921658263557E-002 ;
 pdfmellin_.A2CH_in[11] =  -0.38058475743032821;
 pdfmellin_.A3CH_in[11] =    9.5001184399522387;
 pdfmellin_.A4CH_in[11] =   -3.6358595203072142;
 pdfmellin_.A5CH_in[11] =   -2.5765929335140427;
 pdfmellin_.A6CH_in[11] =    39.954944420669591;
 pdfmellin_.A7CH_in[11] =   -87.280822088260763;
 pdfmellin_.A8CH_in[11] =    67.009595366277239;
 pdfmellin_.A1BO_in[11] =    6.5704196959312303E-002 ;
 pdfmellin_.A2BO_in[11] =  -0.39688027180260982;
 pdfmellin_.A3BO_in[11] =    12.478233013368252;
 pdfmellin_.A4BO_in[11] =    11.510377582634039;
 pdfmellin_.A5BO_in[11] =   -4.5247514603797896;
 pdfmellin_.A6BO_in[11] =  -0.73106557855928145;
 pdfmellin_.A7BO_in[11] =   -46.949856484960975;
 pdfmellin_.A8BO_in[11] =    70.253455732370213;
 pdfmellin_.A1UVp_in[11] =    1.3222599130343413;
 pdfmellin_.A2UVp_in[11] =   0.39652010343379840;
 pdfmellin_.A3UVp_in[11] =    4.4614557553113308;
 pdfmellin_.A4UVp_in[11] =    4.2562465436452293;
 pdfmellin_.A5UVp_in[11] =   0.59340432576595958;
 pdfmellin_.A6UVp_in[11] =   -2.8719944206652031;
 pdfmellin_.A7UVp_in[11] =   -3.0718117806949152;
 pdfmellin_.A8UVp_in[11] =    3.0522487563984839;
 pdfmellin_.A1DVp_in[11] =   0.79445368222792057;
 pdfmellin_.A2DVp_in[11] =   0.50998728964779350;
 pdfmellin_.A3DVp_in[11] =    5.2823968277729447;
 pdfmellin_.A4DVp_in[11] =    3.9815896248014866;
 pdfmellin_.A5DVp_in[11] =    1.6752751306510589;
 pdfmellin_.A6DVp_in[11] =   -6.1035893806633510;
 pdfmellin_.A7DVp_in[11] =   -11.319624754705160;
 pdfmellin_.A8DVp_in[11] =    15.601118076691165;
 pdfmellin_.A1USp_in[11] =   0.45439442507869221;
 pdfmellin_.A2USp_in[11] =   0.55477240801689365;
 pdfmellin_.A3USp_in[11] =    10.848625950417723;
 pdfmellin_.A4USp_in[11] =   -19.945846072413215;
 pdfmellin_.A5USp_in[11] =    12.525794931484455;
 pdfmellin_.A6USp_in[11] =   -55.221439353968741;
 pdfmellin_.A7USp_in[11] =    88.260548032846629;
 pdfmellin_.A8USp_in[11] =    27.716370547755034;
 pdfmellin_.A1DSp_in[11] =    7.3611881398835330E-002 ;
 pdfmellin_.A2DSp_in[11] =   -8.3560297246592197E-002 ;
 pdfmellin_.A3DSp_in[11] =    7.4581575127090582;
 pdfmellin_.A4DSp_in[11] =   -12.902479025757934;
 pdfmellin_.A5DSp_in[11] =    12.315637564816456;
 pdfmellin_.A6DSp_in[11] =   -41.560166315713865;
 pdfmellin_.A7DSp_in[11] =    78.061083760125683;
 pdfmellin_.A8DSp_in[11] =   -56.848143267650997;
 pdfmellin_.A1SSp_in[11] =   0.92390495241538506;
 pdfmellin_.A2SSp_in[11] =   0.18450992140373068;
 pdfmellin_.A3SSp_in[11] =    8.4413267353895360;
 pdfmellin_.A4SSp_in[11] =    14.115545148646646;
 pdfmellin_.A5SSp_in[11] =   -4.6732417516378177;
 pdfmellin_.A6SSp_in[11] =   -44.586611948766226;
 pdfmellin_.A7SSp_in[11] =    89.602941620735749;
 pdfmellin_.A8SSp_in[11] =   -66.615937888180412;
 pdfmellin_.A1GLp_in[11] =   0.65925682022707066;
 pdfmellin_.A2GLp_in[11] =  -0.52293354536244352;
 pdfmellin_.A3GLp_in[11] =    8.2507347505423851;
 pdfmellin_.A4GLp_in[11] =   -2.1372125975393139;
 pdfmellin_.A5GLp_in[11] =   0.56770263801271881;
 pdfmellin_.A6GLp_in[11] =   -1.3221981519935468;
 pdfmellin_.A7GLp_in[11] =    2.1653020805604797;
 pdfmellin_.A8GLp_in[11] =    16.920478330977843;
 pdfmellin_.A1CHp_in[11] =   0.63911901153025497;
 pdfmellin_.A2CHp_in[11] =   0.32820193072571602;
 pdfmellin_.A3CHp_in[11] =    13.439113914913287;
 pdfmellin_.A4CHp_in[11] =   0.67303147187832835;
 pdfmellin_.A5CHp_in[11] =   -1.8467150081113104;
 pdfmellin_.A6CHp_in[11] =   -2.7347120617359622;
 pdfmellin_.A7CHp_in[11] =    1.5745261551205658;
 pdfmellin_.A8CHp_in[11] =    17.184556623209872;
 pdfmellin_.A1BOp_in[11] =   0.50797287164813254;
 pdfmellin_.A2BOp_in[11] =    1.0918943613163172;
 pdfmellin_.A3BOp_in[11] =    20.905633797254577;
 pdfmellin_.A4BOp_in[11] =    2.4354253936017685;
 pdfmellin_.A5BOp_in[11] =   0.81285783072733864;
 pdfmellin_.A6BOp_in[11] =    43.877885710692738;
 pdfmellin_.A7BOp_in[11] =    8.3963736267461542;
 pdfmellin_.A8BOp_in[11] =    89.928488413781537;
 pdfmellin_.A1UV_in[12] =   0.66050917905738515;
 pdfmellin_.A2UV_in[12] =   0.28845531747310083;
 pdfmellin_.A3UV_in[12] =    4.7813652435186631;
 pdfmellin_.A4UV_in[12] =    6.1819260589626595;
 pdfmellin_.A5UV_in[12] =    3.1253336645390064;
 pdfmellin_.A6UV_in[12] =   -7.5077724975485722;
 pdfmellin_.A7UV_in[12] =    6.2912145428107067;
 pdfmellin_.A8UV_in[12] =   0.85230391006659545;
 pdfmellin_.A1DV_in[12] =   0.62261347620667307;
 pdfmellin_.A2DV_in[12] =   0.38830743520250754;
 pdfmellin_.A3DV_in[12] =    6.7522240475246544;
 pdfmellin_.A4DV_in[12] =    4.2584837050887607;
 pdfmellin_.A5DV_in[12] =    1.7304573848926381;
 pdfmellin_.A6DV_in[12] =   -2.6324915897797467;
 pdfmellin_.A7DV_in[12] =   -2.5269478502486322;
 pdfmellin_.A8DV_in[12] =    18.168797878749871;
 pdfmellin_.A1US_in[12] =   0.11756882740680168;
 pdfmellin_.A2US_in[12] =  -0.36645714129015033;
 pdfmellin_.A3US_in[12] =    9.2909481294507117;
 pdfmellin_.A4US_in[12] =    3.4683072855916208E-002 ;
 pdfmellin_.A5US_in[12] =   -1.0026885103037273;
 pdfmellin_.A6US_in[12] =    19.204557588156163;
 pdfmellin_.A7US_in[12] =   -64.179107743928810;
 pdfmellin_.A8US_in[12] =    76.481210874824427;
 pdfmellin_.A1DS_in[12] =   0.11458702523507418;
 pdfmellin_.A2DS_in[12] =  -0.36696153990644109;
 pdfmellin_.A3DS_in[12] =    14.876500747449763;
 pdfmellin_.A4DS_in[12] =    2.7292320811558248;
 pdfmellin_.A5DS_in[12] =   -3.7236492897712026E-002 ;
 pdfmellin_.A6DS_in[12] =    10.364584164134939;
 pdfmellin_.A7DS_in[12] =    55.140421121374942;
 pdfmellin_.A8DS_in[12] =    8.6684816075191975;
 pdfmellin_.A1SS_in[12] =   0.19205898596450596;
 pdfmellin_.A2SS_in[12] =  -0.31341449558811019;
 pdfmellin_.A3SS_in[12] =    14.754769080179059;
 pdfmellin_.A4SS_in[12] =    32.089154516776752;
 pdfmellin_.A5SS_in[12] =   -6.3212298435934144;
 pdfmellin_.A6SS_in[12] =   -80.575851721957761;
 pdfmellin_.A7SS_in[12] =    88.768522270814771;
 pdfmellin_.A8SS_in[12] =    17.379850258735559;
 pdfmellin_.A1GL_in[12] =    2.3127868146197135;
 pdfmellin_.A2GL_in[12] =  -0.39043428219487397;
 pdfmellin_.A3GL_in[12] =    11.514486108691226;
 pdfmellin_.A4GL_in[12] =    21.073134561242249;
 pdfmellin_.A5GL_in[12] =   -5.3521767531480933;
 pdfmellin_.A6GL_in[12] =   -28.062311912302810;
 pdfmellin_.A7GL_in[12] =   -23.896019156346199;
 pdfmellin_.A8GL_in[12] =    81.312466965900711;
 pdfmellin_.A1CH_in[12] =   0.12589398778294347;
 pdfmellin_.A2CH_in[12] =  -0.34847785552321264;
 pdfmellin_.A3CH_in[12] =    13.541452117339096;
 pdfmellin_.A4CH_in[12] =    20.938113000139488;
 pdfmellin_.A5CH_in[12] =   -5.1916016081460299;
 pdfmellin_.A6CH_in[12] =   -36.882321169913297;
 pdfmellin_.A7CH_in[12] =    2.2626730782502449;
 pdfmellin_.A8CH_in[12] =    63.351608961693984;
 pdfmellin_.A1BO_in[12] =    6.7379127563602287E-002 ;
 pdfmellin_.A2BO_in[12] =  -0.39365466257835841;
 pdfmellin_.A3BO_in[12] =    13.054670004067015;
 pdfmellin_.A4BO_in[12] =    12.005738660368657;
 pdfmellin_.A5BO_in[12] =   -4.6066427653645263;
 pdfmellin_.A6BO_in[12] =   0.15070762100279467;
 pdfmellin_.A7BO_in[12] =   -53.316584691945295;
 pdfmellin_.A8BO_in[12] =    80.204711520947683;
 pdfmellin_.A1UVp_in[12] =    1.4434973729408163;
 pdfmellin_.A2UVp_in[12] =   0.41453025234870844;
 pdfmellin_.A3UVp_in[12] =    4.2884359836064334;
 pdfmellin_.A4UVp_in[12] =    4.3049252287324116;
 pdfmellin_.A5UVp_in[12] =   0.26933431115236317;
 pdfmellin_.A6UVp_in[12] =   -2.5665298550234041;
 pdfmellin_.A7UVp_in[12] =   -4.9942691869670597;
 pdfmellin_.A8UVp_in[12] =    3.9905463894171334;
 pdfmellin_.A1DVp_in[12] =   0.72369627323952934;
 pdfmellin_.A2DVp_in[12] =   0.52895223146457337;
 pdfmellin_.A3DVp_in[12] =    4.6352786479669454;
 pdfmellin_.A4DVp_in[12] =    3.3995418062388154;
 pdfmellin_.A5DVp_in[12] =    2.1317062063439636;
 pdfmellin_.A6DVp_in[12] =   -7.5757189545501262;
 pdfmellin_.A7DVp_in[12] =   -11.625882873826328;
 pdfmellin_.A8DVp_in[12] =    15.559529151276692;
 pdfmellin_.A1USp_in[12] =   0.95840329790228140;
 pdfmellin_.A2USp_in[12] =   0.91953371609179890;
 pdfmellin_.A3USp_in[12] =    10.008828737354523;
 pdfmellin_.A4USp_in[12] =   -11.282314378322955;
 pdfmellin_.A5USp_in[12] =    7.1902333830433633;
 pdfmellin_.A6USp_in[12] =   -28.077001542727338;
 pdfmellin_.A7USp_in[12] =    37.926558917524218;
 pdfmellin_.A8USp_in[12] =    16.307367679100381;
 pdfmellin_.A1DSp_in[12] =   0.22339775680212481;
 pdfmellin_.A2DSp_in[12] =   0.20118795311920024;
 pdfmellin_.A3DSp_in[12] =    7.7312014652499830;
 pdfmellin_.A4DSp_in[12] =  -0.17527624770551498;
 pdfmellin_.A5DSp_in[12] =    3.8822120205256709;
 pdfmellin_.A6DSp_in[12] =   -23.321620685576832;
 pdfmellin_.A7DSp_in[12] =    21.049951739149037;
 pdfmellin_.A8DSp_in[12] =   -9.1582710048185021;
 pdfmellin_.A1SSp_in[12] =   0.65086575454148299;
 pdfmellin_.A2SSp_in[12] =    1.4894465309754186;
 pdfmellin_.A3SSp_in[12] =    9.2666108994158840;
 pdfmellin_.A4SSp_in[12] =    5.8100474664805581;
 pdfmellin_.A5SSp_in[12] =    1.2563272356966664;
 pdfmellin_.A6SSp_in[12] =    3.5418698220375404;
 pdfmellin_.A7SSp_in[12] =   -5.8022966341391822;
 pdfmellin_.A8SSp_in[12] =   -39.931753485402197;
 pdfmellin_.A1GLp_in[12] =   0.65263542582955336;
 pdfmellin_.A2GLp_in[12] =  -0.54561949426187817;
 pdfmellin_.A3GLp_in[12] =    8.2196309898019919;
 pdfmellin_.A4GLp_in[12] =   -2.1911814964873848;
 pdfmellin_.A5GLp_in[12] =   0.41301974749995907;
 pdfmellin_.A6GLp_in[12] =  -0.91000629166325098;
 pdfmellin_.A7GLp_in[12] =    3.4700999991042067;
 pdfmellin_.A8GLp_in[12] =    14.499818868773346;
 pdfmellin_.A1CHp_in[12] =   0.61191580765090370;
 pdfmellin_.A2CHp_in[12] =   0.94883445898358509;
 pdfmellin_.A3CHp_in[12] =    12.064231965757651;
 pdfmellin_.A4CHp_in[12] =    2.7275773385230870;
 pdfmellin_.A5CHp_in[12] =   -1.1317563776545327;
 pdfmellin_.A6CHp_in[12] =  -0.40107767211124212;
 pdfmellin_.A7CHp_in[12] =  -0.31189063024302754;
 pdfmellin_.A8CHp_in[12] =   0.38967986416075462;
 pdfmellin_.A1BOp_in[12] =   0.37020619055607762;
 pdfmellin_.A2BOp_in[12] =    1.1865671226820154;
 pdfmellin_.A3BOp_in[12] =    17.931287140542764;
 pdfmellin_.A4BOp_in[12] =   -6.4481321328707377;
 pdfmellin_.A5BOp_in[12] =   0.15181524000909263;
 pdfmellin_.A6BOp_in[12] =    16.218985881319369;
 pdfmellin_.A7BOp_in[12] =    71.670568714804119;
 pdfmellin_.A8BOp_in[12] =    56.226644614436196;
 pdfmellin_.A1UV_in[13] =   0.66050917905738515;
 pdfmellin_.A2UV_in[13] =   0.28845531747310083;
 pdfmellin_.A3UV_in[13] =    4.7813652435186631;
 pdfmellin_.A4UV_in[13] =    6.1819260589626595;
 pdfmellin_.A5UV_in[13] =    3.1253336645390064;
 pdfmellin_.A6UV_in[13] =   -7.5077724975485722;
 pdfmellin_.A7UV_in[13] =    6.2912145428107067;
 pdfmellin_.A8UV_in[13] =   0.85230391006659545;
 pdfmellin_.A1DV_in[13] =   0.62261347620667307;
 pdfmellin_.A2DV_in[13] =   0.38830743520250754;
 pdfmellin_.A3DV_in[13] =    6.7522240475246544;
 pdfmellin_.A4DV_in[13] =    4.2584837050887607;
 pdfmellin_.A5DV_in[13] =    1.7304573848926381;
 pdfmellin_.A6DV_in[13] =   -2.6324915897797467;
 pdfmellin_.A7DV_in[13] =   -2.5269478502486322;
 pdfmellin_.A8DV_in[13] =    18.168797878749871;
 pdfmellin_.A1US_in[13] =   0.11756882740680168;
 pdfmellin_.A2US_in[13] =  -0.36645714129015033;
 pdfmellin_.A3US_in[13] =    9.2909481294507117;
 pdfmellin_.A4US_in[13] =    3.4683072855916208E-002 ;
 pdfmellin_.A5US_in[13] =   -1.0026885103037273;
 pdfmellin_.A6US_in[13] =    19.204557588156163;
 pdfmellin_.A7US_in[13] =   -64.179107743928810;
 pdfmellin_.A8US_in[13] =    76.481210874824427;
 pdfmellin_.A1DS_in[13] =   0.11458702523507418;
 pdfmellin_.A2DS_in[13] =  -0.36696153990644109;
 pdfmellin_.A3DS_in[13] =    14.876500747449763;
 pdfmellin_.A4DS_in[13] =    2.7292320811558248;
 pdfmellin_.A5DS_in[13] =   -3.7236492897712026E-002 ;
 pdfmellin_.A6DS_in[13] =    10.364584164134939;
 pdfmellin_.A7DS_in[13] =    55.140421121374942;
 pdfmellin_.A8DS_in[13] =    8.6684816075191975;
 pdfmellin_.A1SS_in[13] =   0.19205898596450596;
 pdfmellin_.A2SS_in[13] =  -0.31341449558811019;
 pdfmellin_.A3SS_in[13] =    14.754769080179059;
 pdfmellin_.A4SS_in[13] =    32.089154516776752;
 pdfmellin_.A5SS_in[13] =   -6.3212298435934144;
 pdfmellin_.A6SS_in[13] =   -80.575851721957761;
 pdfmellin_.A7SS_in[13] =    88.768522270814771;
 pdfmellin_.A8SS_in[13] =    17.379850258735559;
 pdfmellin_.A1GL_in[13] =    2.3127868146197135;
 pdfmellin_.A2GL_in[13] =  -0.39043428219487397;
 pdfmellin_.A3GL_in[13] =    11.514486108691226;
 pdfmellin_.A4GL_in[13] =    21.073134561242249;
 pdfmellin_.A5GL_in[13] =   -5.3521767531480933;
 pdfmellin_.A6GL_in[13] =   -28.062311912302810;
 pdfmellin_.A7GL_in[13] =   -23.896019156346199;
 pdfmellin_.A8GL_in[13] =    81.312466965900711;
 pdfmellin_.A1CH_in[13] =   0.12589398778294347;
 pdfmellin_.A2CH_in[13] =  -0.34847785552321264;
 pdfmellin_.A3CH_in[13] =    13.541452117339096;
 pdfmellin_.A4CH_in[13] =    20.938113000139488;
 pdfmellin_.A5CH_in[13] =   -5.1916016081460299;
 pdfmellin_.A6CH_in[13] =   -36.882321169913297;
 pdfmellin_.A7CH_in[13] =    2.2626730782502449;
 pdfmellin_.A8CH_in[13] =    63.351608961693984;
 pdfmellin_.A1BO_in[13] =    6.7379127563602287E-002 ;
 pdfmellin_.A2BO_in[13] =  -0.39365466257835841;
 pdfmellin_.A3BO_in[13] =    13.054670004067015;
 pdfmellin_.A4BO_in[13] =    12.005738660368657;
 pdfmellin_.A5BO_in[13] =   -4.6066427653645263;
 pdfmellin_.A6BO_in[13] =   0.15070762100279467;
 pdfmellin_.A7BO_in[13] =   -53.316584691945295;
 pdfmellin_.A8BO_in[13] =    80.204711520947683;
 pdfmellin_.A1UVp_in[13] =    1.4434973729408163;
 pdfmellin_.A2UVp_in[13] =   0.41453025234870844;
 pdfmellin_.A3UVp_in[13] =    4.2884359836064334;
 pdfmellin_.A4UVp_in[13] =    4.3049252287324116;
 pdfmellin_.A5UVp_in[13] =   0.26933431115236317;
 pdfmellin_.A6UVp_in[13] =   -2.5665298550234041;
 pdfmellin_.A7UVp_in[13] =   -4.9942691869670597;
 pdfmellin_.A8UVp_in[13] =    3.9905463894171334;
 pdfmellin_.A1DVp_in[13] =   0.72369627323952934;
 pdfmellin_.A2DVp_in[13] =   0.52895223146457337;
 pdfmellin_.A3DVp_in[13] =    4.6352786479669454;
 pdfmellin_.A4DVp_in[13] =    3.3995418062388154;
 pdfmellin_.A5DVp_in[13] =    2.1317062063439636;
 pdfmellin_.A6DVp_in[13] =   -7.5757189545501262;
 pdfmellin_.A7DVp_in[13] =   -11.625882873826328;
 pdfmellin_.A8DVp_in[13] =    15.559529151276692;
 pdfmellin_.A1USp_in[13] =   0.95840329790228140;
 pdfmellin_.A2USp_in[13] =   0.91953371609179890;
 pdfmellin_.A3USp_in[13] =    10.008828737354523;
 pdfmellin_.A4USp_in[13] =   -11.282314378322955;
 pdfmellin_.A5USp_in[13] =    7.1902333830433633;
 pdfmellin_.A6USp_in[13] =   -28.077001542727338;
 pdfmellin_.A7USp_in[13] =    37.926558917524218;
 pdfmellin_.A8USp_in[13] =    16.307367679100381;
 pdfmellin_.A1DSp_in[13] =   0.22339775680212481;
 pdfmellin_.A2DSp_in[13] =   0.20118795311920024;
 pdfmellin_.A3DSp_in[13] =    7.7312014652499830;
 pdfmellin_.A4DSp_in[13] =  -0.17527624770551498;
 pdfmellin_.A5DSp_in[13] =    3.8822120205256709;
 pdfmellin_.A6DSp_in[13] =   -23.321620685576832;
 pdfmellin_.A7DSp_in[13] =    21.049951739149037;
 pdfmellin_.A8DSp_in[13] =   -9.1582710048185021;
 pdfmellin_.A1SSp_in[13] =   0.65086575454148299;
 pdfmellin_.A2SSp_in[13] =    1.4894465309754186;
 pdfmellin_.A3SSp_in[13] =    9.2666108994158840;
 pdfmellin_.A4SSp_in[13] =    5.8100474664805581;
 pdfmellin_.A5SSp_in[13] =    1.2563272356966664;
 pdfmellin_.A6SSp_in[13] =    3.5418698220375404;
 pdfmellin_.A7SSp_in[13] =   -5.8022966341391822;
 pdfmellin_.A8SSp_in[13] =   -39.931753485402197;
 pdfmellin_.A1GLp_in[13] =   0.65263542582955336;
 pdfmellin_.A2GLp_in[13] =  -0.54561949426187817;
 pdfmellin_.A3GLp_in[13] =    8.2196309898019919;
 pdfmellin_.A4GLp_in[13] =   -2.1911814964873848;
 pdfmellin_.A5GLp_in[13] =   0.41301974749995907;
 pdfmellin_.A6GLp_in[13] =  -0.91000629166325098;
 pdfmellin_.A7GLp_in[13] =    3.4700999991042067;
 pdfmellin_.A8GLp_in[13] =    14.499818868773346;
 pdfmellin_.A1CHp_in[13] =   0.61191580765090370;
 pdfmellin_.A2CHp_in[13] =   0.94883445898358509;
 pdfmellin_.A3CHp_in[13] =    12.064231965757651;
 pdfmellin_.A4CHp_in[13] =    2.7275773385230870;
 pdfmellin_.A5CHp_in[13] =   -1.1317563776545327;
 pdfmellin_.A6CHp_in[13] =  -0.40107767211124212;
 pdfmellin_.A7CHp_in[13] =  -0.31189063024302754;
 pdfmellin_.A8CHp_in[13] =   0.38967986416075462;
 pdfmellin_.A1BOp_in[13] =   0.37020619055607762;
 pdfmellin_.A2BOp_in[13] =    1.1865671226820154;
 pdfmellin_.A3BOp_in[13] =    17.931287140542764;
 pdfmellin_.A4BOp_in[13] =   -6.4481321328707377;
 pdfmellin_.A5BOp_in[13] =   0.15181524000909263;
 pdfmellin_.A6BOp_in[13] =    16.218985881319369;
 pdfmellin_.A7BOp_in[13] =    71.670568714804119;
 pdfmellin_.A8BOp_in[13] =    56.226644614436196;

}


// pdfmellin_.A1UV_in[0] = 0.53182008693990623; pdfmellin_.A1UV_in[1] = 0.71727726399152714; pdfmellin_.A1UV_in[2] = 1.2899654186846481; pdfmellin_.A1UV_in[3] = 1.1223773183455772; pdfmellin_.A1UV_in[4] = 1.3222599130343413; pdfmellin_.A1UV_in[5] = 1.4434973729408163; pdfmellin_.A1UV_in[6] = 1.4434973729408163; pdfmellin_.A1UV_in[7] = 0.49027644610728283; pdfmellin_.A1UV_in[8] = 0.51685244834104105; pdfmellin_.A1UV_in[9] = 0.57450065355042967; pdfmellin_.A1UV_in[10] = 0.61859216591310839; pdfmellin_.A1UV_in[11] = 0.66132868045082427; pdfmellin_.A1UV_in[12] = 0.66050917905738515; pdfmellin_.A1UV_in[13] = 0.66050917905738515; 

// pdfmellin_.A2UV_in[0] = 0.25385510801108202; pdfmellin_.A2UV_in[1] = 0.28834904552249085; pdfmellin_.A2UV_in[2] = 0.38427813095022700; pdfmellin_.A2UV_in[3] = 0.34330462864301126; pdfmellin_.A2UV_in[4] = 0.39652010343379840; pdfmellin_.A2UV_in[5] = 0.41453025234870844; pdfmellin_.A2UV_in[6] = 0.41453025234870844; pdfmellin_.A2UV_in[7] = 0.23453214546659007; pdfmellin_.A2UV_in[8] = 0.24996760215134528; pdfmellin_.A2UV_in[9] = 0.28165687444648668; pdfmellin_.A2UV_in[10] = 0.28097923019224225; pdfmellin_.A2UV_in[11] = 0.28891255910238689; pdfmellin_.A2UV_in[12] = 0.28845531747310083; pdfmellin_.A2UV_in[13] = 0.28845531747310083;

// pdfmellin_.A3UV_in[0] = 4.5598731752055262; pdfmellin_.A3UV_in[1] = 4.6295665919134663; pdfmellin_.A3UV_in[2] = 4.8142780035678694; pdfmellin_.A3UV_in[3] = 4.7314665292517617; pdfmellin_.A3UV_in[4] = 4.4614557553113308; pdfmellin_.A3UV_in[5] = 4.2884359836064334; pdfmellin_.A3UV_in[6] = 4.2884359836064334; pdfmellin_.A3UV_in[7] = 5.3698156134531061; pdfmellin_.A3UV_in[8] = 5.5038112267316484; pdfmellin_.A3UV_in[9] = 4.7272155034606822; pdfmellin_.A3UV_in[10] = 4.6958301322922038; pdfmellin_.A3UV_in[11] = 4.7742301245272927; pdfmellin_.A3UV_in[12] = 4.7813652435186631; pdfmellin_.A3UV_in[13] = 4.7813652435186631;

// pdfmellin_.A4UV_in[0] = 7.5443035002262633; pdfmellin_.A4UV_in[1] = 6.5339067518293525; pdfmellin_.A4UV_in[2] = 5.0834092365064301; pdfmellin_.A4UV_in[3] = 5.7855824714827762; pdfmellin_.A4UV_in[4] = 4.2562465436452293; pdfmellin_.A4UV_in[5] = 4.3049252287324116; pdfmellin_.A4UV_in[6] = 4.3049252287324116; pdfmellin_.A4UV_in[7] = 13.839174277221019; pdfmellin_.A4UV_in[8] = 12.779693065231193; pdfmellin_.A4UV_in[9] = 4.5647751239050436; pdfmellin_.A4UV_in[10] = 6.6819599346342073; pdfmellin_.A4UV_in[11] = 6.1271638905535326; pdfmellin_.A4UV_in[12] = 6.1819260589626595; pdfmellin_.A4UV_in[13] = 6.1819260589626595;

// pdfmellin_.A5UV_in[0] = 3.9607612507892753; pdfmellin_.A5UV_in[1] = 2.3094537585471713; pdfmellin_.A5UV_in[2] = 0.46431208825390513; pdfmellin_.A5UV_in[3] = 0.35703647865567945; pdfmellin_.A5UV_in[4] = 0.59340432576595958; pdfmellin_.A5UV_in[5] = 0.26933431115236317; pdfmellin_.A5UV_in[6] = 0.26933431115236317; pdfmellin_.A5UV_in[7] = 3.2660649803689950; pdfmellin_.A5UV_in[8] = 3.5269345867673181; pdfmellin_.A5UV_in[9] = 4.6831350159729368; pdfmellin_.A5UV_in[10] = 3.4870935436515254; pdfmellin_.A5UV_in[11] = 3.1377040989818070; pdfmellin_.A5UV_in[12] = 3.1253336645390064; pdfmellin_.A5UV_in[13] = 3.1253336645390064;


// pdfmellin_.A6UV_in[0] = -13.386861890746502; pdfmellin_.A6UV_in[1] = -6.8650155428908022; pdfmellin_.A6UV_in[2] = -2.1887682992041277; pdfmellin_.A6UV_in[3] = -0.48612055622362504; pdfmellin_.A6UV_in[4] = -2.8719944206652031; pdfmellin_.A6UV_in[5] = -2.5665298550234041; pdfmellin_.A6UV_in[6] = -2.5665298550234041; pdfmellin_.A6UV_in[7] = -13.295080289095679; pdfmellin_.A6UV_in[8] = -14.510447141449546; pdfmellin_.A6UV_in[9] = -13.261344153093006; pdfmellin_.A6UV_in[10] = -11.577203606521721; pdfmellin_.A6UV_in[11] = -7.5729723314940145; pdfmellin_.A6UV_in[12] = -7.5077724975485722; pdfmellin_.A6UV_in[13] = -7.5077724975485722;

// pdfmellin_.A7UV_in[0] = 17.089350998276430; pdfmellin_.A7UV_in[1] = 3.5373262564872334; pdfmellin_.A7UV_in[2] = -5.0356330238049765; pdfmellin_.A7UV_in[3] = -9.3562723994230765; pdfmellin_.A7UV_in[4] = -3.0718117806949152; pdfmellin_.A7UV_in[5] = -4.9942691869670597; pdfmellin_.A7UV_in[6] = -4.9942691869670597; pdfmellin_.A7UV_in[7] = 0.37532216188564860; pdfmellin_.A7UV_in[8] = 6.3229102197679623; pdfmellin_.A7UV_in[9] = 27.539925197781258; pdfmellin_.A7UV_in[10] = 14.599557874147735; pdfmellin_.A7UV_in[11] = 6.5507764669784194; pdfmellin_.A7UV_in[12] = 6.2912145428107067; pdfmellin_.A7UV_in[13] = 6.2912145428107067;

// pdfmellin_.A8UV_in[0] = -8.7769116613552427; pdfmellin_.A8UV_in[1] = 0.44003299976876065; pdfmellin_.A8UV_in[2] =  6.5862017849834444; pdfmellin_.A8UV_in[3] =  8.6284929094567246; pdfmellin_.A8UV_in[4] =  3.0522487563984839; pdfmellin_.A8UV_in[5] =  3.9905463894171334; pdfmellin_.A8UV_in[6] =  3.9905463894171334; pdfmellin_.A8UV_in[7] =  27.288921919385388; pdfmellin_.A8UV_in[8] =  25.846938552369238; pdfmellin_.A8UV_in[9] = -16.647264790569125; pdfmellin_.A8UV_in[10] = -5.7088981288157896; pdfmellin_.A8UV_in[11] = 0.49347889657130395; pdfmellin_.A8UV_in[12] = 0.85230391006659545; pdfmellin_.A8UV_in[13] = 0.85230391006659545;

// pdfmellin_.A1DV_in[0] = 0.43077675581514940; pdfmellin_.A1DV_in[1] = 0.45180043636065115; pdfmellin_.A1DV_in[2] = 0.65755171434834103; pdfmellin_.A1DV_in[3] = 0.72591864865854272; pdfmellin_.A1DV_in[4] = 0.79445368222792057; pdfmellin_.A1DV_in[5] = 0.72369627323952934; pdfmellin_.A1DV_in[6] = 0.72369627323952934; pdfmellin_.A1DV_in[7] = 0.60497816276916594; pdfmellin_.A1DV_in[8] = 0.61221307167317984; pdfmellin_.A1DV_in[9] = 0.63758601610350907; pdfmellin_.A1DV_in[10] = 0.63275648736883416; pdfmellin_.A1DV_in[11] = 0.63874212694063248; pdfmellin_.A1DV_in[12] = 0.62261347620667307; pdfmellin_.A1DV_in[13] = 0.62261347620667307;

// pdfmellin_.A2DV_in[0] = 0.34109614609453964; pdfmellin_.A2DV_in[1] = 0.30778249293637994; pdfmellin_.A2DV_in[2] = 0.36692763907862025; pdfmellin_.A2DV_in[3] = 0.39996965099153026; pdfmellin_.A2DV_in[4] = 0.50998728964779350; pdfmellin_.A2DV_in[5] = 0.52895223146457337; pdfmellin_.A2DV_in[6] = 0.52895223146457337; pdfmellin_.A2DV_in[7] = 0.38371539491749962; pdfmellin_.A2DV_in[8] = 0.38866735920551077; pdfmellin_.A2DV_in[9] = 0.39293264750677692; pdfmellin_.A2DV_in[10] = 0.39098185317811440; pdfmellin_.A2DV_in[11] = 0.39283352148177819; pdfmellin_.A2DV_in[12] = 0.38830743520250754; pdfmellin_.A2DV_in[13] = 0.38830743520250754;

// pdfmellin_.A3DV_in[0] = 4.8210777290567783; pdfmellin_.A3DV_in[1] = 5.1269875176610054; pdfmellin_.A3DV_in[2] = 4.9965210638698023; pdfmellin_.A3DV_in[3] = 5.1016775083349204; pdfmellin_.A3DV_in[4] = 5.2823968277729447; pdfmellin_.A3DV_in[5] = 4.6352786479669454; pdfmellin_.A3DV_in[6] = 4.6352786479669454; pdfmellin_.A3DV_in[7] = 6.2015915109354331; pdfmellin_.A3DV_in[8] = 6.6096758958491719; pdfmellin_.A3DV_in[9] = 6.6889774279213423; pdfmellin_.A3DV_in[10] = 6.7302683525479239; pdfmellin_.A3DV_in[11] = 6.7258258390570154; pdfmellin_.A3DV_in[12] = 6.7522240475246544; pdfmellin_.A3DV_in[13] = 6.7522240475246544;

// pdfmellin_.A4DV_in[0] = -2.3807064538577194; pdfmellin_.A4DV_in[1] =  4.5388858341096991; pdfmellin_.A4DV_in[2] =  4.4947061349220228; pdfmellin_.A4DV_in[3] =  4.1183500458449949; pdfmellin_.A4DV_in[4] =  3.9815896248014866; pdfmellin_.A4DV_in[5] =  3.3995418062388154; pdfmellin_.A4DV_in[6] =  3.3995418062388154; pdfmellin_.A4DV_in[7] =  3.8333700755124447; pdfmellin_.A4DV_in[8] =  3.8895470653386539; pdfmellin_.A4DV_in[9] =  4.0716007127975935; pdfmellin_.A4DV_in[10] =  4.0691171708794123; pdfmellin_.A4DV_in[11] =  4.1447371477965582; pdfmellin_.A4DV_in[12] =  4.2584837050887607; pdfmellin_.A4DV_in[13] =  4.2584837050887607;

// pdfmellin_.A5DV_in[0] = 4.0548284640925658; pdfmellin_.A5DV_in[1] = 1.7207775453772669; pdfmellin_.A5DV_in[2] = .48362766864251938; pdfmellin_.A5DV_in[3] = .52151800196992326; pdfmellin_.A5DV_in[4] = 1.6752751306510589; pdfmellin_.A5DV_in[5] = 2.1317062063439636; pdfmellin_.A5DV_in[6] = 2.1317062063439636; pdfmellin_.A5DV_in[7] = 1.7719406886700213; pdfmellin_.A5DV_in[8] = 1.9088277724394374; pdfmellin_.A5DV_in[9] = 1.6851531043236392; pdfmellin_.A5DV_in[10] = 1.6964298410317440; pdfmellin_.A5DV_in[11] = 1.6661408408769134; pdfmellin_.A5DV_in[12] = 1.7304573848926381; pdfmellin_.A5DV_in[13] = 1.7304573848926381;

// pdfmellin_.A6DV_in[0] = 1.2850091387489186; pdfmellin_.A6DV_in[1] = -1.1066070667837522; pdfmellin_.A6DV_in[2] = -3.2356085430582482; pdfmellin_.A6DV_in[3] = -3.4064311696132563; pdfmellin_.A6DV_in[4] = -6.1035893806633510; pdfmellin_.A6DV_in[5] = -7.5757189545501262; pdfmellin_.A6DV_in[6] = -7.5757189545501262; pdfmellin_.A6DV_in[7] = -2.8937665785466464; pdfmellin_.A6DV_in[8] = -2.6607750588581496; pdfmellin_.A6DV_in[9] = -2.5990193117991822; pdfmellin_.A6DV_in[10] =-2.3982380140101469; pdfmellin_.A6DV_in[11] = -2.6170459814664184; pdfmellin_.A6DV_in[12] = -2.6324915897797467; pdfmellin_.A6DV_in[13] = -2.6324915897797467;

// pdfmellin_.A7DV_in[0] = -6.3401038266721343; pdfmellin_.A7DV_in[1] = -15.459292857973693; pdfmellin_.A7DV_in[2] = -9.3700906157165917; pdfmellin_.A7DV_in[3] = -7.9747688975980537; pdfmellin_.A7DV_in[4] = -11.319624754705160; pdfmellin_.A7DV_in[5] = -11.625882873826328; pdfmellin_.A7DV_in[6] = -11.625882873826328; pdfmellin_.A7DV_in[7] = -4.3811125515175320; pdfmellin_.A7DV_in[8] = -3.4322532178971699; pdfmellin_.A7DV_in[9] = -2.5707206515893688; pdfmellin_.A7DV_in[10] = -2.2449545959599533; pdfmellin_.A7DV_in[11] = -2.4399766542318133; pdfmellin_.A7DV_in[12] = -2.5269478502486322; pdfmellin_.A7DV_in[13] = -2.5269478502486322;

// pdfmellin_.A8DV_in[0] = 4.3542597728671382; pdfmellin_.A8DV_in[1] = 13.829497414820068; pdfmellin_.A8DV_in[2] = 9.3706840168242991; pdfmellin_.A8DV_in[3] = 8.5552432092028852; pdfmellin_.A8DV_in[4] = 15.601118076691165; pdfmellin_.A8DV_in[5] = 15.559529151276692; pdfmellin_.A8DV_in[6] = 15.559529151276692; pdfmellin_.A8DV_in[7] = 13.179166754565287; pdfmellin_.A8DV_in[8] = 17.886094754201011; pdfmellin_.A8DV_in[9] = 17.066438831434965; pdfmellin_.A8DV_in[10] = 16.983459216362888; pdfmellin_.A8DV_in[11] = 17.307137190155686; pdfmellin_.A8DV_in[12] = 18.168797878749871; pdfmellin_.A8DV_in[13] = 18.168797878749871;
#endif
