#include "../include/classes.cxx"
#include "../include/definitions.phasespace.set.cxx"
#include "../qTsubtraction/header/more.h"
#include "../qTsubtraction/header/radish.h"

void observable_set::determine_integrand_CX_ncollinear_VT(phasespace_set & psi){
  Logger logger("observable_set::determine_integrand_CX_ncollinear_VT");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

#ifdef MORE
  static int initialization = 1;
  static int gg_initiated = 0;
  static int charged = 0;
  if (initialization && switch_resummation){
    string initial_state = name_process.substr(0,3);
    if (initial_state == "uu~") {
      gg_initiated = -1;
    }
    else if (initial_state == "dd~") {
      gg_initiated = -2;
    }
    else if (initial_state == "bb~") {
      gg_initiated = -12;
    }
    else if (initial_state == "gg_") {
      gg_initiated = 1;
    }
    else if (initial_state == "du~") {
      charged = 1;
      gg_initiated = -1;
    }
    else if (initial_state == "ud~") {
      charged = 1;
      gg_initiated = -2;
    }
    else {
      assert(false && "initial state for resummation not known");
    }
    initialization = 0;
  }
  if (charged) {logger << LOG_DEBUG_VERBOSE << "charged qq process" << endl;}
#endif

  double global_factor = alpha_S / pi;

  if (QT_finalstate_massive_coloured){calculate_Gamma1();}

  determine_splitting_tH1F_tH1(psi);

  double LQ = 0.;
  if(switch_resummation && QT_Qres != 0.) {
    // MW: LQ = 0 for fixed-order runs, but it is required for more (b-space) resummation
    LQ = log(psi_xbs_all[0][0] / pow(QT_Qres, 2));
  }

  for (int i_vr = 0; i_vr < value_mu_ren.size(); i_vr++){
    for (int i_mr = 0; i_mr < value_mu_ren[i_vr].size(); i_mr++){
      value_LR[i_vr][i_mr] = log(psi_xbs_all[0][0] / pow(value_mu_ren[i_vr][i_mr], 2));
    }
  }

  for (int i_vf = 0; i_vf < value_mu_fact.size(); i_vf++){
    for (int i_mf = 0; i_mf < value_mu_fact[i_vf].size(); i_mf++){
      value_LF[i_vf][i_mf] = log(psi_xbs_all[0][0] / pow(value_mu_fact[i_vf][i_mf], 2));
      for (int i_l = 0; i_l < list_combination_pdf.size(); i_l++){
	coll_tH1F_pdf[i_vf][i_mf][i_l] = value_list_pdf_factor[i_vf][i_mf][i_l][0] * coll_tH1F[i_l];
	coll_tH1_pdf[i_vf][i_mf][i_l] = value_list_pdf_factor[i_vf][i_mf][i_l][0] * coll_tH1[i_l];
      }
      value_tH1F[i_vf][i_mf] = accumulate(coll_tH1F_pdf[i_vf][i_mf].begin(), coll_tH1F_pdf[i_vf][i_mf].end(), 0.);
      value_tH1[i_vf][i_mf] = accumulate(coll_tH1_pdf[i_vf][i_mf].begin(), coll_tH1_pdf[i_vf][i_mf].end(), 0.);
      if (switch_distribution){
        for (int i_y = 0; i_y < 3; i_y++){
	  for (int i_l = 0; i_l < list_combination_pdf.size(); i_l++){
	    coll_tH1F_pdf_D[i_vf][i_mf][i_y][i_l] = value_list_pdf_factor[i_vf][i_mf][i_l][i_y] * coll_tH1F[i_l];
	    coll_tH1_pdf_D[i_vf][i_mf][i_y][i_l] = value_list_pdf_factor[i_vf][i_mf][i_l][i_y] * coll_tH1[i_l];
	  }
	  value_tH1F_D[i_vf][i_mf][i_y] = accumulate(coll_tH1F_pdf_D[i_vf][i_mf][i_y].begin(), coll_tH1F_pdf_D[i_vf][i_mf][i_y].end(), 0.);
	  value_tH1_D[i_vf][i_mf][i_y] = accumulate(coll_tH1_pdf_D[i_vf][i_mf][i_y].begin(), coll_tH1_pdf_D[i_vf][i_mf][i_y].end(), 0.);
        }
      }
    }
  }

  for (int i_vf = 0; i_vf < value_mu_fact.size(); i_vf++){
    for (int i_mf = 0; i_mf < value_mu_fact[i_vf].size(); i_mf++){
      for (int i_vr = 0; i_vr < value_mu_ren.size(); i_vr++){
	for (int i_mr = 0; i_mr < value_mu_ren[i_vr].size(); i_mr++){
	  logger << LOG_DEBUG_VERBOSE << "QT_H1_delta                 = " << QT_H1_delta << endl;
	  logger << LOG_DEBUG_VERBOSE << "value_LR          [" << i_vr << "]   [" << i_mr << "] = " << value_LR[i_vr][i_mr] << endl;
	  logger << LOG_DEBUG_VERBOSE << "value_LF       [" << i_vf << "]   [" << i_mf << "]    = " << value_LF[i_vf][i_mf] << endl;
	  logger << LOG_DEBUG_VERBOSE << "value_tH1      [" << i_vf << "]   [" << i_mf << "]    = " << value_tH1[i_vf][i_mf] << endl;
	  logger << LOG_DEBUG_VERBOSE << "value_tH1F     [" << i_vf << "]   [" << i_mf << "]    = " << value_tH1F[i_vf][i_mf] << endl;
	}
      }
    }
  }

  if (switch_radish) {
#ifdef RADISH
// {{{ call of RadISH event-by-event
    double Q = sqrt(psi_xbs_all[0][0]);
    double muF = var_mu_fact;
    double muRen = var_mu_ren;
    // // mapping:
    // // index1 = -5, .., 0, .., 5  -- b-bar, c-bar, s-bar, u-bar, d-bar, gluon, d, u, s, c, b
    // // index2 = -5, .., 0, .., 5  -- b-bar, c-bar, s-bar, u-bar, d-bar, gluon, d, u, s, c, b
    // // (index1 + 5) + (index2 + 5)*11
    // if (index_1 == 0){ // gluon
    //   msqB[60]  = VA_b_ME2; // g g --  0, 0
    //   msqV1[60] = QT_H1_delta;
    //   msqV2[60] = QT_H2_delta;
    // }
    // else if (index_1 == 1){ // d-type
    //   msqB[10]   = VA_b_ME2; // b b-bar --  5,-5
    //   msqV1[10]  = QT_H1_delta;
    //   msqV2[10]  = QT_H2_delta;
    //   msqB[30]   = VA_b_ME2; // s s-bar --  3,-3
    //   msqV1[30]  = QT_H1_delta;
    //   msqV2[30]  = QT_H2_delta;
    //   msqB[50]   = VA_b_ME2; // d d-bar --  1,-1
    //   msqV1[50]  = QT_H1_delta;
    //   msqV2[50]  = QT_H2_delta;
    //   msqB[70]   = VA_b_ME2; // d-bar d -- -1, 1
    //   msqV1[70]  = QT_H1_delta;
    //   msqV2[70]  = QT_H2_delta;
    //   msqB[90]   = VA_b_ME2; // s-bar s -- -3, 3
    //   msqV1[90]  = QT_H1_delta;
    //   msqV2[90]  = QT_H2_delta;
    //   msqB[110]  = VA_b_ME2; // b-bar b -- -5, 5
    //   msqV1[110] = QT_H1_delta;
    //   msqV2[110] = QT_H2_delta;
    // }
    // else if (index_1 == 2){ // u-type
    //   msqB[20]   = VA_b_ME2; // c c-bar --  4,-4
    //   msqV1[20]  = QT_H1_delta;
    //   msqV2[20]  = QT_H2_delta;
    //   msqB[40]   = VA_b_ME2; // u u-bar --  2,-2
    //   msqV1[40]  = QT_H1_delta;
    //   msqV2[40]  = QT_H2_delta;
    //   msqB[80]   = VA_b_ME2; // u-bar u -- -2, 2
    //   msqV1[80]  = QT_H1_delta;
    //   msqV2[80]  = QT_H2_delta;
    //   msqB[100]  = VA_b_ME2; // c-bar c -- -4, 4
    //   msqV1[100] = QT_H1_delta;
    //   msqV2[100] = QT_H2_delta;
    // }

    int nf = N_f;

    double * msqB_p1p2 = new double [(2*nf+1)*(2*nf+1)];
    double * msqV1_p1p2 = new double [(2*nf+1)*(2*nf+1)];
    double * msqV2_p1p2 = new double [(2*nf+1)*(2*nf+1)];
    double * msqB_p2p1 = new double [(2*nf+1)*(2*nf+1)];
    double * msqV1_p2p1 = new double [(2*nf+1)*(2*nf+1)];
    double * msqV2_p2p1 = new double [(2*nf+1)*(2*nf+1)];

    // should be the default of the initialization above,
    // but to make sure there is no compiler dependence, we set them all to zero explicitly:
    for (int i = 0; i < (2*nf+1)*(2*nf+1); ++i) {
      msqB_p1p2[i] = 0;
      msqV1_p1p2[i] = 0;
      msqV2_p1p2[i] = 0;
      msqB_p2p1[i] = 0;
      msqV1_p2p1[i] = 0;
      msqV2_p2p1[i] = 0;
    }

    bool p1p2 = false;
    bool p2p1 = false;
    logger << LOG_DEBUG << "-------------- type partons ------------" << endl;
    // convert combination_pdf to flavour matrix expressed as an array
    for (int i=0; i<combination_pdf.size(); i++) {
      for (int j=0; j<combination_pdf[i].size(); j++) {
    	logger << LOG_DEBUG << "contribution = " << combination_pdf[i][j] << endl;
      }
      int index1 = 0;
      int index2 = 0;
      //      if(charged) {
	if(combination_pdf[i][0] == 1){ // should also work for uncharged
	  index1 = combination_pdf[i][1];
	  index2 = combination_pdf[i][2];
	  logger << LOG_DEBUG << "index1 = " << index1 << ", index2 = " << index2 << endl;
	} else{
	  index1 = combination_pdf[i][2];
	  index2 = combination_pdf[i][1];
	  logger << LOG_DEBUG << "index1 = " << index1 << ", index2 = " << index2 << endl;
	}
	//      } else{
	//	index1 = combination_pdf[i][1] * combination_pdf[i][0];
	//	index2 = combination_pdf[i][2] * combination_pdf[i][0];
	// }
      int mapping = (index1 + nf)*(2*nf+1) + (index2 + nf) ;
      logger << LOG_DEBUG << "index1 = " << index1 << endl;
      logger << LOG_DEBUG << "index2 = " << index2 << endl;
      logger << LOG_DEBUG << "mapping = " << mapping << endl;
      // for the conversion from hard-scheme to MSbar:
      double C1 = 0.;
      double C2 = 0.;
      double Lq = 0.;
      if (process == "DY" || process == "Wp" || process == "Wm") { // quark initiated
	C1 = (-pi2/12. * C_F)/2.;
	C2 = ((C_F*(9.*C_F*pi2*pi2 + 2.*C_A*(4856. - 603.*pi2 + 18.*pi2*pi2 - 2772.*zeta3) + 4.*N_f*(-328. + 45. * pi2 + 252. * zeta3)))/2592.)/4.;
      }
      else if (process == "H"){ // gluon initiated
	C1 = (-pi2/12. * C_A)/2.;
	Lq = 0.; // this should be irrelevant for the conversion!
	C2 = (-12.405092592592593 - (5.*C_A)/192. - C_F/24. + (9.*C_F*C_F)/8. - (137.*Lq)/48. - (1679*pi2)/192. - (37.*pi2*pi2)/64. + C_A*C_F*(-3.0208333333333335 - (11.*Lq)/16. - (7.*pi2)/16.) + C_A*C_A*(5.532986111111111 + (7.*Lq)/16. + (43.*pi2)/36. + (79.*pi2*pi2)/1152. - (55.*zeta3)/36.) + C_A*N_f*(-0.9965277777777778 - (5.*pi2)/72. - (2.*zeta3)/9.) + C_F*N_f*(-0.8541666666666666 + Lq/4. + zeta3/2.) + (499.*zeta3)/48.);
      }
      else{
	logger << LOG_INFO << "no proper process selected" << endl;
	logger << LOG_INFO << "process = " << process << endl;
	assert(false);
      }
      if(combination_pdf[i][0] == 1){
	msqB_p1p2[mapping] = VA_b_ME2;
	msqV1_p1p2[mapping] = VA_b_ME2 * 2. * (QT_H1_delta - 2.*C1);
	msqV2_p1p2[mapping] = VA_b_ME2 * 4. * (QT_H2_delta - C1*C1 - 2. * C2 - 2. * C1 * (QT_H1_delta - 2.*C1));
	p1p2 = true;
      } else{
      	msqB_p2p1[mapping] = VA_b_ME2;
	msqV1_p2p1[mapping] = VA_b_ME2 * 2. * (QT_H1_delta - 2.*C1);
	msqV2_p2p1[mapping] = VA_b_ME2 * 4. * (QT_H2_delta - C1*C1 - 2. * C2 - 2. * C1 * (QT_H1_delta - 2.*C1));
	p2p1 = true;
      }
      //      cout << VA_b_ME2 << " " << 2. * (QT_H1_delta - 2. * C1) << " " << 4. * (QT_H2_delta - C1*C1 - 2 * C2 - 2 * C1 * (QT_H1_delta - 2.*C1)) << endl;
    }

    // cout << VA_b_ME2 << endl;
    // cout << QT_H1_delta << endl;
    // cout << QT_H2_delta << endl;

    logger << LOG_DEBUG << "-------------------  p_parton - cme frame  -----------------" << endl;
    // center of mass frame
    for (int b=1; b<p_parton[0].size(); b++) {
      logger << LOG_DEBUG << p_parton[0][b] << endl;
    }

    // p_parton[0][1] // particle_event[0][0][i_p].momentum // p1 = pa (initial state)
    // p_parton[0][2] // particle_event[0][1][i_p].momentum // p2 = pb (initial state)
    // p_parton[0][3] // particle_event[0][3][i_p].momentum // p3 (final state)
    // p_parton[0][4] // particle_event[0][4][i_p].momentum // p4 (final state)
    // p_parton[0][5] // particle_event[0][5][i_p].momentum // p5 (final state)
    // p_parton[0][6] // particle_event[0][6][i_p].momentum // p6 (final state)
    //                // ...
    logger << LOG_DEBUG << "-------------------  particle event - lab frame  -----------------" << endl;
    // lab frame
    int npart = particle_event[0][0].size()-1;
    double *pborn_p1p2  = new double [npart*4];
    double *pborn_p2p1  = new double [npart*4];
    double E, x, y, z, entry;
    for (int i_p=0; i_p<npart; i_p++) {
      logger << LOG_DEBUG << particle_event[0][0][i_p+1].momentum << endl;
      E = particle_event[0][0][i_p+1].momentum.x0();
      x = particle_event[0][0][i_p+1].momentum.x1();
      y = particle_event[0][0][i_p+1].momentum.x2();
      z = particle_event[0][0][i_p+1].momentum.x3();
      for (int index=0; index<4; index++) {
    	if      (index == 0){entry = x;}
    	else if (index == 1){entry = y;}
    	else if (index == 2){entry = z;}
    	else if (index == 3){entry = E;}
    	pborn_p1p2[index+4*i_p] = entry;
    	if      (index == 0){entry = x;}
    	else if (index == 1){entry = -y;}
    	else if (index == 2){entry = -z;}
    	else if (index == 3){entry = E;}
    	pborn_p2p1[index+4*i_p] = entry;
      }
    }
    for (int i=0; i<4; i++) {
      double tmp = pborn_p2p1[4+i];
      pborn_p2p1[4+i] = pborn_p2p1[i];
      pborn_p2p1[i] = tmp;
    }

    for (int i=0; i<npart*4; i++) {
      logger << LOG_DEBUG << "pborn_p1p2[" << i << "] = " << pborn_p1p2[i] << endl;
      logger << LOG_DEBUG << "pborn_p2p1[" << i << "] = " << pborn_p2p1[i] << endl;
    }


    // int len = 10;
    // double *sigma_res = new double [len];
    // double *sigma_err = new double [len];
    // double ** sigma_exp = new double * [3];
    // for (int i = 0; i < 3; ++i)
    //   sigma_exp[i] = new double [len];
    // double lumi_asympt[3];

    //    double shower_wgt = 0;

    // call to RADISH:
    // MW: reset scales if doing automatic scale variation!
    //    reset_dynamical_parameters(&muRen,&muF,&Q);
    // cout << "muR = " << muRen << endl;
    // cout << "muF = " << muF << endl;
    // cout << "Q = " << Q << endl;
    //    gen_rad_matrix(len, pborn, npart, msqB, msqV1, msqV2, nf, &psi.g_tot, &shower_wgt);
    int *particle_id_dummy = new int [npart];
    for (int i_p=0; i_p<npart; i_p++) {
      particle_id_dummy[i_p] = 1;
    }
    //    gen_rad_matrix(pborn, particle_id_dummy, npart, msqB, msqV1, msqV2, nf, &psi_ps_factor, &shower_wgt,1);
                                                    // FIX THIS FOR HIGGS; maybe by multiplying with alphas^2/PI^?  CORRECT FOR scale in alphas

    // MW: add debug output option?
    // MW: what do we need len still for?

    //    logger << LOG_DEBUG << "shower_wgt: " << shower_wgt << endl;


    // MW: event-by-event inputs:
    //   --> momenta (2->n !) ==> DONE
    //   --> muR and muF ==> DONE
    //   --> Born, 1-loop and 2-loop as a matrix in i,j = -5,...,0,...,5; 0=g, 1=u, -1=u-bar,... ==> DONE?
    //   --> What is the weight we have to pass??? -- > pass jacobian --> psi.g_tot ==> DONE

    // MW: returned event-by-event:
    //   --> weight from RADISH which is integrated over; ==> DONE
    //   --> muR, muF variation has to be done one after the other, works if dynamical muR, muF works ==> OPEN

    // MW: other things:
    //   --> fix hard-coded path in Radish expansion.f90
    //   --> momenta: for 2--> x,y is always =0, problem? ==> NO?

    // MW: do we need to terminate radish at the very end?
    // terminate_radish(); ==> put at very end;
    // MW: take care of histograms, so far only in memory, give back? write out?

    // MW: the number of events that are generated should be passed ==> NOT SURE, DO WE NEED THIS FOR BINNING?

    // MW: ADD to inititalize or gen_rad_matrix whether to bin or not

    // CKM?

    int scale_index = 0;
    int res_scale_var = user.switch_value[user.switch_map["scale_variation_res"]];
    for (int i_vr = 0; i_vr < value_mu_ren.size(); i_vr++){
      for (int i_mr = 0; i_mr < value_mu_ren[i_vr].size(); i_mr++){
	for (int i_vf = 0; i_vf < value_mu_fact.size(); i_vf++){
	  for (int i_mf = 0; i_mf < value_mu_fact[i_vf].size(); i_mf++){
	    for (int i_q = 0; i_q < output_n_qTcut; i_q++){
	      //    double Qres_ratio = 0.5;
	      //    set_q_over_m(double *Q_over_M);

	      //	      CX_value_integrand_RF_qTcut[i_q][i_vr][i_vf][i_mr][i_mf] = VA_b_ME2 * value_list_pdf_factor[i_vf][i_mf][0][0];

	      // cout << "shower_wgt = " << shower_wgt << endl;
	      // cout << "ME2 * PDFs = " << VA_b_ME2 * value_list_pdf_factor[i_vf][i_mf][0][0] << endl;

	      // cout << "original weight = " << global_factor * calculate_H2(value_list_pdf_factor[i_vf][i_mf][0][0], value_tH1[i_vf][i_mf], value_tH1F[i_vf][i_mf], value_H1full[i_vr][i_vf][i_mr][i_mf], value_sig11[i_vf][i_mf], value_tgaga[i_vf][i_mf], value_tcga[i_vf][i_mf], value_tgamma2[i_vf][i_mf], value_tH2[i_vf][i_mf], value_LR[i_vr][i_mr], value_LF[i_vf][i_mf], LQ) << endl;

	      if(i_mr == 0 && i_mf == 2){scale_index = 1;}
	      if(i_mr == 0 && i_mf == 0){scale_index = 2;}
	      if(i_mr == 0 && i_mf == 1){scale_index = 3;}
	      if(i_mr == 2 && i_mf == 1){scale_index = 4;}
	      if(i_mr == 1 && i_mf == 1){scale_index = 5;}
	      if(i_mr == 1 && i_mf == 2){scale_index = 6;}
	      if(i_mr == 1 && i_mf == 0){scale_index = 7;}
	      if(i_mr == 2 && i_mf == 2){scale_index = 8;}
	      if(i_mr == 2 && i_mf == 0){scale_index = 9;}
	      if (n_scales_CV == 7){scale_index--;}
	      if (n_scales_CV == 1){scale_index = 1;}

	      if (n_scales_CV == 1 && !(i_mf == 0 && i_mr == 0)) {
		CX_value_integrand_RF_qTcut[i_q][i_vr][i_vf][i_mr][i_mf] = 0.;
	      }
	      else if (n_scales_CV == 7 && ((i_mf == 2 && i_mr == 0) || (i_mf == 0 && i_mr == 2))) {
		CX_value_integrand_RF_qTcut[i_q][i_vr][i_vf][i_mr][i_mf] = 0.;
	      }
	      else {
		logger << LOG_DEBUG << " =========== scale variation of this event =========== " << endl;
		logger << LOG_DEBUG << "i_vr = " <<  i_vr << endl;
		logger << LOG_DEBUG << "i_mr = " <<  i_mr << endl;
		logger << LOG_DEBUG <<  "value_mu_ren[i_vr][i_mr] = " <<  value_mu_ren[i_vr][i_mr] << endl;
		logger << LOG_DEBUG << "i_vf = " <<  i_vf << endl;
		logger << LOG_DEBUG << "i_mf = " <<  i_mf << endl;
		logger << LOG_DEBUG <<  "value_mu_fact[i_vf][i_mf] = " <<  value_mu_fact[i_vf][i_mf] << endl;
		logger << LOG_DEBUG << "scale_index = " <<  scale_index << endl;

		muRen = value_mu_ren[i_vr][i_mr];
		muF = value_mu_fact[i_vf][i_mf];
		bool Qres_over_Q_updated = false;
		double Qres_over_Q = QT_Qres/Q; // ratio used to be able to switch from dynamical scale (assumed) also to fixed scale
		if (Qres_over_Q != Qres_over_Q_sav){
		  logger << LOG_DEBUG << "resetting Qres_over_Q ratio = " << Qres_over_Q << ", Qres_over_Q_sav = " << Qres_over_Q_sav << endl;
		  set_q_over_m(&Qres_over_Q);
		  Qres_over_Q_sav = Qres_over_Q;
		  Qres_over_Q_updated = true;
		}
		if (muRen_sav != muRen || muF != muF_sav || Q != Q_sav || Qres_over_Q_updated) {
		  logger << LOG_DEBUG << "resetting muR, muF, Q = " << muRen << ", " << muF << ", " << Q << endl;
		  update_dynamical_parameters(&muRen,&muF,&Q); // resetting of muR, muF, and the invariant mass of the colorless system Q !
		  muRen_sav = muRen;
		  muF_sav = muF;
		  Q_sav = Q;
		}

		double shower_wgt_p1p2 = 0;
		logger << LOG_DEBUG_VERBOSE << "================================================================================" << endl;
		logger << LOG_DEBUG_VERBOSE << "p1p2 = " <<  p1p2 << "; PDF = " << value_list_pdf_factor[i_vf][i_mf][0][0] << endl;
		for (int i = 0; i < (2*nf+1)*(2*nf+1); ++i) {
		  logger << LOG_DEBUG_VERBOSE << "msqB_p1p2[" << i << "]*PDF = " <<  msqB_p1p2[i]*value_list_pdf_factor[i_vf][i_mf][0][0] << endl;
		}
		if (p1p2) {
		  gen_rad_matrix(pborn_p1p2, particle_id_dummy, npart, msqB_p1p2, msqV1_p1p2, msqV2_p1p2, nf, &psi_ps_factor, &shower_wgt_p1p2,scale_index);
		  if (std::isnan(shower_wgt_p1p2)){shower_wgt_p1p2 = 0.; logger << LOG_INFO << "shower_wgt_p1p2 from RadISH is nan, setting to zero!" << endl;}
		}
 		logger << LOG_DEBUG_VERBOSE << "shower_wgt_p1p2 = " <<  shower_wgt_p1p2 << endl;


		logger << LOG_DEBUG_VERBOSE << "p2p1 = " <<  p1p2 << endl;
		for (int i = 0; i < (2*nf+1)*(2*nf+1); ++i) {
		  logger << LOG_DEBUG_VERBOSE << "msqB_p2p1[" << i << "]*PDF = " <<  msqB_p2p1[i]*value_list_pdf_factor[i_vf][i_mf][0][0] << endl;
		}
		double shower_wgt_p2p1 = 0;
		if (p2p1) {
		  gen_rad_matrix(pborn_p2p1, particle_id_dummy, npart, msqB_p2p1, msqV1_p2p1, msqV2_p2p1, nf, &psi_ps_factor, &shower_wgt_p2p1,scale_index);
		  if (std::isnan(shower_wgt_p2p1)){shower_wgt_p2p1 = 0.; logger << LOG_INFO << "shower_wgt_p2p1 from RadISH is nan, setting to zero!" << endl;}
		}
 		logger << LOG_DEBUG_VERBOSE << "shower_wgt_p2p1 = " <<  shower_wgt_p2p1 << endl;
 		logger << LOG_DEBUG_VERBOSE << "shower_wgt_p1p2+shower_wgt_p2p1 = " <<  shower_wgt_p1p2+shower_wgt_p2p1 << endl;

		logger << LOG_DEBUG_VERBOSE << "msqB_p2p1[" << 10 << "]*PDF = " <<  msqB_p2p1[10]*value_list_pdf_factor[i_vf][i_mf][0][0] << endl;
		logger << LOG_DEBUG_VERBOSE << "msqB_p2p1[" << 39 << "]*PDF = " <<  msqB_p2p1[39]*value_list_pdf_factor[i_vf][i_mf][0][0] << endl;

		logger << LOG_DEBUG << "muF=" << muF << ", " << "muR=" << muRen << ", "  << "Q=" << Q << ", Qres=" << QT_Qres << ", Qres_over_Q=" << Qres_over_Q << endl;
		logger << LOG_DEBUG_VERBOSE << "shower_wgt_p1p2: " << shower_wgt_p1p2 << endl;
		logger << LOG_DEBUG_VERBOSE << "shower_wgt_p2p1: " << shower_wgt_p2p1 << endl;

		CX_value_integrand_RF_qTcut[i_q][i_vr][i_vf][i_mr][i_mf] = shower_wgt_p1p2+shower_wgt_p2p1;

		//if (n_scales_CV == 7 && scale_index==4 && res_scale_var) {
		//  scale_index = 8;
		//}
		//else if (n_scales_CV == 9 && scale_index==5 && res_scale_var) {
		//  scale_index = 10;
		//}
		if ((n_scales_CV == 7 && scale_index==4 && res_scale_var) || (n_scales_CV == 9 && scale_index==5 && res_scale_var)) {
		  if (n_scales_CV == 7 ) scale_index = 8;
                  if (n_scales_CV == 9 ) scale_index = 10;
                  Qres_over_Q = QT_Qres/Q/user.cut_value[user.cut_map["variation_factor_res"]];
		  set_q_over_m(&Qres_over_Q);
		  Qres_over_Q_sav = Qres_over_Q;
		  update_dynamical_parameters(&muRen,&muF,&Q); // resetting of muR, muF, and the invariant mass of the colorless system Q !
		  logger << LOG_DEBUG << "resummation scale variation 1" << endl;
		  logger << LOG_DEBUG << "muF=" << muF << ", " << "muR=" << muRen << ", "  << "Q=" << Q << ", Qres=" << QT_Qres << ", Qres_over_Q=" << Qres_over_Q << endl;
		  shower_wgt_p2p1 = 0;
                  shower_wgt_p1p2 = 0;
                  if (p1p2) {
		    gen_rad_matrix(pborn_p1p2, particle_id_dummy, npart, msqB_p1p2, msqV1_p1p2, msqV2_p1p2, nf, &psi_ps_factor, &shower_wgt_p1p2,scale_index);
		  }
		  if (p2p1) {
		    gen_rad_matrix(pborn_p2p1, particle_id_dummy, npart, msqB_p2p1, msqV1_p2p1, msqV2_p2p1, nf, &psi_ps_factor, &shower_wgt_p2p1,scale_index);
		  }
		  Qres_over_Q = QT_Qres/Q*user.cut_value[user.cut_map["variation_factor_res"]];
		  set_q_over_m(&Qres_over_Q);
		  Qres_over_Q_sav = Qres_over_Q;
		  update_dynamical_parameters(&muRen,&muF,&Q); // resetting of muR, muF, and the invariant mass of the colorless system Q !
		  logger << LOG_DEBUG << "resummation scale variation 2" << endl;
		  logger << LOG_DEBUG << "muF=" << muF << ", " << "muR=" << muRen << ", "  << "Q=" << Q << ", Qres=" << QT_Qres << ", Qres_over_Q=" << Qres_over_Q << endl;
		  shower_wgt_p2p1 = 0;
                  shower_wgt_p1p2 = 0;
                  if (p1p2) {
		    gen_rad_matrix(pborn_p1p2, particle_id_dummy, npart, msqB_p1p2, msqV1_p1p2, msqV2_p1p2, nf, &psi_ps_factor, &shower_wgt_p1p2,scale_index+1);
		  }
		  if (p2p1) {
		    gen_rad_matrix(pborn_p2p1, particle_id_dummy, npart, msqB_p2p1, msqV1_p2p1, msqV2_p2p1, nf, &psi_ps_factor, &shower_wgt_p2p1,scale_index+1);
		  }		  
		  // MW: not needed as Qres_over_Q_sav is set
                  // // restore Qres_over_Q
                  // Qres_over_Q = QT_Qres/Q;
                  // set_q_over_m(&Qres_over_Q);
		}
	      }
	    }
	  }
	}
      }
    }

    integrand = psi_ps_factor * CX_value_integrand_RF_qTcut[0][dynamic_scale][dynamic_scale][map_value_scale_ren][map_value_scale_fact];

    if (switch_CV){
      for (int i_s = 0; i_s < n_scales_CV; i_s++){
        integrand_CV[i_s] = psi_ps_factor * CX_value_integrand_RF_qTcut[0][dynamic_scale_CV][dynamic_scale_CV][map_value_scale_ren_CV[i_s]][map_value_scale_fact_CV[i_s]];
        for (int i_q = 0; i_q < output_n_qTcut; i_q++){
          integrand_qTcut_CV[i_q][i_s] = psi_ps_factor * CX_value_integrand_RF_qTcut[i_q][dynamic_scale_CV][dynamic_scale_CV][map_value_scale_ren_CV[i_s]][map_value_scale_fact_CV[i_s]];
        }
      }
    }

    if (switch_distribution){
      for (int i_y = 0; i_y < 3; i_y++){
	integrand_D[i_y][0] = psi_ps_factor * CX_value_integrand_RF_qTcut[0][dynamic_scale][dynamic_scale][map_value_scale_ren][map_value_scale_fact];
        if (i_y > 0){integrand_D[i_y][0] /= 2;}  //         TODO: asymmetric distributions not implemented !!! (refers to issues pointed out before...)

        if (switch_CV){
          for (int i_s = 0; i_s < n_scales_CV; i_s++){
	    integrand_D_CV[i_s][i_y][0] = psi_ps_factor * CX_value_integrand_RF_qTcut[0][dynamic_scale_CV][dynamic_scale_CV][map_value_scale_ren_CV[i_s]][map_value_scale_fact_CV[i_s]];
    	    if (i_y > 0){integrand_D_CV[i_s][i_y][0] /= 2;}  //         TODO: asymmetric distributions not implemented !!! (refers to issues pointed out before...)

          }
        }
      }
    }
// }}}
#endif
  }
  // MW: if RadISH and MoRE could be compiled within one code this is not correct anymore as for radish it would call both, should introduce switch_more in that case --> maybe we can just use switch_resummation for MoRE
  // if (switch_more) {
  else if (switch_resummation){
#ifdef MORE
// {{{ call of MoRE event-by-event
    double Q = sqrt(psi_xbs_all[0][0]);
    double y = log(sqrt(psi_x_pdf[1]/psi_x_pdf[2]));
    double muF = var_mu_fact;
    double muRen = var_mu_ren;
    int channel = 0;
    double sqrt_shad = 2 * psi_E;
    double qt = sqrt(psi_QT_qt2);
    //  performQTBoost(psi_QT_qt2, Q, y, particle_event);
    logger << LOG_DEBUG << "Q=" << Q << ", " << "y=" << y << ", " << "qt=" << qt << ", x1=" << x_pdf[1] << ", x2=" << x_pdf[2] << endl;

    double temp = 0.;
    if (abs(y) > -0.5 * log(pow(somescale_.startscale / sqrt_shad, 2))){logger << LOG_INFO << "y too large, skipping point" << endl;}
    else {
      logger << LOG_DEBUG << "muF=" << muF << ", " << "muR=" << muRen << ", " << "sqrts=" << sqrt_shad << ", born=" << VA_b_ME2 << ", H1=" << QT_H1_delta << ", H2=" << QT_H2_delta << endl;
      resumm_(&temp, &gg_initiated, &channel, &Q, &y, &qt, &QT_Qres, &muF, &muRen, &sqrt_shad, &VA_b_ME2, &QT_H1_delta, &QT_H2_delta);
      if (std::isnan(temp)){temp = 0.; logger << LOG_INFO << "CX_value_integrand_RF_qTcut[i_q][i_vr][i_vf][i_mr][i_mf] is nan, setting to zero!" << endl;}
    }
    for (int i_vr = 0; i_vr < value_mu_ren.size(); i_vr++){
      for (int i_mr = 0; i_mr < value_mu_ren[i_vr].size(); i_mr++){
	for (int i_vf = 0; i_vf < value_mu_fact.size(); i_vf++){
	  for (int i_mf = 0; i_mf < value_mu_fact[i_vf].size(); i_mf++){
	    for (int i_q = 0; i_q < output_n_qTcut; i_q++){
	      CX_value_integrand_RF_qTcut[i_q][i_vr][i_vf][i_mr][i_mf] = temp;
	    }
	  }
	}
      }
    }

    integrand = psi_ps_factor * CX_value_integrand_RF_qTcut[0][dynamic_scale][dynamic_scale][map_value_scale_ren][map_value_scale_fact] / x_pdf[0] * psi_QT_jacqt2 / 2 / qt;
    if (switch_CV){
      for (int i_s = 0; i_s < n_scales_CV; i_s++){
        integrand_CV[i_s] = psi_ps_factor * CX_value_integrand_RF_qTcut[0][dynamic_scale_CV][dynamic_scale_CV][map_value_scale_ren_CV[i_s]][map_value_scale_fact_CV[i_s]] / x_pdf[0] * psi_QT_jacqt2 / 2 / qt;
	// probably wrong !!!
        for (int i_q = 0; i_q < output_n_qTcut; i_q++){
          integrand_qTcut_CV[i_q][i_s] = psi_ps_factor * CX_value_integrand_RF_qTcut[0][dynamic_scale_CV][dynamic_scale_CV][map_value_scale_ren_CV[i_s]][map_value_scale_fact_CV[i_s]] / x_pdf[0] * psi_QT_jacqt2 / 2 / qt;
        }
      }
    }

    if (switch_distribution){
      for (int i_y = 0; i_y < 3; i_y++){
        integrand_D[i_y][0] = psi_ps_factor * CX_value_integrand_RF_qTcut[0][dynamic_scale][dynamic_scale][map_value_scale_ren][map_value_scale_fact] / x_pdf[0] * psi_QT_jacqt2 / 2 / qt;
        if (i_y > 0){integrand_D[i_y][0] /= 2;}  //         TODO: asymmetric distributions not implemented !!! (refers to issues pointed out before...)

        if (switch_CV){
          for (int i_s = 0; i_s < n_scales_CV; i_s++){
            integrand_D_CV[i_s][i_y][0] = psi_ps_factor * CX_value_integrand_RF_qTcut[0][dynamic_scale_CV][dynamic_scale_CV][map_value_scale_ren_CV[i_s]][map_value_scale_fact_CV[i_s]] / x_pdf[0] * psi_QT_jacqt2 / 2 / qt;
	    if (i_y > 0){integrand_D_CV[i_s][i_y][0] /= 2;}  //         TODO: asymmetric distributions not implemented !!! (refers to issues pointed out before...)
          }
        }
      }
    }
// }}}
#endif
  }
  else {
    for (int i_vf = 0; i_vf < value_mu_fact.size(); i_vf++){
      for (int i_mf = 0; i_mf < value_mu_fact[i_vf].size(); i_mf++){
	for (int i_vr = 0; i_vr < value_mu_ren.size(); i_vr++){
	  for (int i_mr = 0; i_mr < value_mu_ren[i_vr].size(); i_mr++){
	    for (int i_q = 0; i_q < output_n_qTcut; i_q++){
	      CX_value_integrand_RF_qTcut[i_q][i_vr][i_vf][i_mr][i_mf] = global_factor * calculate_H1(value_list_pdf_factor[i_vf][i_mf][0][0], value_tH1[i_vf][i_mf], value_tH1F[i_vf][i_mf], value_LR[i_vr][i_mr], value_LF[i_vf][i_mf], LQ);
	      if (switch_distribution){
		for (int i_y = 0; i_y < 3; i_y++){
		  CX_value_integrand_RF_qTcut_D[i_q][i_vr][i_vf][i_mr][i_mf][i_y] = global_factor * calculate_H1(value_list_pdf_factor[i_vf][i_mf][0][i_y], value_tH1_D[i_vf][i_mf][i_y], value_tH1F_D[i_vf][i_mf][i_y], value_LR[i_vr][i_mr], value_LF[i_vf][i_mf], LQ);
		}
	      }
	    }
	  }
	}
      }
    }

    int x_q = 0;
    integrand = var_rel_alpha_S * psi_ps_factor * VA_b_ME2 * CX_value_integrand_RF_qTcut[x_q][dynamic_scale][dynamic_scale][map_value_scale_ren][map_value_scale_fact];
    if (switch_CV){
      for (int i_s = 0; i_s < n_scales_CV; i_s++){
	integrand_CV[i_s] = var_rel_alpha_S_CV[i_s] * psi_ps_factor * VA_b_ME2 * CX_value_integrand_RF_qTcut[x_q][dynamic_scale_CV][dynamic_scale_CV][map_value_scale_ren_CV[i_s]][map_value_scale_fact_CV[i_s]];
	for (int i_q = 0; i_q < output_n_qTcut; i_q++){
	  integrand_qTcut_CV[i_q][i_s] = var_rel_alpha_S_CV[i_s] * psi_ps_factor * VA_b_ME2 * CX_value_integrand_RF_qTcut[i_q][dynamic_scale_CV][dynamic_scale_CV][map_value_scale_ren_CV[i_s]][map_value_scale_fact_CV[i_s]];
	}
      }
    }

    if (switch_distribution){
      for (int i_y = 0; i_y < 3; i_y++){
	integrand_D[i_y][0] = var_rel_alpha_S * psi_ps_factor * VA_b_ME2 * CX_value_integrand_RF_qTcut_D[x_q][dynamic_scale][dynamic_scale][map_value_scale_ren][map_value_scale_fact][i_y];
	if (switch_CV){
	  for (int i_s = 0; i_s < n_scales_CV; i_s++){
	    integrand_D_CV[i_s][i_y][0] = var_rel_alpha_S_CV[i_s] * psi_ps_factor * VA_b_ME2 * CX_value_integrand_RF_qTcut_D[x_q][dynamic_scale_CV][dynamic_scale_CV][map_value_scale_ren_CV[i_s]][map_value_scale_fact_CV[i_s]][i_y];
	    for (int i_q = 0; i_q < output_n_qTcut; i_q++){
	      integrand_D_qTcut_CV[i_q][i_s][i_y][0] = var_rel_alpha_S_CV[i_s] * psi_ps_factor * VA_b_ME2 * CX_value_integrand_RF_qTcut_D[i_q][dynamic_scale_CV][dynamic_scale_CV][map_value_scale_ren_CV[i_s]][map_value_scale_fact_CV[i_s]][i_y];
	    }
	  }
	}
      }
    }
  }



  if (switch_TSV){
    for (int i_vr = 0; i_vr < max_dyn_ren + 1; i_vr++){
      for (int i_mr = 0; i_mr < n_scale_dyn_ren[i_vr]; i_mr++){
	value_LR_TSV[i_vr][i_mr] = log(psi_xbs_all[0][0] / pow(value_scale_ren[0][i_vr][i_mr], 2));
      }
    }
    for (int i_vf = 0; i_vf < max_dyn_fact + 1; i_vf++){
      for (int i_mf = 0; i_mf < n_scale_dyn_fact[i_vf]; i_mf++){
	value_LF_TSV[i_vf][i_mf] = log(psi_xbs_all[0][0] / pow(value_scale_fact[0][i_vf][i_mf], 2));
	for (int i_l = 0; i_l < list_combination_pdf.size(); i_l++){
	  for (int i_y = 0; i_y < 3; i_y++){
	    coll_tH1F_pdf_TSV[i_vf][i_mf][i_y][i_l] = value_list_pdf_factor_TSV[0][i_vf][i_mf][i_l][i_y] * coll_tH1F[i_l];
	    coll_tH1_pdf_TSV[i_vf][i_mf][i_y][i_l] = value_list_pdf_factor_TSV[0][i_vf][i_mf][i_l][i_y] * coll_tH1[i_l];
	  }
	}
	for (int i_y = 0; i_y < 3; i_y++){
	  value_tH1F_TSV[i_vf][i_mf][i_y] = accumulate(coll_tH1F_pdf_TSV[i_vf][i_mf][i_y].begin(), coll_tH1F_pdf_TSV[i_vf][i_mf][i_y].end(), 0.);
	  value_tH1_TSV[i_vf][i_mf][i_y] = accumulate(coll_tH1_pdf_TSV[i_vf][i_mf][i_y].begin(), coll_tH1_pdf_TSV[i_vf][i_mf][i_y].end(), 0.);
	}
      }
    }

    for (int i_vf = 0; i_vf < max_dyn_fact + 1; i_vf++){
      for (int i_mf = 0; i_mf < n_scale_dyn_fact[i_vf]; i_mf++){
	for (int i_vr = 0; i_vr < max_dyn_ren + 1; i_vr++){
	  for (int i_mr = 0; i_mr < n_scale_dyn_ren[i_vr]; i_mr++){
	    logger << LOG_DEBUG_VERBOSE << "QT_H1_delta                     = " << QT_H1_delta << endl;
	    logger << LOG_DEBUG_VERBOSE << "value_LR_TSV          [" << i_vr << "]   [" << i_mr << "] = " << value_LR_TSV[i_vr][i_mr] << endl;
	    logger << LOG_DEBUG_VERBOSE << "value_LF_TSV       [" << i_vf << "]   [" << i_mf << "]    = " << value_LF_TSV[i_vf][i_mf] << endl;
	    logger << LOG_DEBUG_VERBOSE << "value_tH1_TSV      [" << i_vf << "]   [" << i_mf << "]    = " << value_tH1_TSV[i_vf][i_mf][0] << endl;
	    logger << LOG_DEBUG_VERBOSE << "value_tH1F_TSV     [" << i_vf << "]   [" << i_mf << "]    = " << value_tH1F_TSV[i_vf][i_mf][0] << endl;
	    logger.newLine(LOG_DEBUG_VERBOSE);
   	  }
	}
      }
    }


    if (switch_resummation){
#ifdef MORE
      double Q = sqrt(psi_xbs_all[0][0]);
      double y = log(sqrt(psi_x_pdf[1]/psi_x_pdf[2]));
      double muF = var_mu_fact;
      double muRen = var_mu_ren;
      int channel = 0;
      double sqrt_shad = 2 * psi_E;
      double qt = sqrt(psi_QT_qt2);
      //performQTBoost(psi_QT_qt2, Q, y, particle_event);

      double temp = 0.;
      if (abs(y) > -0.5 * log(pow(somescale_.startscale / sqrt_shad, 2))){logger << LOG_INFO << "y too large, skipping point" << endl;}
      else {
	logger << LOG_DEBUG << "muF=" << muF << ", " << "muR=" << muRen << ", " << "sqrts=" << sqrt_shad << ", born=" << VA_b_ME2 << ", H1=" << QT_H1_delta << ", H2=" << QT_H2_delta << endl;
	resumm_(&temp, &gg_initiated, &channel, &Q, &y, &qt, &QT_Qres, &muF, &muRen, &sqrt_shad, &VA_b_ME2, &QT_H1_delta, &QT_H2_delta);
	if (std::isnan(temp)){temp = 0.; logger << LOG_INFO << "value_integrand_qTcut_TSV[i_q][i_vr][i_vf][i_mr][i_mf][i_y] is nan, setting to zero!" << endl;}
      }
      for (int i_vf = 0; i_vf < max_dyn_fact + 1; i_vf++){
	for (int i_mf = 0; i_mf < n_scale_dyn_fact[i_vf]; i_mf++){
	  for (int i_vr = 0; i_vr < max_dyn_ren + 1; i_vr++){
	    for (int i_mr = 0; i_mr < n_scale_dyn_ren[i_vr]; i_mr++){
	      for (int i_y = 0; i_y < 3; i_y++){
		for (int i_q = 0; i_q < output_n_qTcut; i_q++){
		  // Most likely, psi_ps_factor needs to be removed !!! What about global_factor, or VA_b_ME2?
		  value_integrand_qTcut_TSV[i_q][i_vr][i_vf][i_mr][i_mf][i_y] = psi_ps_factor * temp / x_pdf[0] * psi_QT_jacqt2 / 2 / qt;
		}
	      }
	    }
	  }
	}
      }
#endif
    }
    else {
      for (int i_vr = 0; i_vr < max_dyn_ren + 1; i_vr++){
	for (int i_mr = 0; i_mr < n_scale_dyn_ren[i_vr]; i_mr++){
	  for (int i_vf = 0; i_vf < max_dyn_fact + 1; i_vf++){
	    for (int i_mf = 0; i_mf < n_scale_dyn_fact[i_vf]; i_mf++){
	      for (int i_y = 0; i_y < 3; i_y++){
		for (int i_q = 0; i_q < output_n_qTcut; i_q++){
		  value_integrand_qTcut_TSV[i_q][i_vr][i_vf][i_mr][i_mf][i_y] = global_factor * VA_b_ME2 * calculate_H1(value_list_pdf_factor_TSV[0][i_vf][i_mf][0][i_y], value_tH1_TSV[i_vf][i_mf][i_y], value_tH1F_TSV[i_vf][i_mf][i_y], value_LR_TSV[i_vr][i_mr], value_LF_TSV[i_vf][i_mf], LQ);
		}
	      }
	    }
	  }
	}
      }
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}
