module ew_parameters
  use types; use consts_dp
  use iso_c_binding
  implicit none
  private

  ! EW masses
  real(dp), public :: MZ
  real(dp), public :: MW
  ! EW widths
  real(dp), public :: Wwidth
  real(dp), public :: Zwidth
  real(dp), public :: Hwidth
  ! EW couplings
  real(dp), public :: aemmz
  real(dp), public :: sinwsq
  real(dp), parameter, public :: invGev2_to_nb = 389379.323_dp
  real(dp), public :: higgs_vev
  real(dp), public :: GF_GeVm2
  real(dp), public :: gv2_ga2_u
  real(dp), public :: gv2_ga2_d
  real(dp), public :: gv2_ga2(6)
  ! quark masses
  real(dp), public :: ms
  real(dp), public :: mc
  real(dp), public :: mb
  real(dp), public :: mt
  ! Yukawa couplings
  real(dp), public :: yb
  real(dp), public :: yc
  real(dp), public :: ys
  ! Higgs mass
  real(dp), public :: mH
  !CKM
  real(dp), public :: Vud_in,Vus_in,Vub_in,Vcd_in,Vcs_in,Vcb_in

  public :: init_yukawas
  public :: init_default
  public :: init_external
  public :: init_masses

contains
    !======================================================================
  subroutine init_yukawas(ybottom,ycharm,ystrange) BIND(C)
    use iso_c_binding
    implicit none   
    real(dp), intent(in) :: ybottom, ycharm, ystrange
    yb=ybottom
    yc=ycharm
    ys=ystrange
  end subroutine init_yukawas

  subroutine init_default() BIND(C)
    use iso_c_binding 
    implicit none   
    MZ=91.1876_dp
    MW=80.379_dp
    Wwidth=2.085_dp
    Zwidth=2.4952_dp
    Hwidth=4.0296d-3
    aemmz=one/132.18417914 !one/137.035999074_dp !aemmz=7.55638390728735360e-03_dp
    sinwsq =  0.22301323_dp
    ! invGev2_to_nb = 389379.323_dp
    higgs_vev = 246.21846_dp 
    GF_GeVm2 = 1.1663787e-05_dp 
    gv2_ga2_u = (half-four/three*sinwsq)**2 + one/four
    gv2_ga2_d = (-half+two/three*sinwsq)**2 + one/four
    gv2_ga2 = (/gv2_ga2_d, gv2_ga2_u,&
      & gv2_ga2_d, gv2_ga2_u,&
      & gv2_ga2_d, gv2_ga2_u /)
    ms = 0.095
    mc = 1.275
    mb = 4.65
    mt = 173.5
    mH = 125d0
    Vud_in=1d0
    Vus_in=0d0
    Vub_in=0d0
    Vcd_in=0d0
    Vcs_in=1d0
    Vcb_in=0d0
  end subroutine init_default

  subroutine init_external(MZin, MWin, Wwidthin, Zwidthin, Hwidthin, aemmzin, sinwsqin, higgs_vevin, msin, mcin, mbin, mtin, mHin) BIND(C)
    use iso_c_binding    
    implicit none   
    real(dp), intent(in) :: MZin, MWin, Wwidthin, Zwidthin, Hwidthin, aemmzin, sinwsqin, higgs_vevin, msin, mcin, mbin, mtin, mHin
    MZ=MZin
    MW=MWin
    Wwidth=Wwidthin
    Zwidth=Zwidthin
    Hwidth=Hwidthin
    aemmz=aemmzin
    sinwsq =  sinwsq
    higgs_vev = higgs_vevin
    gv2_ga2_u = (half-four/three*sinwsq)**2 + one/four
    gv2_ga2_d = (-half+two/three*sinwsq)**2 + one/four
    gv2_ga2 = (/gv2_ga2_d, gv2_ga2_u,&
      & gv2_ga2_d, gv2_ga2_u,&
      & gv2_ga2_d, gv2_ga2_u /)
    ms = msin
    mc = mcin
    mb = mbin
    mt = mtin
    mH = mHin
    !TODO to be fixed
    Vud_in=1d0
    Vus_in=0d0
    Vub_in=0d0
    Vcd_in=0d0
    Vcs_in=1d0
    Vcb_in=0d0
  end subroutine init_external

  subroutine init_masses(scheme)
    implicit none
    character(len=4), intent(in) :: scheme
    if (scheme=='M_MS') then
       ms = 0.095_dp ! this is to be fixed to a realistic value if used
       mc = 1.245_dp ! MSbar mass from YR4 eq. (I.1.18). This gives mc(3 GeV)=0.986 when run in the MSbar at two loops
       mb = 4.18_dp  ! MSbar mass from YR4 eq. (I.1.15)
       !mb = 4.47_dp  ! MSbar mass for paper on bottom effects
       mt = 162.7_dp ! MSbar mass from YR4 eq. (I.1.12)
    elseif (scheme=='M_OS') then
       ms = 0.095_dp ! this is to be fixed to a realistic value if used
       mc = 1.51_dp  ! OS mass from YR4 eq. (I.1.19)
       !mb = 4.92_dp  ! OS mass from YR4 eq. (I.1.16)
       mb = 4.75_dp  ! OS mass for ATLAS runs, as in hnnlo_v2.0 (and paper on bottom effects)
       !mt = 172.5_dp ! OS mass from YR4 eq. (I.1.11)
       mt = 173.2_dp ! OS mass for paper on bottom effects
       ! OLD values for paper on Yukawa couplings
       !ms = 0.095_dp
       !mc = 1.275_dp
       !mb = 4.65_dp
       !mt = 173.5_dp       
    end if
  end subroutine init_masses
end module ew_parameters

