!--------------------------------------------------------------------
! contains the random number generator; for the time being keep it
! simple and just use the built-in one -- then replace it with
! something better shortly
module random_dists
  use types; use consts_dp
  implicit none
  integer, parameter :: dp15 = selected_real_kind(15) 

  !-- the following combination causes problems with pgf90
  !   so eliminate it, and shield the dp stuff
  !private
  !public :: ran,rangen
  private :: dp
  public :: rd_exp, rd_2d, rphi_2d

contains
  function ran()
    real(dp) :: ran
    real(dp15) :: rann(1)
    !call random_number(ran)
    !call rangen(1,rann)
    call rm48(rann,1)
    ran = rann(1)
  end function ran
 
  SUBROUTINE RANGEN(N,R)
    !IMPLICIT NONE
    !---RANDOM NUMBER GENERATOR
    !   USES METHOD OF l'Ecuyer, (VIA F.JAMES, COMP PHYS COMM 60(1990)329)
    !   RETURNS A VECTOR OF N RANDOM VALUES
    !   IF (N.EQ.0) THE FIRST TWO VALUES IN R SET THE SEEDS
    !   IF (N.LT.0) PRINT THE CURRENT VALUES OF THE SEEDS
    real(dp) r(*)
      integer n,iseed(3)
      if (n.gt.0) then
         call rm48(r,n)
      else if (n.lt.0) then
         call rm48ut(iseed(1),iseed(2),iseed(3))
         write (*,'(i10,a,i10,i11,i11)') -n-1,', iseed=',&
              &        iseed(1),iseed(2),iseed(3)
      else ! n=0
        if(nint(r(1)) .eq. 0) then
           !-- retrieve seed --
           call rm48ut(iseed(1),iseed(2),iseed(3))
           r(1) = iseed(1)
           r(2) = iseed(2)
           r(3) = iseed(3)
        else
           !-- set seed -------
           iseed(1)=nint(r(1))
           iseed(2)=nint(r(2))
           iseed(3)=nint(r(3))
           call rm48in(iseed(1),iseed(2),iseed(3))
        end if
      end if
  END SUBROUTINE RANGEN
 
!======================================================================
! Designed for generating particular distributions
  
  !---------------------------------------------------------------
  ! Returns a value for x distributed as [r exp(-rx)]
  function rd_exp(r) result (x)
    real(dp), intent(in) :: r
    real(dp)             :: x
    !--------------------------
    x = - log(ran()) / r
  end function rd_exp

  !---------------------------------------------------------------
  ! returns a 2d vector of length r with a random direction
  function rd_2d(r) result(res)
    real(dp), intent(in) :: r
    real(dp)             :: res(2)
    !------------------------------
    real(dp) :: c, s, cs, qt, pt, px, py
    do
       !call rangen(2,res)
       !C=two*res(1)-one
       !S=two*res(2)-one
       C=two*ran()-one
       S=two*ran()-one
       CS=C*C+S*S
       IF (CS <= one .and. CS /= zero) exit
    end do
    QT=r/CS
    res(1)=(C*C-S*S)*QT
    res(2)=two*C*S*QT
  end function rd_2d

  !---------------------------------------------------------------
  ! returns a 2d vector of length r with a random direction
  function rphi_2d(r) result(res)
    real(dp), intent(in) :: r
    real(dp)             :: res(2)
    !------------------------------
    real(dp) :: phi
    do
       ! generate azimuth angle
       phi = ran()*twopi-pi
       ! check that phi is an azimuthal angle
       if (abs(phi)<pi) exit
    end do
    res(1)= r * cos(phi)
    res(2)= r * sin(phi)
  end function rphi_2d


end module random_dists
