! this routine is called by the integrator
!#######################################################################
subroutine shower(pborn,msqB,msqV1,msqV2,vegas_wgt,shower_wgt,dryrun)
  use types; use consts_dp
  use rad_tools; use qcd
  use sub_defs_io
  use warnings_and_errors
  use shower_routines
  implicit none

  include 'mxpart.f'

  real(dp), intent(in) :: pborn(mxpart,4)
  real(dp), intent(in) :: msqB(-nf_int:nf_int,-nf_int:nf_int), vegas_wgt
  real(dp), intent(inout) :: msqV1(-nf_int:nf_int,-nf_int:nf_int),msqV2(-nf_int:nf_int,-nf_int:nf_int)
  real(dp), intent(inout) :: shower_wgt
  logical,  intent(out) :: dryrun
  logical   :: bin
  common/bin/bin

  dryrun = cs%dryrun
  if(cs%dryrun .or. (.not.bin)) return
  if( (cs%observable == 'ptj') .and. (cs%numerical_jetveto .eqv. .false.) ) then
    call jetveto(pborn,msqB,msqV1,msqV2,vegas_wgt,shower_wgt,mxpart)
  else
    call make_shower(pborn,msqB,msqV1,msqV2,vegas_wgt,shower_wgt,mxpart)
  endif

end subroutine shower


!#######################################################################
subroutine update_scales_and_alphas(pborn,new_muf,new_mur)
  use types; use consts_dp
  use rad_tools
  use sub_defs_io
  use warnings_and_errors
  use shower_routines
  use dynamical_scale
  implicit none

  include 'mxpart.f'
  include 'scale.f'
  include 'facscale.f'
  include 'qcdcouple.f'

  real(dp), intent(in) :: pborn(mxpart,4)
  real(dp), intent(out):: new_muR, new_muF
  real(dp):: new_Msinglet
  real(dp):: alphasPDF
  logical   :: bin
  common/bin/bin

  new_muR=cs%muR
  new_muF=cs%muF

  !--- recalculate if using dynamical scale:
  if (cs%dynamical) then
     new_muR=cs%xmuR_init*set_dynamical_scale(pborn)
     new_muF=cs%xmuF_init*set_dynamical_scale(pborn)
  endif
  if ((.not.cs%dryrun) .and. bin) then
     new_Msinglet=singlet_invariant_mass(pborn)
     call reset_dynamical_parameters(new_muR,new_muF, new_Msinglet)
     call reset_dynamical_shower()
     call reset_dynamical_expansion()
  endif

  ! update coupling
  if (cs%use_alphas_LHAPDF) then
   ! LHAPDF coupling
   as = alphasPDF(new_muR)
  else
  ! hoppet's running instead. There could numerical differences
   as = RunningCoupling(new_muR)
  endif
  ason2pi = as/(twopi)
  ason4pi = half*as/(twopi)

end subroutine update_scales_and_alphas

