module sudakov_radiators
  use types; use consts_dp
  use rad_tools; use qcd
  use observables
  implicit none

  public :: Sudakov, Radp_v, dRadp_v, Rads_v, Radt_v
  integer, save, private :: sudakov_errors = 0

contains

  !-----------------------------------------------------------------------
  function Sudakov(L, order) result(res)
    real(dp), intent(in) :: L
    integer, intent(in)  :: order
    !--------------------------
    real(dp) :: lambda, res
    !----------------------------------------
    real(dp) :: part0,part1,part2,part3
    !----------------------------------------
    integer  :: max_errors = 20
    real(dp) :: resmax = 10

    ! replace log with L_tilde eventually
    lambda=cs%alphas_muR*L*beta0

    if (lambda < half) then
       ! include the L*g1 contribution
       ! eq. (10a) [1206.4998]
       res = L*A(1)/pi/beta0*(one+half*log(one-two*lambda)/lambda)
       if (lambda == zero) res=zero ! limit needs to be taken analytically

       ! add g2
       ! eq. (10b) [1206.4998]
       if (order>=order_NLL) then
          res = res + B(1)/twopi/beta0*log(one-two*lambda)&
               &-A(2)/four/pisq/beta0**2*(two*lambda/(one-two*lambda)+log(one-two*lambda))&
               & +A(1)/twopi/beta0*((two*lambda/(one-two*lambda)&
               &+log(one-two*lambda))*cs%ln_Q2_muR2+log(one-two*lambda)*(-cs%ln_Q2_M2)) &
               & +A(1)/two*(pisq*beta1)/(pi*beta0)**3*(half*log(one-two*lambda)**2&
               &+(log(one-two*lambda)+two*lambda)/(one-two*lambda))
       end if

       ! add as/Pi*g3
       if (order>order_NLL) then
          res = res + cs%as2pi*two&
               & * (-half*A(3)/8._dp/pisq/beta0**2*(two*lambda/(one-two*lambda))**2 &
               & -(B(2)+A(2)*(-cs%ln_Q2_M2))/four/pi/beta0*two*lambda/(one-two*lambda) &
               & +A(2)/four*(pisq*beta1)/(pi*beta0)**3*(lambda*(three*two*lambda-two)/(one-two*lambda)**2 &
               & -(one-four*lambda)/(one-two*lambda)**2*log(one-two*lambda)) &
               & +(B(1)+A(1)*(-cs%ln_Q2_M2))/two*(pisq*beta1)/pisq/beta0**2&
               &        *(two*lambda/(one-two*lambda)+log(one-two*lambda)/(one-two*lambda)) &
               & -half*A(1)/two*(two*lambda/(one-two*lambda))**2*cs%ln_Q2_muR2**2 &
               & +((B(1)+A(1)*(-cs%ln_Q2_M2))/two*two*lambda/(one-two*lambda)&
               & +A(2)/four/pi/beta0*(two*lambda/(one-two*lambda))**2 &
               & +A(1)/two*pisq*beta1/pisq/beta0**2*(two*lambda/(one-two*lambda) &
               & +(one-four*lambda)/(one-two*lambda)**2*log(one-two*lambda)))*cs%ln_Q2_muR2 &
               & +A(1)/two*((pisq*beta1)**2/two/(pi*beta0)**4*(one-four*lambda)/&
               & (one-two*lambda)**2*log(one-two*lambda)**2 &
               & +log(one-two*lambda)*((pi*beta0*pi**3*beta2-(pisq*beta1)**2)/&
               &          (pi*beta0)**4+(pisq*beta1)**2/(pi*beta0)**4/(one-two*lambda))&
               & +one/(pi*beta0)**4*lambda/(one-two*lambda)**2&
               & *(pi*beta0*pi**3*beta2*(two-three*two*lambda)+(pisq*beta1)**2*two*lambda)))
       end if

       ! add (as/pi)^2 *g4
       if( order>order_NNLL )then
          part0=(&
               &   2*lambda*(4*beta0**2*pi*(&
               &      2*beta0**4*cs%ln_Q2_muR2**3*lambda*(-3 + 2*lambda)*pi**2*A(1)&
               &    - 8*beta0**4*(3 - 6*lambda + 4*lambda**2)*pi**2*zeta3*A(1)&
               &    + 3*beta0**2*cs%ln_Q2_muR2**2*pi*(&
               &         beta1*pi*A(1) + beta0*(3 - 2*lambda)*lambda*A(2)&
               &       + 2*beta0**2*(-one+lambda)*(-one+two*lambda)*pi*B(1)   )&
               &    - (3*cs%ln_Q2_muR2*(&
               &         2*beta0*beta1*(-one+lambda)*(-one+two*lambda)*pi*A(2)&
               &       + lambda*(-4*(beta1**2-beta0*beta2)*(1 + 2*lambda)*pi**2*A(1) + beta0**2*(3 - 2*lambda)*A(3))&
               &       + 4*beta0**3*(-one+lambda)*(-one+two*lambda)*pi*B(2)   )&
               &      )/2.    )&
               & + 3*beta0*pi*(&
               &      beta0*beta1*A(3) - 2*pi*(2*beta0*beta3*pi*A(1)&
               &    + beta1*(-2*beta2*pi*A(1) + beta1*A(2) + 2*beta0**2*B(2))  )&
               & + 2*beta0**3*B(3)) + 3*beta0*lambda*(&
               &      beta0**2*A(4) + pi*(&
               &        -5*beta0*beta1*A(3) + 2*pi*(3*beta1**2*A(2) + 2*pi*(&
               &            -5*beta1*beta2*A(1) + 5*beta0*beta3*A(1) + 2*beta0*(-beta1**2 + beta0*beta2)*B(1)  )&
               &        + 6*beta0**2*beta1*B(2)  ) - 6*beta0**3*B(3)  )  )&
               & + 2*lambda**2*( -(beta0**3*A(4))&
               &    + pi*(5*beta0**2*beta1*A(3) + 2*pi*(beta0*(-11*beta1**2 + 8*beta0*beta2)*A(2)&
               & + 2*pi*((2*beta1**3 + 5*beta0*beta1*beta2 - 7*beta0**2*beta3)*A(1)&
               & + 6*beta0**2*(beta1**2 - beta0*beta2)*B(1)) - 6*beta0**3*beta1*B(2)) + 6*beta0**4*B(3))))&
               & - 3*pi*(&
               &   beta0**2*beta1*(-1 + 6*lambda)*A(3)&
               & + 4*beta0**2*beta1*cs%ln_Q2_muR2*pi*(&
               &   beta0**2*cs%ln_Q2_muR2*(-1 + 6*lambda)*pi*A(1) + (beta0 - 6*beta0*lambda)*A(2)&
               & + 2*pi*(-2*beta1*lambda*A(1) + beta0**2*(-one+two*lambda)*B(1)))&
               & + 2*pi*(2*(-(beta0**2*beta3*(-one+two*lambda)**3) - 4*beta1**3*lambda**2*(1 + 2*lambda)&
               & + beta0*beta1*beta2*(-one+two*lambda*(3 - 4*lambda + 8*lambda**2)))*pi*A(1) &
               & + beta0*beta1**2*(one-two*lambda)*A(2) + 2*beta0**3*beta1*(one-two*lambda)*B(2)))*Log(one-two*lambda)&
               & + 6*beta1**2*pi**2*(&
               &   2*beta0**2*cs%ln_Q2_muR2*(1 - 6*lambda)*pi*A(1) + beta0*(-1 + 6*lambda)*A(2)&
               & + 2*pi*(2*beta1*lambda*A(1) + beta0**2*(one-two*lambda)*B(1)))*Log(one-two*lambda)**2&
               & + 4*beta1**3*(1 - 6*lambda)*pi**3*A(1)*Log(one-two*lambda)**3&
               & - 12*beta0**2*cs%ln_Q2_M2*(-one+two*lambda)*pi*(lambda*(&
               &   4*beta0**3*cs%ln_Q2_muR2*(-one+lambda)*pi*(beta0*cs%ln_Q2_muR2*pi*A(1) - A(2))&
               & + 2*pi*(2*(beta1**2 - beta0*beta2)*lambda*pi*A(1) - beta0*beta1*(-one+lambda)*A(2))&
               & + beta0**2*(-one+lambda)*A(3))&
               & + beta0*beta1*pi*(-2*beta0*cs%ln_Q2_muR2*pi*A(1) + A(2))*Log(one-two*lambda)&
               & - beta1**2*pi**2*A(1)*Log(one-two*lambda)**2))/(48._dp*beta0**5*(-one+two*lambda)**3*pi**2)

          part1=(&
               & lambda*(&
               &   beta0**2*A(3) + 4*pi*(-2*(beta1**2 - beta0*beta2)*lambda*pi*A(1) &
               & + beta0**3*(-one+lambda)*(-one+two*lambda)*B(2))&
               &)&
               & + 2*pi*(&
               & 2*beta0**4*cs%ln_Q2_muR2**2*lambda*pi*A(1)&
               & - 2*beta0**2*cs%ln_Q2_muR2*lambda*(&
               &        beta0*A(2) + pi*(beta1*A(1) + 2*beta0**2*(-one+lambda)*(-one+two*lambda)*B(1))&
               &      - 2*beta1*pi*A(1)*Log(one-two*lambda) )&
               & - beta0**2*cs%ln_Q2_M2*(-one+two*lambda)*(&
               &        2*beta0*(-one+lambda)*lambda*(-2*beta0*cs%ln_Q2_muR2*pi*A(1) + A(2))&
               &      + beta1*pi*A(1)*Log(one-two*lambda) )&
               & + beta1*Log(one-two*lambda)*(&
               &       -2*lambda*(beta1*pi*A(1) + beta0*A(2)) + beta0**2*(-one+two*lambda)*pi*B(1)&
               &      + 2*beta1*lambda*pi*A(1)*Log(one-two*lambda))&
               &)&
               &)/(2.*beta0**3*(-one+two*lambda)**3*pi)

          part2=(&
               & two*lambda*(beta0*(two + lambda*(-three + two*lambda))*A(2) &
               & + pi*(beta1*A(1) + two*beta0**2*(-one + lambda)*(-one + two*lambda)*B(1)))&
               & + pi*A(1)*(-four*beta0**2*cs%ln_Q2_M2*(-one + lambda)*lambda*(-one + two*lambda)&
               & - four*beta0**2*cs%ln_Q2_muR2*lambda*(two + lambda*(-three + two*lambda))&
               & - (beta1 + two*beta1*lambda)*log(one - two*lambda))&
               &)/(beta0*(-one + two*lambda)**3)

          part3=(4d0*beta0*(one + (-one + two*lambda)**(-3))*pi*A(1))/three
          res = res + (cs%as2pi*two)**2 *part0
       end if

    else
       res=-1e16
    end if
    res = exp(res)

    ! sanity check, it may fail when we get too close to the Landau pole.
    ! This may happen at N3LL since the g4(as*L) function grows very fast
    ! before going negative in proximity of the pole for certain scale combinations
    ! >> see N3LL_radiator_2019.nb
    if (res > two) then
       if (sudakov_errors < max_errors .and. res < resmax) then
         write(*,*) 'sudakov_radiators >> Error: too close to the Landau pole.'
         sudakov_errors = sudakov_errors + 1
       else
         write(*,*) ''
         write(*,*) 'sudakov_radiators >> Number of errors ', sudakov_errors, ' > max errors ', max_errors, '.'
         write(*,*) 'sudakov_radiators >> or Exp[-R] ', res, ' > max(Exp[-R]) ', resmax, '.'
         write(*,*) 'Too close to the Landau pole, the evolution will not proceed.'
         write(*,*) 'Increase the IR cutoff by choosing a smaller max_rp in shower_routines.f90'
         write(*,*) 'muR = ', cs%muR
         write(*,*) 'muF = ', cs%muF
         write(*,*) 'Q = ', cs%Q
         write(*,*) 'M = ', cs%M
         write(*,*) 'lambda = ', lambda
         write(*,*) '-R = ' , log(res)
         write(*,*) 'Exp[-R] = ', res
         write(*,*) 'Lg1 = ', L*A(1)/pi/beta0*(one+half*log(one-two*lambda)/lambda)
         write(*,*) 'g2 = ', B(1)/twopi/beta0*log(one-two*lambda)&
               &-A(2)/four/pisq/beta0**2*(two*lambda/(one-two*lambda)+log(one-two*lambda))&
               & +A(1)/twopi/beta0*((two*lambda/(one-two*lambda)&
               &+log(one-two*lambda))*cs%ln_Q2_muR2+log(one-two*lambda)*(-cs%ln_Q2_M2)) &
               & +A(1)/two*(pisq*beta1)/(pi*beta0)**3*(half*log(one-two*lambda)**2&
               &+(log(one-two*lambda)+two*lambda)/(one-two*lambda))
         write(*,*) 'as/Pi*g3 = ', cs%as2pi*two&
               & * (-half*A(3)/8._dp/pisq/beta0**2*(two*lambda/(one-two*lambda))**2 &
               & -(B(2)+A(2)*(-cs%ln_Q2_M2))/four/pi/beta0*two*lambda/(one-two*lambda) &
               & +A(2)/four*(pisq*beta1)/(pi*beta0)**3*(lambda*(three*two*lambda-two)/(one-two*lambda)**2 &
               & -(one-four*lambda)/(one-two*lambda)**2*log(one-two*lambda)) &
               & +(B(1)+A(1)*(-cs%ln_Q2_M2))/two*(pisq*beta1)/pisq/beta0**2&
               &        *(two*lambda/(one-two*lambda)+log(one-two*lambda)/(one-two*lambda)) &
               & -half*A(1)/two*(two*lambda/(one-two*lambda))**2*cs%ln_Q2_muR2**2 &
               & +((B(1)+A(1)*(-cs%ln_Q2_M2))/two*two*lambda/(one-two*lambda)&
               & +A(2)/four/pi/beta0*(two*lambda/(one-two*lambda))**2 &
               & +A(1)/two*pisq*beta1/pisq/beta0**2*(two*lambda/(one-two*lambda) &
               & +(one-four*lambda)/(one-two*lambda)**2*log(one-two*lambda)))*cs%ln_Q2_muR2 &
               & +A(1)/two*((pisq*beta1)**2/two/(pi*beta0)**4*(one-four*lambda)/&
               & (one-two*lambda)**2*log(one-two*lambda)**2 &
               & +log(one-two*lambda)*((pi*beta0*pi**3*beta2-(pisq*beta1)**2)/&
               &          (pi*beta0)**4+(pisq*beta1)**2/(pi*beta0)**4/(one-two*lambda))&
               & +one/(pi*beta0)**4*lambda/(one-two*lambda)**2&
               & *(pi*beta0*pi**3*beta2*(two-three*two*lambda)+(pisq*beta1)**2*two*lambda)))
         write(*,*) '(as/Pi)^2*g4 = ', (cs%as2pi*two)**2 *part0
         stop
       endif
    end if

  end function Sudakov
  !-----------------------------------------------------------------------



  !-----------------------------------------------------------------------
  function Radp_v(L) result(res)
    real(dp), intent(in) :: L
    !----------------------------------
    real(dp):: res, lambda
    lambda=cs%alphas_muR*L*beta0
    if (lambda<half) then
       res = A(1)/pi/beta0*two*lambda/(one-two*lambda)
    else
       res = zero
    end if
  end function Radp_v
  !-----------------------------------------------------------------------



  !-----------------------------------------------------------------------
  function dRadp_v(L,res_order) result(res)
    real(dp), intent(in) :: L
    integer,  intent(in) :: res_order ! resummation order
    !----------------------------------
    real(dp):: res, lambda
    lambda=cs%alphas_muR*L*beta0

    if (lambda < half) then
       if (res_order >= order_NNLL) then
          ! derivative of the g2 function
          res =  twopi*cs%as2pi*((-two*lambda*pi*beta1*A(1)*log(one-two*lambda) + &
               &  beta0*(A(2)*lambda + beta0*(one - two*lambda)*pi*B(1) + &
               &  beta0*pi*A(1)*((one - two*lambda)*(-cs%ln_Q2_M2) + &
               &  two*lambda*(-cs%ln_Q2_muR2))))/(beta0**2*(one - two*lambda)**2*pi**2))
       end if

       if (res_order > order_NNLL) then
          ! add derivative of the g3 function
          res = res + (2*cs%as2pi**2*(B(2)*beta0**3*(-one+two*lambda)*Pi + lambda*(-(A(3)*beta0**2) &
               & + 8*A(1)*(beta1**2 - beta0*beta2)*lambda*Pi**2) -           &
               & beta0**2*cs%ln_Q2_M2*(-one+two*lambda)*Pi*(beta0*(A(2) &
               & + 2*A(1)*beta0*(cs%ln_muR2_M2 - cs%ln_Q2_M2)*Pi) - 2*A(1)*beta1*Pi*Log(one-two*lambda)) +    &
               & 2*Pi*(beta0**2*(cs%ln_muR2_M2 - cs%ln_Q2_M2) - beta1*Log(one-two*lambda))*    &
               & (B(1)*beta0**2*(-one+two*lambda)*Pi - 2*lambda*(A(2)*beta0 + A(1)*beta1*Pi)   &
               & + 2*A(1)*lambda*Pi*(beta0**2*(-cs%ln_muR2_M2 + cs%ln_Q2_M2) &
               & + beta1*Log(one-two*lambda)))))/(beta0**3*(-one+two*lambda)**3*Pi)
       end if

       if( res_order > order_NNNLL )then
          ! add derivative of the g4 function with lndlbar=0. This is the derivative of part0 in the g4 expression above
          res = res + (-2*cs%as2pi**3*(8*beta0**3*beta2*cs%ln_Q2_muR2*lambda*pi**3*A(1)&
               & - 20*beta0**4*beta1*cs%ln_Q2_muR2**2*lambda*pi**3*A(1)&
               & + 8*beta0**6*cs%ln_Q2_muR2**3*lambda*pi**3*A(1)&
               & + 16*beta0*beta1*beta2*lambda**2*pi**3*A(1)&
               & - 16*beta0**2*beta3*lambda**2*pi**3*A(1)&
               & - 32*beta0**2*beta1**2*cs%ln_Q2_muR2*lambda**2*pi**3*A(1)&
               & + 32*beta0**3*beta2*cs%ln_Q2_muR2*lambda**2*pi**3*A(1)&
               & + 16*beta1**3*lambda**3*pi**3*A(1) - 32*beta0*beta1*beta2*lambda**3*pi**3*A(1)&
               & + 16*beta0**2*beta3*lambda**3*pi**3*A(1) + 16*beta0**6*pi**3*zeta3*A(1)&
               & + 8*beta0**3*beta1*cs%ln_Q2_muR2*lambda*pi**2*A(2)&
               & - 12*beta0**5*cs%ln_Q2_muR2**2*lambda*pi**2*A(2)&
               & + 16*beta0*beta1**2*lambda**2*pi**2*A(2)&
               & - 16*beta0**2*beta2*lambda**2*pi**2*A(2) + 6*beta0**4*cs%ln_Q2_muR2*lambda*pi*A(3)&
               & - beta0**3*lambda*A(4) + 4*beta0**4*beta1*cs%ln_Q2_muR2*pi**3*B(1)&
               & - 4*beta0**6*cs%ln_Q2_muR2**2*pi**3*B(1) + 8*beta0**2*beta1**2*lambda*pi**3*B(1)&
               & - 8*beta0**3*beta2*lambda*pi**3*B(1) - 8*beta0**4*beta1*cs%ln_Q2_muR2*lambda*pi**3*B(1)&
               & + 8*beta0**6*cs%ln_Q2_muR2**2*lambda*pi**3*B(1)&
               & - 16*beta0**2*beta1**2*lambda**2*pi**3*B(1)&
               & + 16*beta0**3*beta2*lambda**2*pi**3*B(1) + 4*beta0**5*cs%ln_Q2_muR2*pi**2*B(2)&
               & - 8*beta0**5*cs%ln_Q2_muR2*lambda*pi**2*B(2) - beta0**4*pi*B(3)&
               & + 2*beta0**4*lambda*pi*B(3) + 2*beta1*pi*(4*beta0*beta2*lambda*pi**2*A(1)&
               & + 12*beta0**4*cs%ln_Q2_muR2**2*lambda*pi**2*A(1) - 16*beta1**2*lambda**2*pi**2*A(1)&
               & + 16*beta0*beta2*lambda**2*pi**2*A(1) + 4*beta0*beta1*lambda*pi*A(2)&
               & + 3*beta0**2*lambda*A(3) + 2*beta0**2*beta1*pi**2*B(1)&
               & - 4*beta0**2*beta1*lambda*pi**2*B(1)&
               & + 4*beta0**2*cs%ln_Q2_muR2*pi*(-(lambda*(5*beta1*pi*A(1) + 3*beta0*A(2)))&
               & + beta0**2*(-one+two*lambda)*pi*B(1))&
               & + 2*beta0**3*(one-two*lambda)*pi*B(2))*log(1-2*lambda)&
               & - 4*beta1**2*pi**2*(-6*beta0**2*cs%ln_Q2_muR2*lambda*pi*A(1)&
               & + lambda*(5*beta1*pi*A(1) + 3*beta0*A(2))&
               & + beta0**2*(one-two*lambda)*pi*B(1))*log(1-2*lambda)**2&
               & + 8*beta1**3*lambda*pi**3*A(1)*log(1-2*lambda)**3&
               & - beta0**2*cs%ln_Q2_M2*(-one+two*lambda)*pi*(4*beta0**4*cs%ln_Q2_muR2**2*pi**2*A(1)&
               & - 8*beta1**2*lambda*pi**2*A(1) + 8*beta0*beta2*lambda*pi**2*A(1)&
               & - 4*beta0**2*cs%ln_Q2_muR2*pi*(beta1*pi*A(1) + beta0*A(2)) + beta0**2*A(3)&
               & - 4*beta1*pi*(beta1*pi*A(1) - 2*beta0**2*cs%ln_Q2_muR2*pi*A(1)&
               & + beta0*A(2))*log(1-2*lambda)&
               & + 4*beta1**2*pi**2*A(1)*log(1-2*lambda)**2)))/((beta0 - 2*beta0*lambda)**4*pi)
       endif
    else
       res = zero
    end if
  end function dRadp_v
  !-----------------------------------------------------------------------



  !-----------------------------------------------------------------------
  function Rads_v(L,res_order) result(res)
    real(dp), intent(in) :: L
    integer, optional, intent(in) :: res_order
    !----------------------------------
    real(dp):: res, lambda
    lambda=cs%alphas_muR*L*beta0
    if (lambda<half) then
       res = 4*A(1)*cs%as2pi/((one - two*lambda)**2)
       if (present(res_order) .and. res_order > order_NNLL) then
          res = res + (cs%as2pi**2*twopi**2*(-(A(2)*(beta0 + 2*beta0*lambda)) + &
               &      2*(-2*A(1)*beta1*lambda + B(1)*beta0**2*(-one+two*lambda))*pi + &
               &      2*A(1)*pi*(-(beta0**2*cs%ln_Q2_M2*(-one+two*lambda)) + &
               &      (1 + 2*lambda)*(beta0**2*(-cs%ln_muR2_M2 + cs%ln_Q2_M2) + beta1*Log(one-two*lambda)))))/ &
               &      (beta0*(-one+two*lambda)**3*pi**2)
       end if
    else
       res = zero
    end if
  end function Rads_v
  !-----------------------------------------------------------------------

  !-----------------------------------------------------------------------
  function Radt_v(L,res_order) result(res)
    real(dp), intent(in) :: L
    integer, intent(in) :: res_order
    !----------------------------------
    real(dp):: res, lambda
    lambda=cs%alphas_muR*L*beta0
    res=zero
    if (lambda<half) then
       if (res_order > order_NNLL) res = (8*A(1)*beta0*cs%as2pi**2*twopi**2)/((one-two*lambda)**3*pi)
    else
       res = zero
    end if
  end function Radt_v
end module sudakov_radiators
