#! /usr/bin/env python
import numpy
import sys
import os
from os import system
from argparse import ArgumentParser
import numpy as np
from decimal import getcontext

"""
This simple program takes as an input a fixed order file where the differential
distribution are saved, as well as the values of the LO, NLO, NNLO and N3LO total
cross section within fiducial cuts and the name of the output file.
It prints a fixed order file in the format suitable for radish.f90.
"""


def main():

    parser = ArgumentParser()
    parser.add_argument("-i", "--input", dest="infilename",
                        help="name of leading order file", metavar="INFILE", required=True)
    parser.add_argument("-b", "--born", dest="xsec_lo", type=float,
                        help="LO cross section", metavar="LO", required=True)
    parser.add_argument("-f", "--nlo", dest="xsec_nlo", type=float,
                        help="LO cross section", metavar="NLO", required=True)
    parser.add_argument("-s", "--nnlo", dest="xsec_nnlo", type=float,
                        help="LO cross section", metavar="NNLO", required=True)
    parser.add_argument("-t", "--n3lo", dest="xsec_n3lo", type=float,
                        help="LO cross section", metavar="N3LO", required=True)
    parser.add_argument("-o", "--output", dest="outfilename",
                        help="LO cross section", metavar="OUTFILE", required=True)

    # read parser options
    args = parser.parse_args()

    # reading xsecs
    xsLO = args.xsec_lo
    xsNLO = args.xsec_nlo
    xsNNLO = args.xsec_nnlo
    xsNNNLO = args.xsec_n3lo

    # computing corrections
    dxsNLO = xsNLO - xsLO
    dxsNNLO = xsNNLO - xsNLO
    dxsNNNLO = xsNNNLO - xsNNLO

    # input file
    inputfile = args.infilename
    print('input files read: ' + inputfile)

    low, up, LO, NLO, NNLO = np.loadtxt(inputfile, unpack=True)
    nbins = len(low)

    # find overflow bin
    file = open(inputfile, 'r')
    found_overflow = False
    while True:
        line = file.readline()
        if line.startswith('#overflow:'):
            overflowLO = float(str.split(line)[1])
            overflowNLO = float(str.split(line)[2])
            overflowNNLO = float(str.split(line)[3])
            found_overflow = True
        if not line:
            break
    if (not found_overflow):
        print("WARNING: could not find overflow bin")

    # Computing cumulants

    # Bin widths
    bin_w = []
    for i in range(0, nbins):
        bin_w.append(up[i] - low[i])

    sLO = []
    sNLO = []
    sNNLO = []

    # differential distribution are multiplied by -1 in order to construct -cumulant
    for i in range(0, nbins):
        sNNLO.append((NNLO[i]-NLO[i])*(-bin_w[i]))
        sNLO.append((NLO[i]-LO[i])*(-bin_w[i]))
        sLO.append((LO[i])*-bin_w[i])
    # adding overflow
    if (found_overflow):
       sNNLO.append(overflowNLO-overflowNNLO)
       sNLO.append(overflowLO-overflowNLO)
       sLO.append(-overflowLO)
       low = np.append(low, up[nbins-1])
       nbins = nbins+1

    # constructing cumulant distributions
    sLO = np.array(sLO)
    sNLO = np.array(sNLO)
    sNNLO = np.array(sNNLO)
    cumLO = np.cumsum(sLO[::-1])[::-1]
    cumNLO = np.cumsum(sNLO[::-1])[::-1]
    cumNNLO = np.cumsum(sNNLO[::-1])[::-1]

    # print to file
    outfile = args.outfilename
    output = open(outfile, 'w')
    output.write('#--------------------'+'\n'+'# cross sections'+'\n'
                  + 'xsct_lo = '+str(xsLO)+'\n'+'delta_xsct_nlo = '+str(dxsNLO)
                  + '\n'+'delta_xsct_nnlo = '+str(dxsNNLO)
                  + '\n'+'delta_xsct_nnnlo = '+str(dxsNNNLO)+'\n')
    output.write('#----------------------------------'+'\n')
    for k in range(nbins):
        #   write cumulative distributions
        output.write(str(low[k])+' ' + str(cumLO[k])+' ' +
                      str(cumNLO[k])+' '+str(cumNNLO[k])+'\n')

    print('input fixed order file saved: ' + outfile)


if __name__ == "__main__":
    main()
